@@ -1,4 +1,4 @@
-//Axes.cpp, Copyright 2000, 2001, 2002, 2003, 2004 R.Lackner
+//Axes.cpp, Copyright 2000-2007 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -154,6 +154,10 @@ GridLine::Command(int cmd, void *tmpl, anyOutput *o)
 	case CMD_SET_DATAOBJ:
 		Id = GO_GRIDLINE;
 		return true;
+	case CMD_SCALE:
+		LineDef.patlength *= ((scaleINFO*)tmpl)->sy.fy;
+		LineDef.width *= ((scaleINFO*)tmpl)->sy.fy;
+		return true;
 	case CMD_SETSCROLL:
 	case CMD_SET_GO3D:
 	case CMD_REDRAW:
@@ -265,7 +269,7 @@ GridLine3D:: DoPlot(anyOutput *o)
 	if(!gl2) gl2 = (POINT3D*)calloc(4, sizeof(POINT3D));
 	if(!gl3) gl3 = (POINT3D*)calloc(4, sizeof(POINT3D));
 	if(!ls) ls = (line_segment**)calloc(6, sizeof(line_segment*));
-	o->ActualSize(&rDims);
+	if(!(o->ActualSize(&rDims)))return;
 	Swap(rDims.left, rDims.right);	Swap(rDims.top, rDims.bottom);
 	if(gl1 && gl2 && gl3 && ls) {
 		for(i = 0; i < 6; i++) if(ls[i]) {
@@ -501,8 +505,7 @@ Tick::SetSize(int select, double value)
 	case SIZE_AXIS_TICKS:
 		size = value;
 		break;
-	case SIZE_LB_XDIST:
-	case SIZE_LB_YDIST:
+	case SIZE_LB_XDIST:		case SIZE_LB_YDIST:
 		if(label)return label->SetSize(select, value);
 		break;
 	case SIZE_TICK_ANGLE:
@@ -547,6 +550,11 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 	AxisDEF *axis;
 
 	switch(cmd){
+	case CMD_SCALE:
+		if(label) label->Command(cmd, tmpl, o);
+		if(Grid) Grid->Command(cmd, tmpl, o);
+		size *= ((scaleINFO*)tmpl)->sy.fy;
+		break;
 	case CMD_SET_AXDEF:
 		if(axis = (AxisDEF*)tmpl) {
 			flags = (flags & AXIS_MINORTICK) | axis->flags;
@@ -641,7 +649,7 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 		LabelDef->ColTxt = parent ? parent->GetColor(COL_AXIS) : defs.Color(COL_AXIS);
 		LabelDef->ColBg = parent ? parent->GetColor(COL_BG) : defs.Color(COL_AXIS);
 		LabelDef->RotBL = LabelDef->RotCHAR = 0.0f;
-		LabelDef->fSize = parent ? parent->GetSize(SIZE_TICK_LABELS) : defs.GetSize(SIZE_TICK_LABELS);
+		LabelDef->fSize = parent ? parent->GetSize(SIZE_TICK_LABELS) : DefSize(SIZE_TICK_LABELS);
 		switch(flags & 0x70) {
 		case AXIS_USER:		LabelDef->Align = TXA_VCENTER | TXA_HCENTER;	break;
 		case AXIS_LEFT:		LabelDef->Align = TXA_VCENTER | TXA_HRIGHT;		break;
@@ -653,7 +661,7 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 		LabelDef->Style = TXS_NORMAL;
 		LabelDef->Mode = TXM_TRANSPARENT;
 		LabelDef->Font = FONT_HELVETICA;
-		LabelDef->text = tmpl && *((char*)tmpl) ? strdup((char*)tmpl) : 0L;
+		LabelDef->text = tmpl && *((char*)tmpl) ? (char*)memdup(tmpl, (int)strlen((char*)tmpl)+1, 0) : 0L;
 		label = new Label(this, 0L, fix, fiy, LabelDef, LB_X_PARENT | LB_Y_PARENT);
 		if(LabelDef->text) free(LabelDef->text);
 		delete (LabelDef);
@@ -671,6 +679,7 @@ Tick::DoPlot(double six, double csx, anyOutput *o)
 	if(!parent || parent->Id != GO_AXIS) return;
 	if(mo) DelBitmapClass(mo);		mo = 0L;
 	if(ls) delete(ls);				ls = 0L;
+	ip2.x = ip2.y = ip2.z = 0;
 	if(!((Axis*)parent)->GetValuePos(value, &fix, &fiy, &fiz, o))return;
 	lbx = fix;		lby = fiy;
 	if(flags & AXIS_ANGULAR) {
@@ -678,8 +687,8 @@ Tick::DoPlot(double six, double csx, anyOutput *o)
 		dp1.fy = o->co2fiy(parent->GetSize(SIZE_YCENT)+parent->GetSize(SIZE_GRECT_TOP));
 		dp1.fz = o->un2fix(parent->GetSize(SIZE_DRADIUS));
 		six = (fix - dp1.fx)/dp1.fz;		csx = (dp1.fy - fiy)/dp1.fz;
-		lbx += (o->un2fix(defs.GetSize(SIZE_AXIS_TICKS)*3.0*six));
-		lby -= (o->un2fiy(defs.GetSize(SIZE_AXIS_TICKS)*3.0*csx));
+		lbx += (o->un2fix(DefSize(SIZE_AXIS_TICKS)*3.0*six));
+		lby -= (o->un2fiy(DefSize(SIZE_AXIS_TICKS)*3.0*csx));
 		}
 	switch(type & 0x0f) {
 	case 1:		lsi = sin(angle/57.29577951);	lcsi = cos(angle/57.29577951);	break;
@@ -804,6 +813,7 @@ Axis::~Axis()
 	if(ssMITval) free(ssMITval);	ssMATval = ssMATlbl = ssMITval = 0L;
 	if(axisLabel) DeleteGO(axisLabel);	axisLabel = 0L;
 	if(mo) DelBitmapClass(mo);			mo = 0L;
+	if(atv) delete atv;					atv = 0L;
 }
 
 double
@@ -829,11 +839,12 @@ Axis::GetSize(int select)
 	case SIZE_YCENT:			return axis->Center.fy;
 	case SIZE_RADIUS1:	case SIZE_RADIUS2:	case SIZE_DRADIUS:
 		return axis->Radius;
+	case SIZE_BOUNDS_XMIN:		case SIZE_BOUNDS_XMAX:		case SIZE_BOUNDS_YMIN:
+	case SIZE_BOUNDS_YMAX:		case SIZE_BOUNDS_ZMIN:		case SIZE_BOUNDS_ZMAX:
+		if(parent) return parent->GetSize(select);
+		break;
 		}
-	//DEBUG: we should return a reasonable value for SIZE_BOUNDS_...
-	//    if the axis is not scaling (i.e. parent == this).
-	if(parent) return parent->GetSize(select);
-	else return defs.GetSize(select);
+	return DefSize(select);
 }
 
 DWORD
@@ -864,10 +875,8 @@ Axis::SetSize(int select, double value)
 		lbdist.fy = value;
 		if(axisLabel)axisLabel->SetSize(select,value);
 		break;
-	case SIZE_TLB_XDIST:
-	case SIZE_TLB_YDIST:
-	case SIZE_AXIS_TICKS:
-	case SIZE_TICK_ANGLE:
+	case SIZE_TLB_XDIST:		case SIZE_TLB_YDIST:
+	case SIZE_AXIS_TICKS:		case SIZE_TICK_ANGLE:
 		switch (select){
 		case SIZE_TLB_XDIST:
 			tlbdist.fx = value;			select = SIZE_LB_XDIST;
@@ -982,7 +991,7 @@ Axis::DoPlot(anyOutput *o)
 		memcpy(&tmp_axis, axis, sizeof(AxisDEF));
 		tmp_axis.loc[0].fx += dx;		tmp_axis.loc[1].fx += dx;
 		tmp_axis.loc[0].fy += dy;		tmp_axis.loc[1].fy += dy;
-		if(parent->Id == GO_PLOT3D && (axis->flags & AXIS_3D)) {
+		if(IsPlot3D(parent) && (axis->flags & AXIS_3D)) {
 			//set 3D information in scaleOut
 			cu1.fx = axis->loc[0].fx +dx;	cu1.fy = axis->loc[0].fy +dy;
 			cu1.fz = axis->loc[0].fz;		cu2.fx = axis->loc[1].fx +dx;
@@ -1152,60 +1161,40 @@ Axis::DoPlot(anyOutput *o)
 		if(Ticks[i]) Ticks[i]->DoPlot(si, csi, o);
 }
 
-void
-Axis::DoMark(anyOutput *o, bool mark)
-{
-	LineDEF mrkLine;
-	int x1, y1, x2, y2;
-	double minw = defs.GetSize(SIZE_DATA_LINE);
-
-	if(axis->flags & AXIS_ANGULAR) {
-		if(mark) {
-			memcpy(&mrc, &rDims, sizeof(RECT));
-			IncrementMinMaxRect(&mrc, 6 + o->un2ix(axline.width));
-			mo = GetRectBitmap(&mrc, o);
-			memcpy(&mrkLine, &axline, sizeof(LineDEF));
-			mrkLine.width = axline.width < minw ? minw * 3.0 : axline.width *3.0;;
-			x1 = o->co2ix(axis->Center.fx) - o->un2ix(axis->Radius);
-			y1 = o->co2iy(axis->Center.fy) - o->un2iy(axis->Radius);
-			x2 = o->co2ix(axis->Center.fx) + o->un2ix(axis->Radius);
-			y2 = o->co2iy(axis->Center.fy) + o->un2iy(axis->Radius);
-			o->SetLine(&mrkLine);
-			o->oArc(x1, y1, x2, y2, 4);
-			mrkLine.color ^= 0x00ffffffL;
-			mrkLine.width = axline.width;
-			o->SetLine(&mrkLine);
-			o->oArc(x1, y1, x2, y2, 4);
-			o->UpdateRect(&mrc, false);
-			}
-		else RestoreRectBitmap(&mo, &mrc, o);
-		return;
-		}
-	if(mark){
-		memcpy(&mrc, &rDims, sizeof(RECT));
-		IncrementMinMaxRect(&mrc, 6 + o->un2ix(axline.width));
-		mo = GetRectBitmap(&mrc, o);
-		InvertLine(pts, 2, &axline, &rDims, o, mark);
-		}
-	else RestoreRectBitmap(&mo, &mrc, o);
-}
-
 bool
 Axis::Command(int cmd, void *tmpl, anyOutput *o)
 {
 	MouseEvent *mev;
 	GraphObj **tmpPlots;
 	void *sv_ptr;
+	scaleINFO *scale;
 	int i;
 
 	switch (cmd) {
+	case CMD_SCALE:
+		if(!Ticks && (axis->flags & 0x03) != AXIS_NOTICKS)CreateTicks();
+		scale = (scaleINFO*)tmpl;
+		lbdist.fx *= scale->sx.fy;			lbdist.fy *= scale->sy.fy;
+		tlbdist.fx *= scale->sx.fy;			tlbdist.fy *= scale->sy.fy;
+		sizAxTickLabel *= scale->sy.fy;		sizAxTick *= scale->sy.fy;
+		sizAxLine *= scale->sy.fy;			brksymsize *= scale->sy.fy;
+		brkgap *= scale->sy.fy;				GridLine.patlength *= scale->sy.fy;
+		GridLine.width *= scale->sy.fy;		tlbdef.fSize *= scale->sy.fy;
+		axis->loc[0].fx *= scale->sx.fy;	axis->loc[1].fx *= scale->sx.fy;
+		axis->loc[0].fy *= scale->sy.fy;	axis->loc[1].fy *= scale->sy.fy;
+		axis->loc[0].fz *= scale->sz.fy;	axis->loc[1].fz *= scale->sz.fy;
+		axis->Center.fx *= scale->sx.fy;	axis->Center.fy *= scale->sy.fy;
+		axis->Radius *= scale->sy.fy;		tlbdef.iSize = 0;
+		if(axisLabel) axisLabel->Command(cmd, tmpl, o);
+		if(Ticks) for(i = 0; i < NumTicks; i++) if(Ticks[i]) Ticks[i]->Command(cmd, tmpl, o);
+		return true;
 	case CMD_HIDE_MARK:
 		if(!tmpl || !o) return false;
 		if(tmpl == (void*)axisLabel){
 			axisLabel->DoMark(o, false);
 			return true;
 			}
-		if(Ticks && NumTicks) for(i = 0; i < NumTicks; i++) {
+		if(Ticks) for(i = 0; i < NumTicks; i++) {
 			if(Ticks[i]) {
 				if(tmpl == (void*)Ticks[i]) {
 					Ticks[i]->DoMark(o, false);
@@ -1354,7 +1343,6 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 			}
 		if(scaleOut) delete(scaleOut);
 		scaleOut = drawOut = 0L;
-		axis->Start = axis->min;
 		return true;
 	case CMD_AUTOSCALE:		//we receive this command to update ticks after rescaling
 		if(axis && (AxisDEF*)tmpl == axis && (axis->flags & AXIS_AUTOSCALE) && 
@@ -1386,9 +1374,9 @@ Axis::SetTick(long idx, double val, DWORD flags, char *txt)
 	Ticks[idx] = new Tick(this, data, val, flags);
 	if(!txt) {
 		WriteNatFloatToBuff(TmpTxt, val);
-		l = strdup(TmpTxt+1);
+		l = (char*)memdup(TmpTxt+1, (int)strlen(TmpTxt+1)+1, 0);
 		}
-	else l = strdup(txt);
+	else l = (char*)memdup(txt, (int)strlen(txt)+1, 0);
 	if(!gl_type) {
 		}
 	if(Ticks[idx]) {
@@ -1407,10 +1395,12 @@ void
 Axis::CreateTicks()
 {
 	int i, n, nstep;
-	char *format;
+	char *format, *tick_label;
 	double fVal, tmp;
 	DWORD flags;
 
+	if(axis->min == -HUGE_VAL || axis->min == HUGE_VAL) return;
+	if(axis->max == -HUGE_VAL || axis->max == HUGE_VAL) return;
 	Command(CMD_FLUSH, 0L, 0L);
 	if((axis->flags & 0xf000) == AXIS_LOG) {	//log-axis
 		if(axis->Start > defs.min4log) tmp = log10(axis->Start);
@@ -1445,6 +1435,27 @@ Axis::CreateTicks()
 			}
 		}
 	else {										//linear axis
+		if(atv && (nstep = atv->Count()) && (Ticks = (Tick**)calloc(nstep+1, sizeof(Tick*)))) {
+			for(NumTicks = i = n = 0; NumTicks < nstep && atv->GetItem(i, &tick_label, &fVal);	i++) {
+				SetTick(NumTicks, fVal, axis->flags, tick_label);
+				NumTicks++;		n += (int)strlen(tick_label);
+				}
+			type = type;
+#ifdef _WINDOWS
+			if(type == 1 && n > 40) {
+				tlbdef.RotBL = n >100 ? 90.0 : 45.0;		tlbdef.Align = TXA_HRIGHT | TXA_VCENTER;
+				tlbdist.fy = sizAxTick;						SetSize(SIZE_TLB_YDIST, tlbdist.fy);
+				Command(CMD_TLB_TXTDEF, &tlbdef, 0L);
+				}
+#else
+			if(type == 1 && n > 30) {
+				tlbdef.RotBL = n >70 ? 90.0 : 45.0;			tlbdef.Align = TXA_HRIGHT | TXA_VCENTER;
+				tlbdist.fy = sizAxTick;						SetSize(SIZE_TLB_YDIST, tlbdist.fy);
+				Command(CMD_TLB_TXTDEF, &tlbdef, 0L);
+				}
+#endif
+			return;
+			}
 		if((axis->flags & 0xf000) == AXIS_RECI) {
 			NiceStep(axis, 8);
 			format = GetNumFormat(floor(log10(axis->Step)));
@@ -1460,7 +1471,11 @@ Axis::CreateTicks()
 		if(!Ticks) return;
 		for(NumTicks = 0; NumTicks < nstep && fVal <= axis->max;
 			NumTicks++, fVal += axis->Step) {
+#ifdef USE_WIN_SECURE
+			sprintf_s(TmpTxt, TMP_TXT_SIZE, format, fVal);
+#else
 			sprintf(TmpTxt, format, fVal);
+#endif
 			SetTick(NumTicks, fVal, axis->flags, TmpTxt);
 			}
 		}
@@ -1479,12 +1494,20 @@ Axis::ManuTicks(double sa, double st, int n, DWORD flags)
 	format = GetNumFormat(floor(log10(st)));
 	Ticks = (Tick**)calloc(m, sizeof(Tick*));
 	for(NumTicks = 0, fVal = sa; NumTicks < m && fVal <= axis->max; NumTicks++) {
+#ifdef USE_WIN_SECURE
+		sprintf_s(TmpTxt, TMP_TXT_SIZE, format, fVal);
+#else
 		sprintf(TmpTxt, format, fVal);
+#endif
 		SetTick(NumTicks, fVal, flags, TmpTxt);
 		for(j = 0; j < n; j++) {
 			mival = fVal+mist*(double)j +mist;
 			if(mival < axis->max) {
+#ifdef USE_WIN_SECURE
+				sprintf_s(TmpTxt, TMP_TXT_SIZE, format, mival);
+#else
 				sprintf(TmpTxt, format, mival);
+#endif
 				NumTicks++;
 				SetTick(NumTicks, mival, flags | AXIS_MINORTICK, TmpTxt);
 				}
@@ -1496,12 +1519,13 @@ Axis::ManuTicks(double sa, double st, int n, DWORD flags)
 bool
 Axis::GetValuePos(double val, double *fix, double *fiy, double *fiz, anyOutput *op)
 {
-	double tmp1;
+	double temp, tmp1 = 1.0;
 	int i;
 	bool bRet = true;
 	anyOutput *o;
 	fPOINT3D p1, p2;
 	lfPOINT fdp, fip;
+	AxisDEF caxis;
 
 	*fix = *fiy = *fiz = 0.0;
 	if(!op || !parent || (val < axis->min && val < axis->max) || 
@@ -1512,14 +1536,25 @@ Axis::GetValuePos(double val, double *fix, double *fiy, double *fiz, anyOutput *
 	if(axis->owner == this && scaleOut)	o = scaleOut;
 	else o = op;
 	if(axis->flags & AXIS_3D) {
-		p1.fx = GetSize(SIZE_XPOS);	p1.fy = GetSize(SIZE_YPOS);	p1.fz = GetSize(SIZE_ZPOS);
-		tmp1 = defs.cUnits == 1 ? 2.54 : defs.cUnits == 2 ? 1.0 : 25.4;
-		tmp1 /= op->VPscale;
-		switch(type) {
-		case 1: p1.fx += (tmp1 * (op->fx2fix(val)- op->Box1.Xmin)/op->hres);	break;
-		case 2: p1.fy += (tmp1 * (op->fy2fiy(val)- op->Box1.Ymax)/op->vres);	break;
-		case 3: p1.fz += (tmp1 * (op->fz2fiz(val)- op->Box1z.fx)/op->hres);	break;
-		default:	return false;
+		//Get a copy of the axis because GetAxisFac() modifies its contents
+		memcpy(&caxis, axis, sizeof(AxisDEF));
+		p1.fx = op->un2fix(axis->loc[1].fx - caxis.loc[0].fx);
+		p1.fy = op->un2fiy(axis->loc[1].fy - caxis.loc[0].fy);
+		p1.fz = op->un2fiz(axis->loc[1].fz - caxis.loc[0].fz);
+		tmp1 = sqrt(p1.fx*p1.fx + p1.fy*p1.fy + p1.fz*p1.fz);
+		p1.fx /= tmp1;		p1.fy /= tmp1;		p1.fz /= tmp1;
+		tmp1 = GetAxisFac(&caxis, tmp1, (type&0xf)-1);
+		temp = TransformValue(&caxis, val, true);
+		temp = (temp - caxis.min)*tmp1;
+		if(axis->flags & AXIS_INVERT) {
+			p1.fx = op->fix2un(op->un2fix(axis->loc[1].fx) - p1.fx*temp);
+			p1.fy = op->fiy2un(op->un2fiy(axis->loc[1].fy) - p1.fy*temp);
+			p1.fz = op->fix2un(op->un2fiz(axis->loc[1].fz) - p1.fz*temp);
+			}
+		else {
+			p1.fx = op->fix2un(p1.fx*temp+op->un2fix(axis->loc[0].fx));
+			p1.fy = op->fiy2un(p1.fy*temp+op->un2fiy(axis->loc[0].fy));
+			p1.fz = op->fix2un(p1.fz*temp+op->un2fiz(axis->loc[0].fz));
 			}
 		op->cvec2ivec(&p1, &p2);
 		*fix = p2.fx;	*fiy = p2.fy;	*fiz = p2.fz;
@@ -1546,7 +1581,7 @@ Axis::GetValuePos(double val, double *fix, double *fiy, double *fiz, anyOutput *
 		*fix = flim[1].fx + tmp1*(flim[0].fx - flim[1].fx);
 		*fiy = flim[1].fy + tmp1*(flim[0].fy - flim[1].fy);
 		}
-	if(tmp1 < -.005f || tmp1 > 1.005) return false;
+	if(tmp1 < -.005 || tmp1 > 1.005) return false;
 	return bRet;
 }
 
