@@ -1,4 +1,4 @@
-//Export.cpp, Copyright (c) 2002-2006 R.Lackner
+//Export.cpp, Copyright (c) 2002-2007 R.Lackner
 //export graph files
 //
 //    This file is part of RLPlot.
@@ -25,6 +25,9 @@
 #include <math.h>
 #include <fcntl.h>				//file open flags
 #include <sys/stat.h>			//I/O flags
+#ifdef USE_WIN_SECURE
+	#include <share.h>			//I/O flags
+#endif
 
 #ifdef _WINDOWS
 	#include <io.h>					//for read/write
@@ -105,7 +108,7 @@ ExportWMF::ExportWMF(GraphObj *g, char *FileName, float res, DWORD flags)
 	hPen = 0xffff, hBrush = 0xffff, hFont = 0xffff;
 	hres = vres = res;
 	go = g;
-	if(FileName)name = strdup(FileName);
+	if(FileName)name = (char*)memdup(FileName, (int)strlen(FileName)+1, 0);
 	else name = 0L;
 	oFile = 0;
 	rec_size = 28;
@@ -220,8 +223,13 @@ ExportWMF::StartPage()
 {
 	if(!go) return false;
 	if(name) {
+#ifdef USE_WIN_SECURE
+		if(0 !=(_sopen_s(&oFile, name, _O_RDWR | _O_BINARY | _O_CREAT | _O_TRUNC, _SH_DENYNO,
+			_S_IWRITE | _S_IREAD))) {
+#else
 		if(-1 ==(oFile = open(name, O_RDWR | O_BINARY | O_CREAT | O_TRUNC,
 			S_IWRITE | S_IREAD))) {
+#endif
 			ErrorBox("Could not open output file");
 			return false;
 			}
@@ -297,7 +305,7 @@ ExportWMF::oTextOut(int x, int y, char *txt, int cb)
 		TXA_HCENTER) ? 6 : 0) | ((TxtSet.Align & TXA_VBOTTOM) ?	8 : 0));
 	wmfSetBkMode(TxtSet.Mode ? 1 : 2);
 	wmfTextOut(x, (TxtSet.Align & TXA_VCENTER) ? y - TxtSet.iSize/2 : y, txt, 
-		(cb > 0) ? cb : strlen(txt));
+		(unsigned short)((cb > 0) ? cb : strlen(txt)));
 	return true;
 }
 
@@ -376,7 +384,7 @@ ExportWMF::wmfCreateFontIndirect(short lfHeight, short lfWidth, short lfEscapeme
 		lfItalic, lfUnderline, lfStrikeOut, lfCharSet, lfOutPrecision, lfClipPrecision,
 		lfQuality, lfPitchAndFamily, "Arial"};
 
-	if(FaceName && FaceName[0]) strcpy(lf.face, FaceName);
+	if(FaceName && FaceName[0]) rlp_strcpy(lf.face, 32, FaceName);
 	write(oFile, &lf, 56);
 	currGDIobj++;
 	maxGDIobj = currGDIobj > maxGDIobj ? currGDIobj : maxGDIobj;
@@ -527,6 +535,8 @@ void DoExportWmf(GraphObj *g, char *FileName, float res, DWORD flags)
 {
 	ExportWMF *ex;
 	
+
+	InfoBox("The export of Windos metafile (*.wmf) is deprecated.\n\nThis feature will be removed in future\nversions of RLPlot!\n\n");
 	ex = new ExportWMF(g, FileName, res, flags);
 	if(ex->StartPage()) {
 		g->DoPlot(ex);
@@ -559,18 +569,20 @@ public:
 	bool oRectangle(int x1, int y1, int x2, int y2, char *nam = 0L);
 	bool oSolidLine(POINT *p);
 	bool oTextOut(int x, int y, char *txt, int cb);
+	bool oTextOutW(int x, int y, w_char *txt, int cb);
 	bool oPolygon(POINT *pts, int cp, char * nam = 0L);
 
 private:
-	int iLineWidth;
+	int iLineWidth, cb_out;
 	bool bUseGroupLine, bOutputPending;
 	GraphObj *go;
 	char *name, indent[80], output[120], tHatchStyle[80];
 	FILE *oFile;
 	DWORD flags;
 
+	bool com_TextOut(int x, int y, char *txt, int cb);
 	void Indent(bool ind);
-	void AddToOutput(char *txt);
+	void AddToOutput(char *txt, int len);
 	char *ColName(DWORD col);
 };
 
@@ -582,12 +594,12 @@ ExportSVG::ExportSVG(GraphObj *g, char *FileName, DWORD flg)
 	dFillCol = 0xffffffffL;
 	hres = vres = 1000.0f;
 	go = g;
-	if(FileName)name = strdup(FileName);
+	if(FileName)name = (char*)memdup(FileName, (int)strlen(FileName)+1, 0);
 	else name = 0L;
 	oFile = 0L;
 	flags = flg;
-	strcpy(indent, "   ");
-	strcpy(tHatchStyle, "style=\"stroke:black; stroke-width:1\"");
+	rlp_strcpy(indent, 80, "   ");
+	rlp_strcpy(tHatchStyle, 80, "style=\"stroke:black; stroke-width:1\"");
 	bUseGroupLine = false;
 }
 
@@ -620,8 +632,13 @@ ExportSVG::SetFill(FillDEF *fill)
 		if(hgo) hgo->SetFill(fill);
 		if(fill->hatch) {
 			if(1 >(iL  = iround(un2fix(fill->hatch->width)))) iL = 1;
+#ifdef USE_WIN_SECURE
+			sprintf_s(tHatchStyle, 80, "style=\"fill:none; stroke:%s; stroke-width:%d\"",
+				ColName(fill->hatch->color), iL);
+#else
 			sprintf(tHatchStyle, "style=\"fill:none; stroke:%s; stroke-width:%d\"",
 				ColName(fill->hatch->color), iL);
+#endif
 			}
 		}
 	else {
@@ -655,7 +672,11 @@ ExportSVG::StartPage()
 	h = un2iy(go->GetSize(SIZE_GRECT_BOTTOM) - go->GetSize(SIZE_GRECT_TOP))/10;
 	w++; h++;
 	if(name) {
+#ifdef USE_WIN_SECURE
+		fopen_s(&oFile, name, "w");
+#else
 		oFile = fopen(name, "w");
+#endif
 		if(!oFile) {
 			ErrorBox("Could not open\noutput file!");
 			return false;
@@ -722,7 +743,7 @@ ExportSVG::oCircle(int x1, int y1, int x2, int y2, char* nam)
 bool
 ExportSVG::oPolyline(POINT *pts, int cp, char *nam)
 {
-	int i;
+	int i, cb;
 	char tmptxt[40];
 
 	if(cp < 2) return false;
@@ -739,23 +760,27 @@ ExportSVG::oPolyline(POINT *pts, int cp, char *nam)
 	else {
 		if(cp == 2) return oSolidLine(pts);
 		bOutputPending = false;
-		sprintf(output, "<polyline points=\""); 
+		cb_out = sprintf(output, "<polyline points=\""); 
 		for(i = 0; i < cp; i++) {
-			sprintf(tmptxt, "%d %d ", pts[i].x, pts[i].y);
-			AddToOutput(tmptxt);
+			cb = sprintf(tmptxt, "%d %d ", pts[i].x, pts[i].y);
+			AddToOutput(tmptxt, cb);
 			}
-		i = strlen(output);
-		if(i) output[i-1] = 0;
-		strcat(output, "\"");
+		if(cb_out) output[cb_out-1] = '"';
 		if(!bUseGroupLine) {
-			strcpy(tmptxt, " style = \"fill:none; ");
-			AddToOutput(tmptxt);
-			sprintf(tmptxt, "; stroke:%s; ", ColName(dLineCol));
-			AddToOutput(tmptxt);
-			sprintf(tmptxt, "stroke-width:%d\"/>",iLineWidth);
-			AddToOutput(tmptxt);
+			cb = rlp_strcpy(tmptxt, 120, " style = \"fill:none; ");
+			AddToOutput(tmptxt, cb);
+			cb = rlp_strcpy(tmptxt, 120, "; stroke:");
+			cb += rlp_strcpy(tmptxt+cb, 120-cb, ColName(dLineCol));
+			cb += rlp_strcpy(tmptxt+cb, 120-cb, "; ");
+			AddToOutput(tmptxt, cb);
+#ifdef USE_WIN_SECURE
+			cb = sprintf_s(tmptxt, 120, "stroke-width:%d\"/>",iLineWidth);
+#else
+			cb = sprintf(tmptxt, "stroke-width:%d\"/>",iLineWidth);
+#endif
+			AddToOutput(tmptxt, cb);
 			}
-		else strcat(output, "/>");
+		else AddToOutput("/>", 2);
 		fprintf(oFile, "%s%s\n", indent, output);
 		if(bOutputPending)Indent(false);
 		}
@@ -801,10 +826,10 @@ ExportSVG::oSolidLine(POINT *p)
 }
 
 bool
-ExportSVG::oTextOut(int x, int y, char *txt, int cb)
+ExportSVG::com_TextOut(int x, int y, char *txt, int cb)
 {
-	int i, j, h, ix, iy, dy;
-	char tmptxt[140], *nt;
+	int c, h, ix, iy, dy;
+	char tmptxt[120];
 
 	if(!txt || !txt[0]) return false;
 	else h = TxtSet.iSize;
@@ -822,58 +847,48 @@ ExportSVG::oTextOut(int x, int y, char *txt, int cb)
 		else if((TxtSet.Align & TXA_VBOTTOM) == TXA_VBOTTOM) dy = -un2iy(TxtSet.fSize*0.6);
 		else if((TxtSet.Align & TXA_VTOP) == TXA_VTOP) dy = -un2iy(TxtSet.fSize*0.2);
 		}
-	sprintf(output, "<text x=\"%d\" y=\"%d\" dy=\"%d\" ", ix, iy, dy);
+#ifdef USE_WIN_SECURE
+	cb_out = sprintf_s(output, 120, "<text x=\"%d\" y=\"%d\" dy=\"%d\" ", ix, iy, dy);
+#else
+	cb_out = sprintf(output, "<text x=\"%d\" y=\"%d\" dy=\"%d\" ", ix, iy, dy);
+#endif
 	if(fabs(TxtSet.RotBL) >.01 || fabs(TxtSet.RotCHAR) >.01) {
-		sprintf(tmptxt,"transform=\"rotate(%.0f,%d,%d)\" ", -TxtSet.RotBL, ix, iy);
-		strcat(output, tmptxt);
+#ifdef USE_WIN_SECURE
+		cb_out += sprintf_s(output+cb_out, 120-cb_out, "transform=\"rotate(%.0f,%d,%d)\" ", -TxtSet.RotBL, ix, iy);
+#else
+		cb_out += sprintf(output+cb_out,"transform=\"rotate(%.0f,%d,%d)\" ", -TxtSet.RotBL, ix, iy);
+#endif
 		}
-	strcpy(tmptxt, "style=\"font-family:");
+	c = rlp_strcpy(tmptxt, 140, "style=\"font-family:");
 	switch(TxtSet.Font) {
-	case FONT_TIMES:	strcat(tmptxt, "Times;");		break;
-	case FONT_COURIER:	strcat(tmptxt, "Courier;");		break;
-	default:			strcat(tmptxt, "Helvetica;");	break;
+	case FONT_TIMES:	c += rlp_strcpy(tmptxt+c, 120-c, "Times;");		break;
+	case FONT_COURIER:	c += rlp_strcpy(tmptxt+c, 120-c, "Courier;");	break;
+	default:			c += rlp_strcpy(tmptxt+c, 120-c, "Helvetica;");	break;
 		}
-	if(TxtSet.Style & TXS_ITALIC) strcat(tmptxt, " font-style:italic;");
-	if(TxtSet.Style & TXS_BOLD) strcat(tmptxt, " font-weight:bold;");
-	if(TxtSet.Style & TXS_UNDERLINE) strcat(tmptxt, " text-decoration:underline;");
-	AddToOutput(tmptxt);
-	sprintf(tmptxt, " fill:%s; stroke:%s; ", ColName(TxtSet.ColTxt),
-		ColName(TxtSet.ColTxt));
-	AddToOutput(tmptxt);
-	sprintf(tmptxt, "font-size:%d; text-anchor:%s \">", h, 
+	if(TxtSet.Style & TXS_ITALIC) c += rlp_strcpy(tmptxt+c, 120-c, " font-style:italic;");
+	if(TxtSet.Style & TXS_BOLD) c += rlp_strcpy(tmptxt+c, 120-c, " font-weight:bold;");
+	if(TxtSet.Style & TXS_UNDERLINE) c += rlp_strcpy(tmptxt+c, 120-c, " text-decoration:underline;");
+	AddToOutput(tmptxt, c);
+#ifdef USE_WIN_SECURE
+	c = sprintf_s(tmptxt, 120, " fill:%s; stroke:%s; ", ColName(TxtSet.ColTxt), ColName(TxtSet.ColTxt));
+#else
+	c = sprintf(tmptxt, " fill:%s; stroke:%s; ", ColName(TxtSet.ColTxt), ColName(TxtSet.ColTxt));
+#endif
+	AddToOutput(tmptxt, c);
+#ifdef USE_WIN_SECURE
+	c = sprintf(tmptxt, "font-size:%d; text-anchor:%s \">", h, 
 		(TxtSet.Align & TXA_HRIGHT) ? "end" : (TxtSet.Align & TXA_HCENTER) ? "middle":"start");
-	AddToOutput(tmptxt);
-	if(TxtSet.Font == FONT_GREEK) {
-		for(i = j = 0; txt[i] && j < 4090; i++) {
-			switch(txt[i]) {
-			case '"':
-				j += sprintf(TmpTxt+j, "&quot;");
-				break;
-			case '&':
-				j += sprintf(TmpTxt+j, "&amp;");
-				break;
-			case '<':
-				j += sprintf(TmpTxt+j, "&lt;");
-				break;
-			case '>':
-				j += sprintf(TmpTxt+j, "&gt;");
-				break;
-			default:
-				if(txt[i] >= 'A' && txt[i] <= 'Z') j += sprintf(TmpTxt+j, "&#%d;", txt[i] - 'A' + 0x391);
-				else if(txt[i] >= 'a' && txt[i] <= 'z') j += sprintf(TmpTxt+j, "&#%d;", txt[i] - 'a' + 0x3B1);
-				else if(txt[i] > 31) TmpTxt[j++] = txt[i];
-				break;
-				}
-			}
-		nt = TmpTxt;
-		}
-	else nt = str2xml(txt);
-	if((strlen(indent)+strlen(nt)+strlen(output)) <110) strcat(output, nt);
+#else
+	c = sprintf(tmptxt, "font-size:%d; text-anchor:%s \">", h, 
+		(TxtSet.Align & TXA_HRIGHT) ? "end" : (TxtSet.Align & TXA_HCENTER) ? "middle":"start");
+#endif
+	AddToOutput(tmptxt, c);
+	if((strlen(indent)+strlen(txt)+cb_out) <110) cb_out += rlp_strcpy(output+cb_out, 120-cb_out, txt);
 	else {
 		fprintf(oFile, "%s%s\n", indent, output);
-		strcpy(output, nt);
+		cb_out=rlp_strcpy(output, 120, txt);
 		}
-	if((strlen(indent) + strlen(output)) <104) 
+	if((strlen(indent) + cb_out) <104) 
 		fprintf(oFile, "%s%s</text>\n", indent, output);
 	else {
 		fprintf(oFile, "%s%s\n", indent, output);
@@ -882,10 +897,66 @@ ExportSVG::oTextOut(int x, int y, char *txt, int cb)
 	return true;
 }
 
+bool
+ExportSVG::oTextOut(int x, int y, char *txt, int cb)
+{
+	char *nt;
+
+	if(!txt || !txt[0]) return false;
+	nt = str2xml(txt, TxtSet.Font == FONT_GREEK);
+	return com_TextOut(x, y, nt, cb);
+}
+
+bool
+ExportSVG::oTextOutW(int x, int y, w_char *txt, int cb)
+{
+	int i, j;
+	wchar_t wc;
+	char c;
+
+	for(i = j = 0; txt[i]; i++) {
+		switch(txt[i]) {
+			case '"':
+				j += rlp_strcpy(TmpTxt+j, TMP_TXT_SIZE-j, "&quot;");
+				break;
+			case '&':
+				j += rlp_strcpy(TmpTxt+j, TMP_TXT_SIZE-j, "&amp;");
+				break;
+			case '<':
+				j += rlp_strcpy(TmpTxt+j, TMP_TXT_SIZE-j, "&lt;");
+				break;
+			case '>':
+				j += rlp_strcpy(TmpTxt+j, TMP_TXT_SIZE-j, "&gt;");
+				break;
+			default:
+				if(txt[i] > 255) {
+#ifdef USE_WIN_SECURE
+					j += sprintf_s(TmpTxt+j, TMP_TXT_SIZE-j, "&#%d;", txt[i]);
+#else
+					j += sprintf(TmpTxt+j, "&#%d;", txt[i]);
+#endif
+					}
+				else if(txt[i] > 127) {
+					c = (char)txt[i];
+					if(mbtowc(&wc, &c, 1) >0) 
+#ifdef USE_WIN_SECURE
+						j += sprintf_s(TmpTxt+j, TMP_TXT_SIZE-j, "&#%d;", ((unsigned short)wc));
+#else
+						j += sprintf(TmpTxt+j, "&#%d;", ((unsigned short)wc));
+#endif
+					}
+				else TmpTxt[j++] = (char)txt[i];
+				break;
+			}
+		}
+	TmpTxt[j++] = 0;
+	return com_TextOut(x, y, TmpTxt, j-1);
+}
+
 bool
 ExportSVG::oPolygon(POINT *pts, int cp, char *nam)
 {
-	int i;
+	int i, cb;
 	char tmptxt[40];
 
 	if(cp <3) return false;
@@ -894,21 +965,37 @@ ExportSVG::oPolygon(POINT *pts, int cp, char *nam)
 		Indent(true);
 		}
 	bOutputPending = false;
-	sprintf(output, "<polygon%s%s%s points=\"",
+#ifdef USE_WIN_SECURE
+	cb_out = sprintf_s(output, 120, "<polygon%s%s%s points=\"",
 		nam? " name=\"" : "", nam ? nam : "", nam ? "\"" : ""); 
+#else
+	cb_out = sprintf(output, "<polygon%s%s%s points=\"",
+		nam? " name=\"" : "", nam ? nam : "", nam ? "\"" : ""); 
+#endif
 	for(i = 0; i < cp; i++) {
-		sprintf(tmptxt, "%d %d ", pts[i].x, pts[i].y);
-		AddToOutput(tmptxt);
+#ifdef USE_WIN_SECURE
+		cb = sprintf_s(tmptxt, 40, "%d %d ", pts[i].x, pts[i].y);
+#else
+		cb = sprintf(tmptxt, "%d %d ", pts[i].x, pts[i].y);
+#endif
+		AddToOutput(tmptxt, cb);
 		}
-	i = strlen(output);
-	if(i) output[i-1] = 0;
-	strcat(output, "\" ");
-	sprintf(tmptxt, "style=\"fill:%s; ", ColName(dFillCol));
-	AddToOutput(tmptxt);
-	sprintf(tmptxt, "stroke:%s; ", ColName(dLineCol));
-	AddToOutput(tmptxt);
-	sprintf(tmptxt, "stroke-width:%d\"/>",iLineWidth);
-	AddToOutput(tmptxt);
+	if(cb_out) output[cb_out-1] = '"';
+#ifdef USE_WIN_SECURE
+	cb = sprintf(tmptxt, "style=\"fill:%s; ", ColName(dFillCol));
+	AddToOutput(tmptxt, cb);
+	cb = sprintf(tmptxt, "stroke:%s; ", ColName(dLineCol));
+	AddToOutput(tmptxt, cb);
+	cb = sprintf(tmptxt, "stroke-width:%d\"/>",iLineWidth);
+	AddToOutput(tmptxt, cb);
+#else
+	cb = sprintf(tmptxt, "style=\"fill:%s; ", ColName(dFillCol));
+	AddToOutput(tmptxt, cb);
+	cb = sprintf(tmptxt, "stroke:%s; ", ColName(dLineCol));
+	AddToOutput(tmptxt, cb);
+	cb = sprintf(tmptxt, "stroke-width:%d\"/>",iLineWidth);
+	AddToOutput(tmptxt, cb);
+#endif
 	if(output)fprintf(oFile, "%s%s\n", indent, output);
 	if(bOutputPending)Indent(false);
 	if(hgo) {
@@ -928,7 +1015,7 @@ ExportSVG::oPolygon(POINT *pts, int cp, char *nam)
 void
 ExportSVG::Indent(bool ind)
 {
-	int i = strlen(indent);
+	int i = (int)strlen(indent);
 
 	if(i > 20 && ind) return;
 	if(ind) strcat(indent, "   ");
@@ -936,16 +1023,21 @@ ExportSVG::Indent(bool ind)
 }
 
 void
-ExportSVG::AddToOutput(char *txt)
+ExportSVG::AddToOutput(char *txt, int len)
 {
-	if((strlen(txt) + strlen(output) + strlen(indent)) < 110)strcat(output, txt);
+	if(!txt || !txt[0]) return;
+	if(!len) len = (int)strlen(txt);
+	if((len + cb_out + strlen(indent)) < 110){
+		cb_out += rlp_strcpy(output+cb_out, 120, txt);
+		}
 	else {
 		fprintf(oFile, "%s%s\n", indent, output);
 		if(!bOutputPending) Indent(true);
 		bOutputPending = true;
-		strcpy(output, txt);
+		cb_out = rlp_strcpy(output, 120, txt);
 		}
 }
+
 char *
 ExportSVG::ColName(DWORD col)
 {
@@ -984,7 +1076,7 @@ void DoExportSvg(GraphObj *g, char *FileName, DWORD flags)
 		g->DoPlot(ex);
 		ex->EndPage();
 		}
-	HideTextCursor();
+	HideTextCursor();	
 	delete(ex);
 }
 
@@ -1065,36 +1157,34 @@ ExportEPS::SetFill(FillDEF *fill)
 {
 	if(!fill) return false;
 	if((fill->type & 0xff) != FILL_NONE) {
-		if(!hgo) hgo = new HatchOut(this);
-		if(hgo) hgo->SetFill(fill);
+		if(!hgo) hgo = new HatchOut(this);		if(hgo) hgo->SetFill(fill);
 		}
 	else {
-		if(hgo) delete hgo;
-		hgo = 0L;
+		if(hgo) delete hgo;						hgo = 0L;
 		}
-	dFillCol = fill->color;
-	dFillCol2 = fill->color2;
+	dFillCol = fill->color;						dFillCol2 = fill->color2;
 	return true;
 }
 
 bool
 ExportEPS::SetTextSpec(TextDEF *set)
 {
+	int cb;
+
 	if(set->fSize > 0.0) {
-		if((set->Style & TXS_SUPER) || (set->Style & TXS_SUB))
-			set->iSize = un2iy(set->fSize * 0.71);
+		if((set->Style & TXS_SUPER) || (set->Style & TXS_SUB)) set->iSize = un2iy(set->fSize * 0.71);
 		else set->iSize = un2iy(set->fSize);
 		}
 	if(!set->iSize) return false;
 	anyOutput::SetTextSpec(set);
 	switch(TxtSet.Font) {
-	case FONT_TIMES:	sprintf(FontName, "(Times");		break;
-	case FONT_COURIER:	sprintf(FontName, "(Courier");		break;
-	default:			sprintf(FontName, "(Helvetica");	break;		//Helvetica
+	case FONT_TIMES:	cb = rlp_strcpy(FontName, 30, "(Times");		break;		//Serif
+	case FONT_COURIER:	cb = rlp_strcpy(FontName, 30, "(Courier");		break;		//fixed spaced
+	default:			cb = rlp_strcpy(FontName, 30, "(Helvetica");	break;		//Sans Serif	
 		}
-	if(TxtSet.Style & TXS_BOLD) strcat(FontName, "-Bold");
-	if(TxtSet.Style & TXS_ITALIC) strcat(FontName, "-Italic");
-	strcat(FontName, ")");	bFontChange = true;
+	if(TxtSet.Style & TXS_BOLD) cb += rlp_strcpy(FontName+cb, 30-cb, "-Bold");
+	if(TxtSet.Style & TXS_ITALIC) cb += rlp_strcpy(FontName+cb, 30-cb, "-Italic");
+	cb += rlp_strcpy(FontName+cb, 30-cb, ")");		bFontChange = true;
 	return true;
 }
 
@@ -1313,16 +1403,16 @@ char *
 ExportEPS::col2eps(DWORD color)
 {
 	static char txt[50];
-	char tmptxt[10];
 	float r, g, b;
 
 	r = (float)(color & 0xff)/255.0f;
 	g = (float)((color>>8)&0xff)/255.0f;
 	b = (float)((color>>16)&0xff)/255.0f;
-	WriteFloatToBuff(tmptxt, r);	strcpy(txt, tmptxt+1);
-	WriteFloatToBuff(tmptxt, g);	strcat(txt, tmptxt);
-	WriteFloatToBuff(tmptxt, b);	strcat(txt, tmptxt);
-	strcat(txt, " setrgbcolor");
+#ifdef USE_WIN_SECURE
+	sprintf_s(txt, 50, "%g %g %g setrgbcolor", r, g, b);
+#else
+	sprintf(txt, "%g %g %g setrgbcolor", r, g, b);
+#endif
 	return txt;
 }
 
