@@ -1,4 +1,4 @@
-//ODbuttons.cpp, Copyright (c) 2001, 2002, 2003, 2004, 2005 R.Lackner
+//ODbuttons.cpp, Copyright (c) 2001-2007 R.Lackner
 //Property dialogs for graphic objects
 //
 //    This file is part of RLPlot.
@@ -116,7 +116,7 @@ int ExecDrawOrderButt(GraphObj *parent, GraphObj *obj, int id)
 void OD_LineStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		void *data, int id)
 {
-	LineDEF Line = {.1f, 1.0f, 0x0L, 0x0L};
+	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
 	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	POINT *pts;
 	int i, ix, iy, np;
@@ -134,11 +134,12 @@ void OD_LineStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		pts[0].y = pts[1].y = pts[4].y = rec->top;
 		pts[1].x = pts[2].x = rec->right-1;
 		pts[2].y = pts[3].y = rec->bottom-1;
-		o->oPolyline(pts, 5);
-		Line.color = 0x00000000L;
-		o->SetLine(&Line);
-		o->SetFill(&Fill);
+		o->oPolyline(pts, 5);				Line.color = 0x00000000L;
+		o->SetLine(&Line);					o->SetFill(&Fill);
 		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		if(cmd == OD_DRAWSELECTED){
+			Fill.color = 0x000000ffL;		o->SetFill(&Fill);
+			}
 		np = 0;
 		switch(id) {
 		case 201:
@@ -210,26 +211,40 @@ void OD_LineStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 				o->oSolidLine(pts);
 				pts[0].y++;		pts[1].y++;		pts[0].x = pts[1].x;	
 				}
-			o->oCircle(ix-3, rec->top + 7, ix+2, rec->top + 12);
+			o->oCircle(ix-3, rec->top + 9, ix+2, rec->top + 14);
 			o->oCircle(rec->left+11, iy-10, rec->left+16, iy -5);
 			o->oCircle(ix+3, rec->top + 27, ix+8, rec->top + 32);
 			o->oCircle(ix-5, iy+12, ix, iy + 17);
 			break;
+		case 212:
+			for(i = 2; i < (rec->bottom - rec->top - 18); i++) {
+				pts[1].x = ix + iround(pow(20.0, 1.0+((double)-i)/50.0) * -sin(((double)i)/4.0));
+				pts[1].y = iy + iround(pow(20.0, 1.0+((double)-i)/50.0) * -cos(((double)i)/4.0));
+				if(i>2)o->oSolidLine(pts);
+				pts[0].y = pts[1].y;		pts[0].x = pts[1].x;	
+				}
+			o->oCircle(ix-5, iy-4, ix, iy+1);			o->oCircle(ix-10, iy-17, ix-5, iy-12);
+#ifdef _WINDOWS
+			o->oCircle(ix-12, iy+9, ix-7, iy+4);		o->oCircle(ix+9, iy+7, ix+4, iy+2);
+#else
+			o->oCircle(ix-12, iy+9, ix-7, iy+4);		o->oCircle(ix+8, iy+6, ix+4, iy+2);
+#endif
+			break;
 			}
 		if(np) o->oPolyline(pts, np);
 		switch(id) {
 		case 201:	case 202:	case 203:	case 204:	case 205:
 		case 206:	case 207:	case 208:	case 209:
-			o->oCircle(ix-2, iy-2, ix+2, iy+2);
+			o->oCircle(ix-2, iy-2, ix+3, iy+3);
 #ifdef _WINDOWS
-			o->oCircle(rec->left+13, rec->bottom-13, rec->left+17, rec->bottom-17);
-			o->oCircle(rec->right-13, rec->top+13, rec->right-17, rec->top+17);
+			o->oCircle(rec->left+13, rec->bottom-13, rec->left+18, rec->bottom-18);
+			o->oCircle(rec->right-13, rec->top+13, rec->right-18, rec->top+18);
 #else
-			o->oCircle(rec->left+13, rec->bottom-14, rec->left+17, rec->bottom-16);
-			o->oCircle(rec->right-14, rec->top+13, rec->right-16, rec->top+17);
+			o->oCircle(rec->left+13, rec->bottom-14, rec->left+18, rec->bottom-18);
+			o->oCircle(rec->right-14, rec->top+13, rec->right-18, rec->top+18);
 #endif
 			break;
-		case 210:	case 211:
+		case 210:	case 211:	case 212:
 			break;
 			}
 		o->UpdateRect(rec, false);
@@ -245,7 +260,7 @@ void OD_LineStyleTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 void OD_ErrBarTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		void *data, int id)
 {
-	LineDEF Line = {.1f, 1.0f, 0x0L, 0x0L};
+	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
 	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	POINT pts[6];
 	int ix, iy;
@@ -309,7 +324,7 @@ void OD_ErrBarTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 void OD_WhiskerTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		void *data, int id)
 {
-	LineDEF Line = {.1f, 1.0f, 0x0L, 0x0L};
+	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
 	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	POINT pts[6];
 	int ix, iy;
@@ -551,7 +566,7 @@ void OD_AxisDesc3D(int cmd, void *par, RECT *rec, anyOutput *o,
 void OD_BreakTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		void *data, int id)
 {
-	LineDEF Line = {.1f, 1.0f, 0x0L, 0x0L};
+	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
 	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
 	POINT pts[15];
 	int i, ix, iy;
@@ -740,8 +755,30 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 				pts[0].x++;		pts[1].x++;		pts[0].y = pts[1].y;	
 				}
 			break;
-		case 520:
-		case 521:
+		case 506:
+			o->SetLine(&bLine);
+			pts[0].x = rec->left +9;	pts[0].y = rec->bottom-5;	pts[1].x = pts[0].x+1;
+			for(i = 0; i < (rec->right - rec->left - 18); i++) {
+				r = ((double)(i+rec->left-ix+7))/8.0;
+				pts[1].y = pts[0].y - iround(exp(-r*r)*1.0+5.0);
+				pts[1].y = rec->bottom - iround(errf(r)*16.0+26.0);
+				if (i) o->oSolidLine(pts);
+				pts[0].x++;		pts[1].x++;		pts[0].y = pts[1].y;	
+				}
+			o->SetLine(&rLine);
+			pts[0].x = ix-2;	pts[1].x = pts[0].x+1;
+			for(i = 0; i < (rec->right - ix-5); i++) {
+				r = ((double)(i-9))/4.0;
+				pts[1].y = rec->bottom - iround(exp(-r*r)*17.0+7.0);
+				if (i) o->oSolidLine(pts);
+				pts[0].x++;		pts[1].x++;		pts[0].y = pts[1].y;	
+				}
+			o->SetFill(&FillG);			o->SetLine(&Line);
+			o->oCircle(ix, iy, ix+5, iy+5);
+			o->oCircle(ix-5, iy+8, ix, iy+13);
+			o->oCircle(ix+5, iy-10, ix+10, iy-5);
+			break;
+		case 520:		case 521:
 			if(id == 521) {
 				pts[0].x = rec->left+13;	pts[0].y = rec->bottom-12;
 				pts[1].x = rec->left+20;	pts[1].y = rec->top+18;
@@ -1321,7 +1358,7 @@ void OD_AxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Execute axis templates for new axis as owner drawn buttons
+// 2D Plot: Execute axis templates for new axis as owner drawn buttons
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 void OD_NewAxisTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		void *data, int id)
@@ -1387,4 +1424,70 @@ void OD_NewAxisTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		break;
 		}
 }
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+// 3D Plot: Execute axis templates for new axis as owner drawn buttons
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+void OD_NewAxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o,
+		void *data, int id)
+{
+	LineDEF Line = {0.0, 1.0, 0x0L, 0x0L};
+	FillDEF Fill = {FILL_NONE, 0x00ffffffL, 1.0, 0L};
+	POINT spts[24];
+	int i, ix, iy, edge;
+	DWORD col1, col2;
 
+	switch(cmd) {
+	case OD_DRAWNORMAL:
+	case OD_DRAWSELECTED:
+		Line.color = cmd == OD_DRAWSELECTED ? 0x00000000L : 0x00e8e8e8L;
+		Fill.color = cmd == OD_DRAWSELECTED ? 0x00ffffffL : 0x00e8e8e8L;
+		col1 = cmd == OD_DRAWSELECTED ? 0x008080c0L : 0x00c8c8c8L;
+		col2 = cmd == OD_DRAWSELECTED ? 0x0000ff00L : 0x0000c000L;
+		o->SetLine(&Line);
+		ix = (rec->right + rec->left)>>1;		iy = (rec->bottom + rec->top)>>1;
+		spts[0].x = spts[3].x = spts[4].x = rec->left;
+		spts[0].y = spts[1].y = spts[4].y = rec->top;
+		spts[1].x = spts[2].x = rec->right-1;	spts[2].y = spts[3].y = rec->bottom-1;
+		o->oPolyline(spts, 5);					Line.color = 0x00000000L;
+		o->SetLine(&Line);						o->SetFill(&Fill);
+		o->oRectangle(rec->left+3, rec->top+3, rec->right-3, rec->bottom-3);
+		spts[0].x = spts[6].x = spts[8].x = spts[13].x = spts[18].x = spts[19].x = rec->left+20;		
+		spts[0].y = spts[13].y = spts[18].y = rec->bottom-14;
+		spts[1].x = spts[2].x = spts[4].x = spts[5].x = spts[7].x = spts[21].x = rec->left+10;
+		spts[1].y = spts[2].y = spts[4].y = rec->bottom-10;
+		spts[3].x = spts[15].x = spts[16].x = spts[17].x = spts[20].x = spts[22].x = rec->right-20;				
+		spts[3].y = spts[15].y = spts[16].y = spts[2].y + 3;
+		spts[5].y = spts[7].y = spts[21].y = rec->top+12;
+		spts[6].y = spts[8].y = spts[19].y = rec->top+8;
+		spts[9].x = spts[10].x = spts[11].x = spts[12].x = spts[14].x = spts[23].x = rec->right-10;
+		spts[9].y = spts[10].y = spts[23].y = spts[8].y+3;
+		spts[11].y = spts[12].y = spts[14].y = spts[0].y+3;
+		spts[17].y = spts[20].y = spts[22].y = spts[7].y+3;
+		switch(id) {
+			case 201:		edge = 4;			break;
+			case 202:		edge = 16;			break;
+			case 203:		edge = 10;			break;
+			case 204:		edge = 18;			break;
+			case 205:		edge = 2;			break;
+			case 206:		edge = 14;			break;
+			case 207:		edge = 12;			break;
+			case 208:		edge = 0;			break;
+			case 209:		edge = 20;			break;
+			case 210:		edge = 22;			break;
+			case 211:		edge = 8;			break;
+			case 212:		edge = 6;			break;
+			default:		edge = -1;			break;
+			}
+		Line.color = col1;				o->SetLine(&Line);
+		if(true)for(i = 0; i < 24; i+= 2) {
+			if(i == edge){
+				Line.color = col2;		o->SetLine(&Line);
+				o->oSolidLine(spts+i);
+				Line.color = col1;		o->SetLine(&Line);
+				}
+			else o->oSolidLine(spts+i);
+			}
+		o->UpdateRect(rec, false);
+		break;
+		}
+}
