@@ -1,4 +1,4 @@
-//Output.cpp, Copyright (c) 2000-2006 R.Lackner
+//Output.cpp, Copyright (c) 2000-2007 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -56,7 +56,7 @@ anyOutput::anyOutput()
 	RLP.finc = 1.0f;
 	RLP.fp = 0.0f;
 	dPattern = 0xffffffffL;			//impossible (invisible)line pattern to start with
-	dBgCol = defs.Color(COL_BG);
+	dBgCol = defs.Color(COL_BG);	OC_type = OC_UNKNOWN;
 	MrkMode = MRK_NONE;				MrkRect = 0L;
 	VPorg.fx = VPorg.fy = 0.0;		VPscale = 1.0;
 	MenuHeight = 0;					cCursor = MC_ARROW;
@@ -95,6 +95,44 @@ anyOutput::SetRect(fRECT rec, int u, AxisDEF *x_ax, AxisDEF *y_ax)
 	xAxis.owner = yAxis.owner = this;
 }
 
+void
+anyOutput::UseAxis(AxisDEF *ax, int type)
+{
+	AxisDEF *cax;
+
+	MrkMode = MRK_NONE;
+	if(!ax) return;
+	switch (type) {
+		case 1:			//x-axis
+			memcpy(cax = &xAxis, ax, sizeof(AxisDEF));
+			Box1.Xmin = co2fix(ax->loc[0].fx);
+			Box1.Xmax = co2fix(ax->loc[1].fx);
+			ddx = GetAxisFac(&xAxis, Box1.Xmax - Box1.Xmin, 0);
+			break;
+		case 2:			//y-axis
+			memcpy(cax = &yAxis, ax, sizeof(AxisDEF));
+			if(ax->flags & AXIS_3D) {
+				Box1.Ymax = co2fiy(ax->loc[0].fy);
+				Box1.Ymin = co2fiy(ax->loc[1].fy);
+				}
+			else {
+				Box1.Ymin = co2fiy(ax->loc[0].fy);
+				Box1.Ymax = co2fiy(ax->loc[1].fy);
+				}
+			ddy = GetAxisFac(&yAxis, Box1.Ymax - Box1.Ymin, 1);
+			break;
+		case 3:			//z-axis
+			memcpy(cax = &zAxis, ax, sizeof(AxisDEF));
+			Box1z.fx = un2fiz(ax->loc[0].fz);
+			Box1z.fy = un2fiz(ax->loc[1].fz);
+			ddz = GetAxisFac(&zAxis, Box1z.fy - Box1z.fx, 2);
+			break;
+		default:		//unnknown direction
+			return;
+		}
+	cax->owner = this;
+}
+
 void
 anyOutput::SetSpace(fPOINT3D *cub1, fPOINT3D *cub2, int u, double *rot, 
 	fPOINT3D *cent, AxisDEF *x_ax, AxisDEF *y_ax, AxisDEF *z_ax)
@@ -356,7 +394,6 @@ anyOutput::GetLine(LineDEF *lDef)
 	return false;
 }
 
-
 bool
 anyOutput::SetTextSpec(TextDEF *set)
 {
@@ -365,7 +402,6 @@ anyOutput::SetTextSpec(TextDEF *set)
 	return true;
 }
 
-
 bool
 anyOutput::ShowMark(void *src, int Mode)
 {
@@ -424,7 +460,7 @@ anyOutput::CalcCursorPos(char *txt, POINT p, POINT *fit)
 
 	d = TxtSet.iSize >>2;
 	if(!txt || !fit) return 0;
-	if (!(i = strlen(txt)))return 0;
+	if (!(i = (int)strlen(txt)))return 0;
 	//right justified text
 	if(TXA_HRIGHT == (TxtSet.Align & TXA_HRIGHT)){
 		if((p.x - fit->x) < d) return i;
@@ -455,7 +491,7 @@ anyOutput::TextCursor(char *txt, POINT p, POINT *fit, int *pos, int dx)
 	//recalculate caret position
 	if(txt && pos && !fit){
 		if(TxtSet.Align & TXA_HRIGHT) {		//right justfied text
-			if((i = strlen(txt)-(*pos))){
+			if((i = (int)strlen(txt)-(*pos))){
 				if(!oGetTextExtent(txt+(*pos), i, &w, &h)) return false;
 				w = p.x - w;
 				}
@@ -470,7 +506,7 @@ anyOutput::TextCursor(char *txt, POINT p, POINT *fit, int *pos, int dx)
 	else if(!fit)return false;
 	//right justified text: search caret and cursor position
 	else if(txt && (TxtSet.Align & TXA_HRIGHT)){
-		i = strlen(txt);
+		i = (int)strlen(txt);
 		if(i == CurrPos) w = 1;
 		else if(!oGetTextExtent(txt+CurrPos, i-CurrPos, &w, &h)) return false;
 		w = p.x - w;
@@ -638,13 +674,22 @@ anyOutput::oGetTextExtent(char *text, int cb, int *width, int *height)
 	case FONT_COURIER:		CharWidth = Cour_Char_Width;	break;
 	default:				CharWidth = Helv_Char_Width;	break;
 		}
-	if(!cb && text) cb = strlen(text);
-	for(i = w = 0; i < cb; i++) w += CharWidth[text[i]];
+	if(!cb && text) cb = (int)strlen(text);
+	for(i = w = 0; i < cb; i++) w += ((unsigned)text[i] < 256 ? CharWidth[(unsigned)text[i]] : 35);
 	*width = iround(((double)w * (double)TxtSet.iSize)/52.0);
 	*height = TxtSet.iSize;
 	return true;
 }
 
+bool
+anyOutput::oGetTextExtentW(w_char *text, int cb, int *width, int *height)
+{
+	if(cb < 1) for(cb = 0; text[cb]; cb++);
+	*width = (TxtSet.iSize * cb)>>1;
+	*height = TxtSet.iSize;
+	return true;
+}
+
 bool 
 anyOutput::oSphere(int cx, int cy, int r, POINT *pts, int cp, char *nam)
 {
@@ -676,7 +721,7 @@ anyOutput::oSphere(int cx, int cy, int r, POINT *pts, int cp, char *nam)
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 enum {HO_NONE, HO_RECT, HO_CIRCLE, HO_ELLIPSE, HO_BIGELLYPSE,
 	HO_POLYGON};
-struct {
+struct _HatchDef{
 	union {
 		struct {
 			RECT rec;
@@ -1120,8 +1165,8 @@ HatchOut::IsInside(POINT p)
 {
 	long tmp1, tmp2, tmp3, tmp4;
 
-	if(p.x < DeskRect.left || p.x > DeskRect.right || p.y < DeskRect.top
-		|| p.y >DeskRect.bottom) return false;
+	if(out->OC_type != OC_HIMETRIC && (p.x < DeskRect.left || p.x > DeskRect.right || p.y < DeskRect.top
+		|| p.y >DeskRect.bottom)) return false;
 	switch(ho){
 	case HO_RECT:
 		if(p.x > HatchDef.rec.rec.left && p.x < HatchDef.rec.rec.right &&
@@ -1768,7 +1813,7 @@ ExportTif::ExportTif(GraphObj *g, char *FileName, DWORD flags, double res, doubl
 			bmo->Erase(0x00ffffffL);
 			g->DoPlot(bmo);
 			}
-		name = strdup(FileName);
+		name = (char*)memdup(FileName, (int)strlen(FileName)+1, 0);
 		}
 	oFile = 0;
 }
@@ -1820,24 +1865,40 @@ ExportTif::StartPage()
 		};
 	DWORD res_info[4] = {iround(hres), 0x01, iround(vres), 0x01};
 	char prog_name[20];
-
+	int cb;
 
 	if(name && bmo) {
-		if(-1 ==(oFile = open(name, O_RDWR | O_BINARY | O_CREAT | O_TRUNC,
-			S_IWRITE | S_IREAD))) {
+#ifdef USE_WIN_SECURE
+		if(_sopen_s(&oFile, name, O_RDWR | O_BINARY | O_CREAT | O_TRUNC, 
+			0x40, S_IWRITE) || oFile < 0){
 			ErrorBox("Could not open output file");
 			return false;
 			}
+#else
+		if(-1 ==(oFile = open(name, O_RDWR | O_BINARY | O_CREAT | O_TRUNC,
+			S_IWRITE | S_IREAD))){
+			ErrorBox("Could not open output file");
+			return false;
+			}
+#endif
 		*((int*)(header+30))= w;		*((int*)(header+42))= h;
 		*((DWORD*)(header+126)) = (DWORD)(w * h * 3);
 		*((int*)(header+90)) = sizeof(header) + sizeof(res_info) + 20;
 		*((int*)(header+138)) = sizeof(header);
 		*((int*)(header+150)) = sizeof(header)+8;
 		*((int*)(header+186)) = sizeof(header)+16;
-		sprintf(prog_name, "RLPlot %s", SZ_VERSION);
+		cb = rlp_strcpy(prog_name, 20, "RLPlot ");
+		rlp_strcpy(prog_name+cb, 20-cb, SZ_VERSION);
+#ifdef USE_WIN_SECURE
+		_write(oFile, &header, sizeof(header));
+		_write(oFile, &res_info, sizeof(res_info));
+		_write(oFile, &prog_name, 20);
+
+#else
 		write(oFile, &header, sizeof(header));
 		write(oFile, &res_info, sizeof(res_info));
-		write(oFile, &res_info, 20);
+		write(oFile, &prog_name, 20);
+#endif
 		return true;
 		}
 	return false;
@@ -1856,15 +1917,28 @@ ExportTif::EndPage()
 				bmo->oGetPix(j, i, &pix);			pix_data[c++] = cpix[0];
 				pix_data[c++] = cpix[1];			pix_data[c++] = cpix[2];
 				if(c >= 3072) {
+#ifdef USE_WIN_SECURE
+					_write(oFile, pix_data, 3072);
+#else
 					write(oFile, pix_data, 3072);
+#endif
 					c = 0;
 					}
 				}
 			}
+#ifdef USE_WIN_SECURE
+		_write(oFile, pix_data, c);
+#else
 		write(oFile, pix_data, c);
+#endif
 		free(pix_data);
 		}
-	oFile = close(oFile);
+#ifdef USE_WIN_SECURE
+	_close(oFile);
+#else
+	close(oFile);
+#endif
+	oFile = -1;
 	return true;
 }
 
