@@ -1,4 +1,4 @@
-//QT_Spec.cpp, Copyright (c) 2001-2006 R.Lackner
+//QT_Spec.cpp, Copyright (c) 2001-2007 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -46,7 +46,7 @@ extern Default defs;
 extern UndoObj Undo;
 
 QApplication *QAppl;
-QWidget *MainWidget =0L;
+QWidget *MainWidget =0L, *CurrWidget = 0L;
 POINT CurrWidgetPos = {0,0};
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -59,14 +59,19 @@ char *SaveDataAsName(char *oldname)
 	QFileDialog qf(0,0,true);
 	QString filters("RLPlot workbook (*.rlw);;data files (*.csv);;"
 		"tab separated (*.tsv);;XML (*.xml)");
+	int i, cb;
+	char *ext;
 
 	qf.setFilters(filters);				qf.setDir(defs.currPath);
 	qf.setSelection(oldname);			qf.setMode(QFileDialog::AnyFile);
 	if(qf.exec() == QDialog::Accepted) {
-//		InfoBox((char*)qf.selectedFilter().ascii());
-//		InfoBox((char*)qf.selectedFile().ascii());
 		if(!qf.selectedFile().isEmpty()) {
-			strcpy(UserFileName, (char*)qf.selectedFile().ascii());
+			cb = rlp_strcpy(UserFileName, 600, (char*)qf.selectedFile().ascii());
+			if(cb < 4 || UserFileName[cb-4] != '.'){
+				ext = (char*)qf.selectedFilter().ascii();
+				for(i = 0; ext[i] && ext[i] != '*'; i++);
+				rlp_strcpy(UserFileName+cb, 5, ext+i+1);
+				}
 			defs.FileHistory(UserFileName);
 			return UserFileName;
 			}
@@ -78,12 +83,24 @@ char *SaveDataAsName(char *oldname)
 // Get a new file name to store graph
 char *SaveGraphAsName(char *oldname)
 {
-	QString fileName = QFileDialog::getSaveFileName(oldname?oldname:defs.currPath,
-		"RLPlot files (*.rlp)", QAppl->focusWidget());
-	if(!fileName.isEmpty()){
-		strcpy(UserFileName, fileName);
-		defs.FileHistory(UserFileName);
-		return UserFileName;
+	QFileDialog qf(0,0,true);
+	QString filters("RLPlot files (*.rlp)");
+	int i, cb;
+	char *ext;
+
+	qf.setFilters(filters);				qf.setDir(defs.currPath);
+	qf.setSelection(oldname);			qf.setMode(QFileDialog::AnyFile);
+	if(qf.exec() == QDialog::Accepted) {
+		if(!qf.selectedFile().isEmpty()) {
+			cb = rlp_strcpy(UserFileName, 600, (char*)qf.selectedFile().ascii());
+			if(cb < 4 || UserFileName[cb-4] != '.'){
+				ext = (char*)qf.selectedFilter().ascii();
+				for(i = 0; ext[i] && ext[i] != '*'; i++);
+				rlp_strcpy(UserFileName+cb, 5, ext+i+1);
+				}
+			defs.FileHistory(UserFileName);
+			return UserFileName;
+			}
 		}
 	return 0L;
 }
@@ -156,6 +173,7 @@ void OpenExportName(GraphObj *g, char *oldname)
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Common alert boxes
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+static char the_message[500];
 void InfoBox(char *Msg)
 {
 	QMessageBox::information(QAppl->focusWidget(), "Info", Msg);
@@ -168,16 +186,22 @@ void ErrorBox(char *Msg)
 
 bool YesNoBox(char *Msg)
 {
-	if(QMessageBox::information(QAppl->focusWidget(), "RLPlot", Msg,
+	int cb;
+
+	cb = rlp_strcpy(the_message, 450, Msg);
+	rlp_strcpy(the_message+cb, 500-cb, "\n");
+	if(QMessageBox::information(QAppl->focusWidget(), "RLPlot", the_message,
 		"&Yes", "&No", 0L, 0, -1)) return false;
 	return true;
 }
 
 int YesNoCancelBox(char *Msg)
 {
-	int res;
+	int res, cb;
 
-	res = QMessageBox::information(QAppl->focusWidget(), "RLPlot", Msg,
+	cb = rlp_strcpy(the_message, 450, Msg);
+	rlp_strcpy(the_message+cb, 500-cb, "\n");
+	res = QMessageBox::information(QAppl->focusWidget(), "RLPlot", the_message,
 		"&Yes", "&No", "&Cancel", 0, -1);
 	switch(res) {
 	case 0:		return 1;
@@ -197,7 +221,7 @@ void Qt_Box()
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 anyOutput *oTxtCur = 0L;
 RECT rTxtCur, rCopyMark;
-bool bTxtCur = false;
+bool bTxtCur = false,  bSuspend = false;
 DWORD coTxtCur = 0x0L;
 TxtCurBlink *cTxtCur = 0L;
 POINT ptTxtCurLine[2];
@@ -218,9 +242,8 @@ void HideTextCursorObj(anyOutput *out)
 
 void ShowTextCursor(anyOutput *out, RECT *disp, DWORD color)
 {
-	coTxtCur = color;
-	HideTextCursor();
-	oTxtCur = out;
+	coTxtCur = color;		HideTextCursor();
+	oTxtCur = out;			bSuspend = false;
 	memcpy(&rTxtCur, disp, sizeof(RECT));
 	ptTxtCurLine[0].x = rTxtCur.left;	ptTxtCurLine[0].y = rTxtCur.top;
 	ptTxtCurLine[1].x = rTxtCur.right;	ptTxtCurLine[1].y = rTxtCur.bottom;
@@ -252,21 +275,45 @@ void ShowCopyMark(anyOutput *out, RECT *mrk, int nRec)
 	int i;
 
 	if(!out || !mrk || !nRec || !cTxtCur) return;
-	cTxtCur->oCopyMark = out;
+	cTxtCur->oCopyMark = out;	bSuspend = false;
 	rCopyMark.left = mrk[0].left;	rCopyMark.right = mrk[0].right;
 	rCopyMark.top = mrk[0].top;		rCopyMark.bottom = mrk[0].bottom;
 	for(i = 1; i < nRec; i++) {
 		UpdateMinMaxRect(&rCopyMark, mrk[i].left, mrk[i].top);
 		UpdateMinMaxRect(&rCopyMark, mrk[i].right, mrk[i].bottom);
 		}
+	if(cTxtCur->bmCopyMark) delete(cTxtCur->bmCopyMark);
 	cTxtCur->bmCopyMark = new BitMapQT(rCopyMark.right - rCopyMark.left+1, 
 		rCopyMark.bottom - rCopyMark.top+1, out->hres, out->vres);
 }
 
+void InvalidateOutput(anyOutput *o)
+{
+	if(!o || !cTxtCur) return;
+	if(o == cTxtCur->oCopyMark) {
+		cTxtCur->oCopyMark = 0L;
+		if(cTxtCur->bmCopyMark) delete cTxtCur->bmCopyMark;
+		cTxtCur->bmCopyMark = 0L;
+		}
+	if(o == oTxtCur) {
+		oTxtCur = 0L;	bTxtCur = false;
+		}
+}
+
+void SuspendAnimation(anyOutput *o, bool bSusp)
+{
+	if(!o || !cTxtCur) return;
+	if(!bSusp) bSuspend = false;
+	else {
+		if(o == cTxtCur->oCopyMark) bSuspend = bSusp;
+		if(o == oTxtCur) bSuspend = bSusp;
+		}
+}
+
 LineDEF liCopyMark1 = {0.0f, 1.0f, 0x00ffffffL, 0L};
 LineDEF liCopyMark2 = {0.0f, 6.0f, 0x0L, 0xf0f0f0f0L};
 
-TxtCurBlink::TxtCurBlink():QObject(MainWidget, 0)
+TxtCurBlink::TxtCurBlink():QObject(CurrWidget, 0)
 {
 	isVis = false;
 	oCopyMark = 0L;		bmCopyMark = 0L;
@@ -310,7 +357,7 @@ TxtCurBlink::showCopyMark()
 void
 TxtCurBlink::timerEvent(QTimerEvent *ev)
 {
-	showCopyMark();
+	if(bSuspend) return;		showCopyMark();
 	if(!oTxtCur || (ptTxtCurLine[0].x == ptTxtCurLine[1].x &&
 		ptTxtCurLine[0].y == ptTxtCurLine[1].y)) return;
 	count++;
@@ -596,13 +643,13 @@ void EmptyClip()
 	if(rlpsrv) {
 		delete(rlpsrv);			rlpsrv = 0;
 		}
-	else { 
-		QSocket *socket = new QSocket(0);
-		socket->connectToHost("localhost", RLP_PORT);
-		QTextStream os(socket);
-		os << "close\n";
-		socket->close();
-		}
+//	else { 
+//		QSocket *socket = new QSocket(0);
+//		socket->connectToHost("localhost", RLP_PORT);
+//		QTextStream os(socket);
+//		os << "close\n";
+//		socket->close();
+//		}
 #endif //RLP_PORT
 	HideCopyMark();
 	QAppl->clipboard()->clear();
@@ -654,6 +701,9 @@ void GetDesktopSize(int *width, int *height)
 	QWidget *d = QApplication::desktop();
 	*width = d->width();
 	*height = d->height();
+	if(*width < 800 || *height < 600){
+		*width = 800;		*height = 600;
+		}
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -671,29 +721,50 @@ DWORD SwapRB(DWORD col)
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Common code for all QT output classes
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-bool com_TextOut(int x, int y, char *ctxt, TextDEF *TxtSet, QPainter *qP, anyOutput *o)
+bool com_QStringExt(QString txt, int *width, int *height, int cb, TextDEF *TxtSet, QPainter *qP)
+{
+	QRect rc;
+
+	if(cb >0)txt.truncate(cb);
+	rc = qP->boundingRect(0, 0, 10000, 1000, Qt::AlignLeft | Qt::AlignTop, txt);
+	*width = rc.rRight() - rc.rLeft();
+	*height = TxtSet->iSize +2;
+	return true;
+}
+
+bool
+com_GetTextExtent(char  *txt, int *width, int *height, int cb, TextDEF *TxtSet, QPainter *qP)
+{
+	if(!txt || !txt[0]) return com_QStringExt(QString("a"), width, height, 1, TxtSet, qP);
+	else return com_QStringExt(QString(txt), width, height, cb > 0 ? cb : (int)strlen(txt), TxtSet, qP);
+}
+
+bool
+com_GetTextExtentW(w_char  *txt, int *width, int *height, int cb, TextDEF *TxtSet, QPainter *qP)
+{
+	int i;
+	QString wtxt(0);
+
+	if(!txt || !txt[0]) return com_QStringExt(QString("a"), width, height, 1, TxtSet, qP);
+	for(i = 0; txt[i] && i < cb; i++) wtxt.append(QChar(txt[i]));
+	com_QStringExt(wtxt, width, height, cb, TxtSet, qP);
+}
+
+bool com_QStringOut(int x, int y, QString txt, TextDEF *TxtSet, QPainter *qP, anyOutput *o)
 {
 	int i, w, h, ix, iy;
 	QBrush oldBrush;
 	QPen oldPen, currPen;
 	QWMatrix xf, dxf;
-	QString txt(ctxt);
 
-	if(!ctxt || !ctxt[0] || !TxtSet || !qP || !o) return false;
-	if(TxtSet->Font==FONT_GREEK) {
-		txt.truncate(0);
-		for(i = 0; ctxt[i]; i++) {
-			if((ctxt[i] >= 'A' && ctxt[i] <= 'Z')) txt.append(QChar(ctxt[i] - 'A' + 0x391));
-			else if((ctxt[i] >= 'a' && ctxt[i] <= 'z')) txt.append(QChar(ctxt[i] - 'a' + 0x3B1));
-			else txt.append(QChar(ctxt[i]));
-			}
-		}
-	oldBrush = qP->brush();		dxf = qP->worldMatrix();
-	oldPen = currPen = qP->pen();
-	o->oGetTextExtent(ctxt, -1, &w, &h);
-	if(TxtSet->Align & TXA_VCENTER) iy = y + iround(TxtSet->iSize *0.4);
-	else if(TxtSet->Align & TXA_VBOTTOM) iy = y;
-	else iy = y + TxtSet->iSize + 1;
+	if(!TxtSet->iSize && TxtSet->fSize > 0.0) TxtSet->iSize = o->un2ix(TxtSet->fSize);
+	if(!TxtSet->iSize) return false;
+	if(TxtSet->Align & TXA_VCENTER) y += iround(TxtSet->iSize * 0.4);
+	else if(TxtSet->Align & TXA_VBOTTOM) y -= iround(TxtSet->iSize * 0.1);
+	else y += iround(TxtSet->iSize);
+	oldBrush = qP->brush();						dxf = qP->worldMatrix();
+	oldPen = currPen = qP->pen();				iy = y;
+	com_QStringExt(txt, &w, &h, -1, TxtSet, qP);
 	if(TxtSet->Align & TXA_HCENTER) ix = x - (w >> 1);
 	else if(TxtSet->Align & TXA_HRIGHT) ix = x - w;
 	else ix = x;
@@ -711,8 +782,8 @@ bool com_TextOut(int x, int y, char *ctxt, TextDEF *TxtSet, QPainter *qP, anyOut
 			currPen.setColor(SwapRB(TxtSet->ColTxt));
 			qP->setPen(currPen);
 			}
-		if(TxtSet->Style & TXS_SUB) iy += o->un2iy(TxtSet->fSize*0.4);
-		else if(TxtSet->Style & TXS_SUPER) iy -= o->un2iy(TxtSet->fSize*0.4);
+		if(TxtSet->Style & TXS_SUB) iy += ((TxtSet->iSize <<2)/10);
+		else if(TxtSet->Style & TXS_SUPER) iy -= ((TxtSet->iSize <<2)/10);
 		qP->drawText(ix-x, iy-y, txt, -1);
 		}
 	else {
@@ -728,31 +799,55 @@ bool com_TextOut(int x, int y, char *ctxt, TextDEF *TxtSet, QPainter *qP, anyOut
 		else if(TxtSet->Style & TXS_SUPER) iy -= o->un2iy(TxtSet->fSize*0.4);
 		qP->drawText(ix, iy, txt, -1);
 		}
+	oldPen.setCapStyle(Qt::RoundCap);
+	oldPen.setJoinStyle(Qt::RoundJoin);
 	qP->setPen(oldPen);
 	qP->setBrush(oldBrush);
 	qP->setWorldMatrix(dxf, FALSE);
 	return true;
 }
 
+bool com_TextOut(int x, int y, char *ctxt, TextDEF *TxtSet, QPainter *qP, anyOutput *o)
+{
+	int i, w, h, ix, iy;
+	QString txt(ctxt);
+
+	if(!ctxt || !ctxt[0] || !TxtSet || !qP || !o) return false;
+	if(TxtSet->Font==FONT_GREEK) {
+		txt.truncate(0);
+		for(i = 0; ctxt[i]; i++) {
+			if((ctxt[i] >= 'A' && ctxt[i] <= 'Z')) txt.append(QChar(ctxt[i] - 'A' + 0x391));
+			else if((ctxt[i] >= 'a' && ctxt[i] <= 'z')) txt.append(QChar(ctxt[i] - 'a' + 0x3B1));
+			else txt.append(QChar(ctxt[i]));
+			}
+		}
+	return com_QStringOut(x, y, txt, TxtSet, qP, o);
+}
+
+bool com_TextOutW(int x, int y, w_char *wtxt, TextDEF *TxtSet, QPainter *qP, anyOutput *o)
+{
+	int i;
+	QString txt(0);
+
+	if(!wtxt || !wtxt[0] || !TxtSet || !qP || !o) return false;
+	for(i = 0; wtxt[i]; i++) txt.append(QChar(wtxt[i]));
+	return com_QStringOut(x, y, txt, TxtSet, qP, o);
+}
+
 bool com_SetTextSpec(TextDEF *set, TextDEF *TxtSet, anyOutput *o, QFont qF, QPainter *qP)
 {
-	bool IsModified, RetVal;
+	bool RetVal;
 
 	if(!set->iSize && set->fSize > 0.0) set->iSize = o->un2iy(set->fSize);
 	if(!set->iSize) return false;
-	if(TxtSet->iSize != set->iSize || TxtSet->Style != set->Style ||
-		TxtSet->RotBL != set->RotBL || TxtSet->RotCHAR != set->RotCHAR ||
-		TxtSet->Font != set->Font || TxtSet->Mode != set->Mode)
-		IsModified = true;
-	else IsModified = false;
 	RetVal = o->anyOutput::SetTextSpec(set);
-	if(IsModified) {
+	if(true) {
 		qF.setBold((TxtSet->Style & TXS_BOLD) ? true : false);
 		qF.setItalic((TxtSet->Style & TXS_ITALIC) ? true : false);
 		qF.setUnderline((TxtSet->Style &TXS_UNDERLINE) ? true : false);
 		if((TxtSet->Style & TXS_SUPER) || (TxtSet->Style & TXS_SUB))
-			 qF.setPointSize(o->un2iy(set->fSize*0.71));
-		else qF.setPointSize((TxtSet->iSize > 8) ? TxtSet->iSize : 8);
+			qF.setPointSize((TxtSet->iSize > 1) ? (int)(TxtSet->iSize*0.71) : 1);
+		else qF.setPointSize((TxtSet->iSize > 2) ? TxtSet->iSize : 2);
 		switch(TxtSet->Font){
 		case FONT_HELVETICA:
 		default:			qF.setFamily("Helvetica");			break;
@@ -761,6 +856,7 @@ bool com_SetTextSpec(TextDEF *set, TextDEF *TxtSet, anyOutput *o, QFont qF, QPai
 		case FONT_COURIER:	qF.setFamily("Courier");			break;
 			}
 		qP->setFont(qF);
+		if(TxtSet->fSize >0.0) TxtSet->iSize = o->un2iy(TxtSet->fSize);			//correct for printer
 		}
 	return RetVal;
 }
@@ -1155,6 +1251,7 @@ BitMapQT::BitMapQT(GraphObj *g, QWidget *wi, int vr, int hr):anyOutput()
 	mempic->fill(0x00ffffffL);
 	qPainter.begin(mempic);
 	qPen.setCapStyle(Qt::RoundCap);
+	qPen.setJoinStyle(Qt::RoundJoin);
 	qPainter.setPen(qPen);
 	qFont = qPainter.font();
 }
@@ -1172,6 +1269,7 @@ BitMapQT::BitMapQT(int w, int h, double hr, double vr):anyOutput()
 	mempic->fill(0x00ffffffL);
 	qPainter.begin(mempic);
 	qPen.setCapStyle(Qt::RoundCap);
+	qPen.setJoinStyle(Qt::RoundJoin);
 	qPainter.setPen(qPen);
 	qFont = qPainter.font();
 }
@@ -1263,11 +1361,13 @@ BitMapQT::CopyBitmap(int x, int y, anyOutput* sr, int sx, int sy,
 bool
 BitMapQT::oGetTextExtent(char *text, int cb, int *width, int *height)
 {
-	if(!text) return false;
-	QRect rc = qPainter.boundingRect(0, 0, 10000, 1000, Qt::AlignLeft | Qt::AlignTop,
-		text, cb > 0 ? cb : strlen(text));
-	*width = rc.rRight() - rc.rLeft();		*height = TxtSet.iSize +2;
-	return true;
+	return com_GetTextExtent(text, width, height, cb, &TxtSet, &qPainter);
+}
+
+bool
+BitMapQT::oGetTextExtentW(w_char *text, int cb, int *width, int *height)
+{
+	return com_GetTextExtentW(text, width, height, cb, &TxtSet, &qPainter);
 }
 
 bool
@@ -1358,9 +1458,17 @@ BitMapQT::oSolidLine(POINT *p)
 bool
 BitMapQT::oTextOut(int x, int y, char *txt, int cb)
 {
+	if(!txt || !txt[0]) return false;
 	return com_TextOut(x, y, txt, &TxtSet, &qPainter, this);
 }
 
+bool
+BitMapQT::oTextOutW(int x, int y, w_char *txt, int cb)
+{
+	if(!txt || !txt[0]) return false;
+	return com_TextOutW(x, y, txt, &TxtSet, &qPainter, this);
+}
+
 bool
 BitMapQT::oPolygon(POINT *pts, int cp, char *nam)
 {
@@ -1378,47 +1486,63 @@ BitMapQT::oPolygon(POINT *pts, int cp, char *nam)
 	if(hgo) hgo->oPolygon(pts, cp);
 }
 
-bool
-BitMapQT::oArc(int x1, int y1, int x2, int y2, int quads)
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+// The menu class
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+RLPmenu::RLPmenu(QWidget *par, anyOutput *o, GraphObj *g)
+:QMenuBar(par, 0L)
 {
-	int i, j;
+	parent = par;		OutputClass = o;		BaseObj = g;
+	connect(this, SIGNAL(activated(int)), this, SLOT(doMenuItem(int)));
+}
 
-	if(x1 > x2) Swap(x1, x2);	if(y1 > y2) Swap(y1, y2);
-	switch(quads) {
-	case 1:	i = 270*16;		j = 90*16;	break;
-	case 2:	i = 180*16;		j = 180*16;	break;
-	case 3:	i = 90*16;		j = 270*16;	break;
-	case 4:	i = 0;			j = 360*16;	break;
-	default: return false;
+void
+RLPmenu::doMenuItem(int id)
+{
+	if(OutputClass) Undo.SetDisp(OutputClass);
+	if(parent && (parent->y() || parent->x())) {
+		CurrWidgetPos.x = parent->x();		CurrWidgetPos.y = parent->y();		CurrWidget = parent;
 		}
-	qPainter.drawArc(x1, y1, x2-x1, y2-y1, i, j);
-	return true;
+	if(BaseObj) switch(id) {
+	case CM_REPANOV:
+		rep_anova(BaseObj, BaseObj->data);				return;
+	case CM_REPKRUSKAL:
+		rep_kruskal(BaseObj, BaseObj->data);			return;
+	case CM_CORRELM:
+		rep_correl(BaseObj, BaseObj->data, 0);			return;
+	case CM_CORRELT:
+		rep_correl(BaseObj, BaseObj->data, 1);			return;
+	}
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // The display output class
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-OutputQT::OutputQT(GraphObj *g):BitMapQT(g, 0L)
+OutputQT::OutputQT(GraphObj *g):BitMapQT(g, QAppl->desktop())
 {
 	int w, h;
-	RLPwidget *rw;
 
 	HScroll = VScroll = 0L;
+	GetDesktopSize(&w, &h);
 	CreateNewWindow(BaseObj = g);
-	if(rw = (RLPwidget*)widget) {
-		rw->move(CurrWidgetPos.x+50, CurrWidgetPos.y+50);
-		rw->show();
-		rw->mempic = mempic;
-		rw->setBackgroundMode(QWidget::NoBackground);
+	if(widget) {
+		widget->setBackgroundMode(QWidget::NoBackground);
+		if(CurrWidgetPos.x >= ((w>>1)-100))CurrWidgetPos.x = CurrWidgetPos.y = 50;
+		widget->show();
+		widget->move(CurrWidgetPos.x+=50, CurrWidgetPos.y+=50);
+		if(widget->x() || widget->y()) {
+			CurrWidgetPos.x = widget->x();	CurrWidgetPos.y = widget->y();
+			}
 		}
 }
 
 OutputQT::OutputQT(DlgWidget *wi):BitMapQT(0L, wi)
 {
 	//assume fixed size (dialog) widget
-	widget = wi;
-	HScroll = VScroll = 0L;
-	BaseObj = 0L;
+	if(widget = wi) {
+		wi->move(CurrWidgetPos.x+50, CurrWidgetPos.y+50);
+		}
+	HScroll = VScroll = 0L;			BaseObj = 0L;
 	wi->OutputClass = this;
 	wi->mempic = mempic;
 	wi->setBackgroundMode(QWidget::NoBackground);
@@ -1443,7 +1567,7 @@ OutputQT::ActualSize(RECT *rc)
 		rc->left = rc->top = 0;
 		rc->bottom = widget->height() - MenuHeight-6;
 		rc->right = widget->width();
-		return true;
+		return (rc->right > 40 && rc->bottom > 40);
 		}
 	return false;
 }
@@ -1568,24 +1692,29 @@ OutputQT::MouseCursor(int cid, bool force)
 	QBitmap *bits, *mask;
 	bits = mask = 0L;
 
+	CurrWidget = widget;
 	switch(cid) {
 #ifdef Q_CHECK_PTR				//Qt version 3
 	case MC_ARROW:	widget->setCursor(QCursor(Qt::ArrowCursor));	break;
+	case MC_TXTFRM:
 	case MC_CROSS:	widget->setCursor(QCursor(Qt::CrossCursor));	break;
 	case MC_WAIT:	widget->setCursor(QCursor(Qt::WaitCursor));		break;
 	case MC_TEXT:	widget->setCursor(QCursor(Qt::IbeamCursor));	break;
 	case MC_NORTH:	widget->setCursor(QCursor(Qt::SizeVerCursor));	break;
 	case MC_NE:		widget->setCursor(QCursor(Qt::SizeBDiagCursor));break;
+	case MC_COLWIDTH:
 	case MC_EAST:	widget->setCursor(QCursor(Qt::SizeHorCursor));	break;
 	case MC_SE:		widget->setCursor(QCursor(Qt::SizeFDiagCursor));break;
 	case MC_SALL:	widget->setCursor(QCursor(Qt::SizeAllCursor));	break;
 #else							//Qt version 2
 	case MC_ARROW:	widget->setCursor(QCursor(ArrowCursor));	break;
+	case MC_TXTFRM:
 	case MC_CROSS:	widget->setCursor(QCursor(CrossCursor));	break;
 	case MC_WAIT:	widget->setCursor(QCursor(WaitCursor));		break;
 	case MC_TEXT:	widget->setCursor(QCursor(IbeamCursor));	break;
 	case MC_NORTH:	widget->setCursor(QCursor(SizeVerCursor));	break;
 	case MC_NE:		widget->setCursor(QCursor(SizeBDiagCursor));break;
+	case MC_COLWIDTH:
 	case MC_EAST:	widget->setCursor(QCursor(SizeHorCursor));	break;
 	case MC_SE:		widget->setCursor(QCursor(SizeFDiagCursor));break;
 	case MC_SALL:	widget->setCursor(QCursor(SizeAllCursor));	break;
@@ -1636,6 +1765,9 @@ OutputQT::SetScroll(bool isVert, int iMin, int iMax, int iPSize, int iPos)
 {
 	QScrollBar *sb;
 
+	if((CurrWidget = widget) && (widget->x() || widget->y())) {
+		CurrWidgetPos.x = widget->x();		CurrWidgetPos.y = widget->y();
+		}
 	if(isVert) {
 		if(!(sb = VScroll))return false;
 		}
@@ -1703,6 +1835,8 @@ OutputQT::ShowLine(POINT * pts, int cp, DWORD color)
 	qp.setColor(SwapRB(color));
 	qp.setWidth(1);
 	qp.setStyle(Qt::SolidLine);
+	qp.setCapStyle(Qt::RoundCap);
+	qp.setJoinStyle(Qt::RoundJoin);
 	paint.setPen(qp);
 	paint.moveTo(pts[0].x, pts[0].y);
 	for (i = 1; i < cp; i++) paint.lineTo(pts[i].x, pts[i].y);
@@ -1719,6 +1853,8 @@ OutputQT::ShowEllipse(POINT p1, POINT p2, DWORD color)
 	qp.setColor(SwapRB(color));
 	qp.setWidth(1);
 	qp.setStyle(Qt::SolidLine);
+	qp.setCapStyle(Qt::RoundCap);
+	qp.setJoinStyle(Qt::RoundJoin);
 	paint.setPen(qp);
 	paint.drawArc(p1.x, p1.y, p2.x-p1.x, p2.y-p1.y, 0, 5760);
 	paint.flush();
@@ -1772,14 +1908,26 @@ OutputQT::SetMenu(int type)
 		graph->insertSeparator();
 		graph->insertItem("&Settings", widget, SLOT(cmDefaults()));
 
+		QPopupMenu *anov = new QPopupMenu(widget);
+		anov->insertItem("&One Way Anova", CM_REPANOV);
+		anov->insertItem("&Kruskal Wallis", CM_REPKRUSKAL);
+
+		QPopupMenu *corr = new QPopupMenu(widget);
+		corr->insertItem("Correlation &Matrix", CM_CORRELM);
+		corr->insertItem("Tiled &Plots", CM_CORRELT);
+
 		QPopupMenu *stats = new QPopupMenu(widget);
-		stats->insertItem("&Anova", widget, SLOT(cmRepanov()));
+		stats->insertItem("&Sample Stats", widget, SLOT(cmSmplStat()));
+		stats->insertItem("&Comp. Means", widget, SLOT(cmRepCmeans()));
+		stats->insertItem("&Anova", anov);
 		stats->insertItem("&Regression", widget, SLOT(cmRepregr()));
+		stats->insertItem("C&orrelations", corr);
+		stats->insertItem("&2x2 Table", widget, SLOT(cmReptwoway()));
 
 		QPopupMenu *about = new QPopupMenu(widget);
 		about->insertItem("&About ...", widget, SLOT(cmAbout()));
 
-		menu = new QMenuBar(widget);
+		menu = new RLPmenu(widget, this, BaseObj);
 		menu->insertItem("&File", file);			menu->insertItem("&Edit", edit);
 		menu->insertItem("&Statistics", stats);		menu->insertItem("&Graph", graph);
 		menu->insertItem("&?", about);
@@ -1851,7 +1999,7 @@ OutputQT::SetMenu(int type)
 		QPopupMenu *about = new QPopupMenu(widget);
 		about->insertItem("&About ...", widget, SLOT(cmAbout()));
 
-		menu = new QMenuBar(widget);
+		menu = new RLPmenu(widget, this, BaseObj);
 		menu->insertItem("&File", file);
 		menu->insertItem("&Edit", edit);
 		menu->insertItem("&Display", displ);
@@ -1921,7 +2069,7 @@ OutputQT::SetMenu(int type)
 		QPopupMenu *about = new QPopupMenu(widget);
 		about->insertItem("&About ...", widget, SLOT(cmAbout()));
 
-		menu = new QMenuBar(widget);
+		menu = new RLPmenu(widget, this, BaseObj);
 		menu->insertItem("&File", file);
 		menu->insertItem("&Edit", edit);
 		menu->insertItem("&Display", displ);
@@ -1982,6 +2130,7 @@ OutputQT::CreateNewWindow(GraphObj *g)
 	if(widget = new RLPwidget(0, 0, this, g)) {
 		widget->setCaption("OutputQT::CreateNewWindow");
 		widget->setGeometry(0, 0, (int)(w*.7f), (int)(h*.7f));
+		((RLPwidget*)widget)->mempic = mempic;
 		HScroll = ((RLPwidget*)widget)->HScroll;
 		VScroll = ((RLPwidget*)widget)->VScroll;
 		}
@@ -2013,6 +2162,7 @@ RLPwidget::RLPwidget(QWidget *par, const char *name, anyOutput *o, GraphObj *g)
 	if(!MainWidget) QAppl->setMainWidget(MainWidget = this);
 	setMouseTracking(true);
 	setFocusPolicy(StrongFocus);
+	setKeyCompression(true);
 }
 
 RLPwidget::~RLPwidget()
@@ -2147,7 +2297,11 @@ RLPwidget::cmPaste()
 {
 	if(BaseObj) {
 		OutputClass->MouseCursor(MC_WAIT, true);
-		TestClipboard(BaseObj);
+		if(BaseObj->Id == GO_SPREADDATA) TestClipboard(BaseObj);
+		else if(BaseObj->Id == GO_PAGE || BaseObj->Id == GO_GRAPH){
+			if(CurrGO && CurrGO->Id == GO_TEXTFRAME && CurrGO->Command(CMD_PASTE, 0L, OutputClass));
+			else TestClipboard(BaseObj);
+			}
 		BaseObj->Command(CMD_MOUSECURSOR, 0L, OutputClass);
 		}
 }
@@ -2155,6 +2309,8 @@ RLPwidget::cmPaste()
 void
 RLPwidget::cmCopyGraph()
 {
+	EmptyClip();
+	if(CurrGO && CurrGO->Id == GO_TEXTFRAME) if(CurrGO->Command(CMD_COPY, 0L, OutputClass))return;
 	if(BaseObj) CopyGraph(BaseObj);
 }
 
@@ -2375,9 +2531,15 @@ RLPwidget::cmtText()
 }
 
 void
-RLPwidget::cmRepanov()
+RLPwidget::cmSmplStat()
 {
-	if(BaseObj) rep_anova(BaseObj, BaseObj->data);
+	if(BaseObj) rep_samplestats(BaseObj, BaseObj->data);
+}
+
+void
+RLPwidget::cmRepCmeans()
+{
+	if(BaseObj) rep_compmeans(BaseObj, BaseObj->data);
 }
 
 void
@@ -2386,6 +2548,12 @@ RLPwidget::cmRepregr()
 	if(BaseObj) rep_regression(BaseObj, BaseObj->data);
 }
 
+void
+RLPwidget::cmReptwoway()
+{
+	if(BaseObj) rep_twowaytable(BaseObj, BaseObj->data);
+}
+
 //protected: widget events
 void
 RLPwidget::paintEvent(QPaintEvent *range)
@@ -2400,6 +2568,7 @@ RLPwidget::paintEvent(QPaintEvent *range)
 void
 RLPwidget::resizeEvent(QResizeEvent *)
 {
+	CurrWidget = this;
 	HScroll->resize(width() -16, 16);
 	HScroll->move(0, height()-16);
 	VScroll->resize(16, height()-OutputClass->MenuHeight-16);
@@ -2434,8 +2603,8 @@ RLPwidget::mousePressEvent(QMouseEvent *e)
 	int i;
 	MouseEvent mev = {1, e->button() == Qt::LeftButton ? MOUSE_LBDOWN : -1, e->x(), e->y()};
 
-	HideTextCursor();
-	i = e->state();
+	HideTextCursor();			i = e->state();
+	CurrWidget = this;
 	if(i & Qt::ShiftButton) mev.StateFlags |= 0x08;
 	if(i & Qt::ControlButton) mev.StateFlags |= 0x10;
 	if (BaseObj)BaseObj->Command(CMD_MOUSE_EVENT, (void *)&mev, OutputClass);
@@ -2471,6 +2640,8 @@ void
 RLPwidget::keyPressEvent(QKeyEvent *e)
 {
 	int i, c;
+	QChar qc;
+	w_char uc;
 
 	if(BaseObj) switch(c = e->key()) {
 		case Key_Prior:
@@ -2515,12 +2686,15 @@ RLPwidget::keyPressEvent(QKeyEvent *e)
 			BaseObj->Command(CMD_POS_LAST, 0L, OutputClass);
 			break;
 		default:
-			c = e->ascii();
-			if(c == 3) cmCopy();
-			else if(c == 22) cmPaste();
-			else if(c == 26) cmUndo();
-			else if(c >1 && c < 256)
-				BaseObj->Command(CMD_ADDCHAR, (void *)(& c), OutputClass);
+			QString kres = e->text();
+			for(i = 0; i < kres.length(); i++) {
+				qc = kres.at(i);	uc = qc.unicode();
+				if(uc == 3) cmCopy();
+				else if(uc == 22) cmPaste();
+				else if(uc == 26) cmUndo();
+				else if(uc > 255) BaseObj->Command(CMD_ADDCHARW, (void *)(&uc), OutputClass);
+				else BaseObj->Command(CMD_ADDCHAR, (void *)(&uc), OutputClass);
+				}
 			break;
 		}
 	e->accept();
@@ -2529,7 +2703,10 @@ RLPwidget::keyPressEvent(QKeyEvent *e)
 void
 RLPwidget::focusInEvent(QFocusEvent *e)
 {
-	CurrWidgetPos.x = x();			CurrWidgetPos.y = y();
+	if(x() || y()) {
+		CurrWidgetPos.x = x();			CurrWidgetPos.y = y();
+		}
+	CurrWidget = this;
 	if(BaseObj) {
 		if(BaseObj->Id == GO_GRAPH) CurrGraph = (Graph*)BaseObj;
 		}
@@ -2556,7 +2733,6 @@ RLPwidget::openHistoryFile(int idx)
 		}
 }
 
-
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Print and output EPS to file
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -2598,20 +2774,20 @@ FileEPS::metric(int m) const
 
 PrintQT::PrintQT(GraphObj *g, char *file)
 {
-	hres = vres = 720.0;
 	units = defs.cUnits;
-	Box1.Xmin = Box1.Ymin = 0.0;
-	Box1.Xmax = Box1.Ymax = 6000;
-	DeskRect.left = DeskRect.top = 0;
-	DeskRect.right = (long)(hres*6.0);
-	DeskRect.bottom = (long)(vres*8.0);
 	dxf.setMatrix(0.1, 0.0, 0.0, 0.1, 0.0, 0.0);
 	hgo = 0L;
 	if(file) fileName = strdup(file);
 	else fileName = 0L;
 	go = g;
 	if(fileName && go) printer = new FileEPS(g, this);
-	else printer = new QPrinter;
+	else printer = new QPrinter(QPrinter::HighResolution);
+	hres = vres = (9.5*((double)printer->resolution()));
+	Box1.Xmin = Box1.Ymin = 0.0;
+	Box1.Xmax = Box1.Ymax = 6000;
+	DeskRect.left = DeskRect.top = 0;
+	DeskRect.right = (long)(hres*6.0);
+	DeskRect.bottom = (long)(vres*8.0);
 	bPrinting = false;
 }
 
@@ -2679,6 +2855,7 @@ PrintQT::SetFill(FillDEF *fill)
 bool
 PrintQT::SetTextSpec(TextDEF *set)
 {
+	set->iSize = un2ix(set->fSize/7.5);
 	return com_SetTextSpec(set, &TxtSet, this, qFont, &qPainter);
 }
 
@@ -2695,7 +2872,6 @@ PrintQT::StartPage()
 		qPainter.setWorldMatrix(dxf, FALSE);
 		return bPrinting = true;
 		}
-
 	if(printer->setup(0)){
 		qPainter.begin(printer);
 		qPainter.setWorldMatrix(dxf, FALSE);
@@ -2723,12 +2899,28 @@ PrintQT::Eject()
 bool
 PrintQT::oGetTextExtent(char *text, int cb, int *width, int *height)
 {
-	QRect rc = qPainter.boundingRect(0, 0, 10000, 1000, Qt::AlignLeft | Qt::AlignTop,
+	return com_GetTextExtent(text, width, height, cb, &TxtSet, &qPainter);
+
+
+	if(!text || !text[0]){
+		QRect rc = qPainter.boundingRect(0, 0, 10000, 1000, Qt::AlignLeft | Qt::AlignTop, "A", 1);
+		*width = rc.rLeft();
+		}
+	else {
+		QRect rc = qPainter.boundingRect(0, 0, 10000, 1000, Qt::AlignLeft | Qt::AlignTop,
 		text, cb > 0 ? cb : strlen(text));
-	*width = rc.rRight() - rc.rLeft();		*height = rc.rBottom() - rc.rTop();
+		*width = rc.rRight() - rc.rLeft();
+		}
+	*height = TxtSet.iSize +2;
 	return true;
 }
 
+bool
+PrintQT::oGetTextExtentW(w_char *text, int cb, int *width, int *height)
+{
+	return com_GetTextExtentW(text, width, height, cb, &TxtSet, &qPainter);
+}
+
 bool
 PrintQT::oCircle(int x1, int y1, int x2, int y2, char* nam)
 {
@@ -2771,9 +2963,17 @@ PrintQT::oSolidLine(POINT *p)
 bool
 PrintQT::oTextOut(int x, int y, char *txt, int cb)
 {
+	if(!txt || !txt[0]) return false;
 	return com_TextOut(x, y, txt, &TxtSet, &qPainter, this);
 }
 
+bool
+PrintQT::oTextOutW(int x, int y, w_char *txt, int cb)
+{
+	if(!txt || !txt[0]) return false;
+	return com_TextOutW(x, y, txt, &TxtSet, &qPainter, this);
+}
+
 bool
 PrintQT::oPolygon(POINT *pts, int cp, char *nam)
 {
@@ -2792,12 +2992,6 @@ PrintQT::oPolygon(POINT *pts, int cp, char *nam)
 	return true;
 }
 
-bool
-PrintQT::oArc(int x1, int y1, int x2, int y2, int quads)
-{
-	return false;
-}
-
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Find a suitable www browser
 void FindBrowser()
@@ -2881,7 +3075,10 @@ DlgWidget::mousePressEvent(QMouseEvent *e)
 {
 	MouseEvent mev = {1, e->button() == Qt::LeftButton ? MOUSE_LBDOWN : -1, e->x(), e->y()};
 
-	HideTextCursor();
+	HideTextCursor();				CurrWidget = this;
+	if(x() || y()) {
+		CurrWidgetPos.x = x();		CurrWidgetPos.y = y();
+		}
 	if (dlg) dlg->Command(CMD_MOUSE_EVENT, (void *)&mev, OutputClass);
 }
 
@@ -2906,6 +3103,10 @@ DlgWidget::keyPressEvent(QKeyEvent *e)
 {
 	int c;
 
+	CurrWidget = this;
+	if(x() || y()) {
+		CurrWidgetPos.x = x();			CurrWidgetPos.y = y();
+		}
 	if(dlg) switch(c = e->key()) {
 		case Key_Left:
 			if(e->state() & ShiftButton) dlg->Command(CMD_SHIFTLEFT, 0L, OutputClass);
@@ -2953,7 +3154,10 @@ DlgWidget::keyPressEvent(QKeyEvent *e)
 void
 DlgWidget::focusInEvent(QFocusEvent *e)
 {
-	CurrWidgetPos.x = x();		CurrWidgetPos.y = y();
+	CurrWidget = this;
+	if(x() || y()) {
+		CurrWidgetPos.x = x();		CurrWidgetPos.y = y();
+		}
 }
 
 void
@@ -2984,26 +3188,30 @@ DlgWidget::timerEvent(QTimerEvent *)
 void *CreateDlgWnd(char *title, int x, int y, int width, int height, tag_DlgObj *d, DWORD flags)
 {
 	DlgWidget *w;
+	QWidget *pw;
 	OutputQT *o;
 	int dw, dh;
 
-	w = new DlgWidget(QAppl->focusWidget(), 0, d);
+	w = new DlgWidget(pw = QAppl->activeWindow(), 0, d);
 	w->setCaption(title);
 	if(flags & 0x2) w->setFixedSize(width, height);
 	else w->setFixedSize(width-6, height-16);
+	o = new OutputQT(w);	o->units = defs.cUnits;
+	o->Erase(0x00e0e0e0L);	if(flags & 0x04) w->startTimer(100);
 	if(flags & 0x1) {
 		GetDesktopSize(&dw, &dh);
 		w->move((dw>>1) - ((w->width())>>1), (dh>>1) - ((w->height())>>1));
 		}
-	else w->move(CurrWidgetPos.x+x, CurrWidgetPos.y+y);
-	o = new OutputQT(w);
-	o->units = defs.cUnits;
-	o->Erase(0x00e0e0e0L);
-	if(flags & 0x04) {
-		w->startTimer(100);
+	else if(pw) {
+		if(pw->x() || pw->y()) {
+			x += pw->x();			y += pw->y();
+			}
+		else {
+			x += CurrWidgetPos.x;	y += CurrWidgetPos.y;
+			}
+		w->move(x, y);
 		}
-	d->DoPlot(o);
-	w->show();
+	d->DoPlot(o);			w->show();
 	return w;
 }
 
@@ -3017,6 +3225,9 @@ void CloseDlgWnd(void *hDlg)
 	HideCopyMark();
 	if(hDlg) {
 		delete((DlgWidget*) hDlg);
+		if(CurrWidgetPos.x > 50 && CurrWidgetPos.y > 50) {
+			CurrWidgetPos.x -= 50;		CurrWidgetPos.y -= 50;
+			}
 		}
 }
 
@@ -3029,6 +3240,12 @@ void ShowDlgWnd(void *hDlg)
 		}
 }
 
+void ResizeDlgWnd(void *hDlg, int w, int h)
+{
+	((DlgWidget*)hDlg)->setFixedSize(w-6, h-16);
+	((DlgWidget*)hDlg)->show();
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // OS independent interface to Qt specific classes
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
