@@ -1,4 +1,4 @@
-//QT_Spec.h, Copyright (c) 2001, 2002, 2003, 2004 R.Lackner
+//QT_Spec.h, Copyright (c) 2001-2007 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -109,6 +109,22 @@ private:
 
 #endif //RLP_PORT
 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+class RLPmenu:public QMenuBar {
+	Q_OBJECT
+public:
+	RLPmenu(QWidget *par, anyOutput *o, GraphObj *g);
+
+public slots:
+	void doMenuItem(int id);
+
+private:
+	anyOutput *OutputClass;
+	QWidget *parent;
+	GraphObj *BaseObj;
+
+};
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // The Qt widget class implementet for RLPlot
 class RLPwidget:public QWidget {
@@ -172,8 +188,10 @@ public slots:
 	void cmtEllipse();
 	void cmtArrow();
 	void cmtText();
-	void cmRepanov();
+	void cmSmplStat();
+	void cmRepCmeans();
 	void cmRepregr();
+	void cmReptwoway();
 	void cmFile1() {openHistoryFile(0);};
 	void cmFile2() {openHistoryFile(1);};
 	void cmFile3() {openHistoryFile(2);};
@@ -249,6 +267,7 @@ public:
 	bool CopyBitmap(int x, int y, anyOutput* src, int sx, int sy,
 		int sw, int sh, bool invert);
 	bool oGetTextExtent(char *text, int cb, int *width, int *height);
+	bool oGetTextExtentW(w_char *text, int cb, int *width, int *height);
 	bool oGetPix(int x, int y, DWORD *col);
 	bool oDrawIcon(int type, int x, int y);
 	bool oCircle(int x1, int y1, int x2, int y2, char* nam = 0L);
@@ -256,8 +275,8 @@ public:
 	bool oRectangle(int x1, int y1, int x2, int y2, char *nam = 0L);
 	bool oSolidLine(POINT *p);
 	bool oTextOut(int x, int y, char *txt, int cb);
+	bool oTextOutW(int x, int y, w_char *txt, int cb);
 	bool oPolygon(POINT *pts, int cp, char *nam = 0L);
-	bool oArc(int x1, int y1, int x2, int y2, int quads);
 };
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -285,7 +304,7 @@ public:
 
 private:
 	GraphObj *BaseObj;
-	QMenuBar *menu;
+	RLPmenu *menu;
 };
 
 class PrintQT:public anyOutput{
@@ -303,13 +322,14 @@ public:
 	bool EndPage();
 	bool Eject();
 	bool oGetTextExtent(char *text, int cb, int *width, int *height);
+	bool oGetTextExtentW(w_char *text, int cb, int *width, int *height);
 	bool oCircle(int x1, int y1, int x2, int y2, char* nam = 0L);
 	bool oPolyline(POINT * pts, int cp, char *nam = 0L);
 	bool oRectangle(int x1, int y1, int x2, int y2, char *nam = 0L);
 	bool oSolidLine(POINT *p);
 	bool oTextOut(int x, int y, char *txt, int cb);
+	bool oTextOutW(int x, int y, w_char *txt, int cb);
 	bool oPolygon(POINT *pts, int cp, char *nam = 0L);
-	bool oArc(int x1, int y1, int x2, int y2, int quads);
 
 private:
 	QPen qPen;
