@@ -1,4 +1,4 @@
-//TheDialog.h, Copyright (c) 2001-2006 R.Lackner
+//TheDialog.h, Copyright (c) 2001-2007 R.Lackner
 //Definitions for TheDialog.cpp
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
@@ -37,7 +37,7 @@ typedef struct {
 } TabSHEET;
 
 //types of dialogs
-enum {NONE, PUSHBUTTON, ARROWBUTT, COLBUTTON, COLBUTT, FILLBUTTON, SHADE3D, LINEBUTT, SYMBUTT,
+enum {NONE, PUSHBUTTON, ARROWBUTT, COLBUTT, FILLBUTTON, SHADE3D, LINEBUTT, SYMBUTT,
 	FILLRADIO, SYMRADIO, CHECKBOX, RADIO0, RADIO1, RADIO2, LTEXT, RTEXT, CTEXT, EDTEXT, 
 	RANGEINPUT, EDVAL1, INCDECVAL1, HSCROLL, VSCROLL, TXTHSP, ICON, GROUP, 
 	GROUPBOX, SHEET, ODBUTTON, LISTBOX1, TREEVIEW, LINEPAT, TEXTBOX, CHECKPIN, TRASH,
@@ -80,7 +80,7 @@ public:
 	virtual bool GetInt(int id, int *val) {return false;};
 	virtual bool SetCheck(int id, anyOutput *o, bool state) {return false;};
 	virtual bool GetCheck(int Id) {return bChecked;};
-	virtual bool GetText(int id, char *txt) {return false;};
+	virtual bool GetText(int id, char *txt, int size) {return false;};
 	virtual void MBtrack(MouseEvent *mev, anyOutput *o) {return;};
 	virtual void Activate(int id, bool active){return;};
 };
@@ -111,7 +111,7 @@ class DlgRoot:public tag_DlgObj {
 public:
 	anyOutput *CurrDisp;
 
-	DlgRoot(DlgInfo *tmpl);
+	DlgRoot(DlgInfo *tmpl, DataObj *d);
 	~DlgRoot();
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 	void DoPlot(anyOutput *o);
@@ -123,8 +123,9 @@ public:
 	bool SetCheck(int id, anyOutput *o, bool state);
 	bool GetCheck(int Id);
 	void Activate(int id, bool active);
-	bool GetText(int id, char *txt);
+	bool GetText(int id, char *txt, int size);
 	bool SetText(int id, char *txt);
+	bool SetValue(int id, double val);
 	bool TextStyle(int id, int style);
 	bool TextFont(int id, int font);
 	bool TextSize(int id, int size);
@@ -136,11 +137,13 @@ public:
 	anyOutput *GetOutputClass(){return CurrDisp;};
 
 private:
+	anyOutput *ParentOut;
 	int cDlgs, Result, cContinue;
+	DataObj *data;
 	Dialog *oldFocus, *oldDefault, *oldTabStop;
-	bool bActive;
+	bool bActive, bRedraw;
 	GraphObj *c_go;
-	Dialog **tabstops;
+	Dialog **tabstops, *mrk_item;
 	DlgTmpl **dlg;
 	MouseEvent *mev;
 };
@@ -164,10 +167,11 @@ public:
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 	void DoPlot(anyOutput *o);
 	bool Select(int x, int y, anyOutput *o);
-	bool GetText(int id, char *txt);
+	void MBtrack(MouseEvent *mev, anyOutput *o);
+	bool GetText(int id, char *txt, int size);
 
 private:
-	mLabel *cont;
+	TextFrame *cont;
 };
 
 class ArrowButton:public Dialog {
@@ -182,7 +186,6 @@ private:
 
 class ColorButton:public Dialog {
 public:
-	ColorButton(tag_DlgObj *par, DlgInfo * desc, RECT rec, unsigned long color);
 	ColorButton(tag_DlgObj *par, DlgInfo * desc, RECT rec, DWORD *color);
 	void DoPlot(anyOutput *o);
 	bool Select(int x, int y, anyOutput *o);
@@ -314,6 +317,7 @@ public:
 	void DoPlot(anyOutput *o);
 	bool Select(int x, int y, anyOutput *o);
 	void SetColor(int id, DWORD color);
+	void MBtrack(MouseEvent *mev, anyOutput *o);
 
 private:
 	char *txt;
@@ -330,7 +334,7 @@ public:
 	virtual bool Select(int x, int y, anyOutput *o);
 	bool GetValue(int id, double *val);
 	bool GetInt(int id, int *val);
-	bool GetText(int id, char *txt);
+	bool GetText(int id, char *txt, int size);
 	virtual void MBtrack(MouseEvent *mev, anyOutput *o);
 	virtual void Activate(int id, bool active);
 
@@ -340,7 +344,7 @@ private:
 
 class RangeInput:public InputText {
 public:
-	RangeInput(tag_DlgObj *par, DlgInfo * desc, RECT rec, char *text);
+	RangeInput(tag_DlgObj *par, DlgInfo * desc, RECT rec, char *text, DataObj *d);
 	~RangeInput();
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 	bool Select(int x, int y, anyOutput *o);
@@ -454,12 +458,13 @@ private:
 
 class TabSheet:public Group {
 public:
-	TabSheet(tag_DlgObj *par, DlgInfo * desc, RECT rec, TabSHEET *sh);
+	TabSheet(tag_DlgObj *par, DlgInfo * desc, RECT rec, TabSHEET *sh, DataObj *d);
 	~TabSheet();
 	void DoPlot(anyOutput *o);
 	bool Select(int x, int y, anyOutput *o);
 
 private:
+	DataObj *data;
 	RECT rctab;
 	char *Text;
 };
@@ -485,7 +490,7 @@ public:
 	void DoPlot(anyOutput *o);
 	bool Select(int x, int y, anyOutput *o);
 	bool GetInt(int id, int *val);
-	bool GetText(int id, char *txt);
+	bool GetText(int id, char *txt, int size);
 	void MBtrack(MouseEvent *mev, anyOutput *o);
 
 private:
@@ -531,6 +536,7 @@ private:
 //prototypes TheDialog.cpp
 bool UseRangeMark(DataObj *d, int type, char* =0L, char* =0L, char* =0L, char* =0L,
 	char* =0L, char* =0L, char* =0L, char* =0L, char* =0L, char* =0L, char* =0L);
+int com_StackDlg(int,DlgRoot*,AccRange**,int*,char***,int*,AccRange**,bool*,int*,int*,bool*);
 DlgInfo *CompileDialog(char* tmpl, void **ptypes);
 
 //prototypes ODbutton.cpp
@@ -547,3 +553,4 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int i
 void OD_AxisTempl(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int id);
 void OD_AxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int id);
 void OD_NewAxisTempl(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int id);
+void OD_NewAxisTempl3D(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int id);
