@@ -1,4 +1,4 @@
-//Utils.cpp, Copyright (c) 2000-2006 R.Lackner
+//Utils.cpp, Copyright (c) 2000-2007 R.Lackner
 //Collection of utility functions and classes for RLPlot
 //
 //    This file is part of RLPlot.
@@ -89,7 +89,11 @@ void NiceAxis(AxisDEF *axis, int nTick)
 	HiVal = LoVal = Step * floor(axis->min/Step);
 	axis->max *=1.05f;
 	while(HiVal < axis->max) HiVal += Step;
-	axis->min = LoVal;
+	if((axis->flags & AXIS_LOG) == AXIS_LOG) {
+		if (LoVal > defs.min4log) axis->min = LoVal;
+		if ((LoVal + Step) > defs.min4log && (LoVal + Step) < axis->min) axis->min = LoVal+Step;
+		}
+	else axis->min = LoVal;
 	axis->max = HiVal;
 	axis->Start = axis->min;
 	axis->Step = Step;
@@ -126,8 +130,7 @@ double base4log(AxisDEF *axis, int direc)
 	case 2:		cmd = SIZE_BOUNDS_ZMIN;		break;
 	default:	return 1.0;
 		}
-	if(axis->min > defs.min4log) lv = axis->min;
-	else lv = ((GraphObj*)axis->owner)->GetSize(cmd);
+	lv = axis->min > defs.min4log ? axis->min : defs.min4log;
 	if(lv <= defs.min4log) return defs.min4log;
 	lv = log10(lv);
 	lv -= (Magn = floor(lv));
@@ -257,7 +260,7 @@ char *str_ltrim(char *str) {
 	}
 
 char *str_rtrim(char *str) {
-	int i;
+	size_t i;
 
 	i = strlen(str);
 	while(i > 0 && str[i-1] <= ' ') str[--i] = '\0';
@@ -268,13 +271,104 @@ char *str_trim(char *str) {
 	str = str_ltrim(str);			return str_rtrim(str);
 }
 
+//remove leading and tailing quotatation
+void rmquot(char *str)
+{
+	size_t i, len;
+	char c;
+
+	if(str && str[0] && (*str == '"' || *str == '\'')) {
+		len = strlen(str);		c = *str;
+		if(str[len-1] == c) {
+			str[len-1] = 0;
+			for(i = 1; i < len; str[i-1] = str[i++]);
+			}
+		}
+}
+
+int strpos(char *needle, char *haystack)
+{
+	int i, j;
+
+	if(!needle || !needle[0] || !haystack || !haystack[0]) return -1;
+	for(i = j = 0; haystack[i]; i++, j=0) {
+		if(haystack[i] == needle[0]) for (j = 1; haystack[i+j]; j++) {
+			if(needle[j] != haystack[i+j]) break; 
+			}
+		if(j && !needle[j]) return i;
+		}
+	return -1;
+}
+
+char *strreplace(char *needle, char *replace, char *haystack)
+{
+	static char *result = 0L;
+	static size_t reslen = 0;
+	size_t i, j, k, l;
+
+	if(!needle || !needle[0] || !haystack || !haystack[0]) return result;
+	if(!result) result = (char*)malloc(reslen = 100);
+	result[0] = 0;		l = strlen(needle);
+	for(i = j = k = 0; haystack[i]; i++, j=0) {
+		if(haystack[i] == needle[0]) for (j = 1; haystack[i+j]; j++) {
+			if(needle[j] != haystack[i+j]) break; 
+			}
+		if(j && !needle[j]) {
+			if(replace && replace[0]) {
+				if(reslen < (i + (int)strlen(replace) + 10)) {
+					result = (char*)realloc(result, reslen += 100);
+					}
+				for(j = 0; replace[j]; j++) result[k++] = replace[j];
+				}
+			i += (l-1);
+			}
+		else result[k++] = haystack[i];
+		}
+	result[k++] = 0;
+	return result;
+}
+
+char *substr(char *text, int pos1, int pos2)
+{
+	static char *result = 0L;
+	static size_t reslen = 0;
+	int i, j;
+	size_t l;
+
+	if(!text || !text[0]) return 0L;
+	l = strlen(text);
+	if(pos1 < 0) pos1 = 0;			if(pos2 < pos1) pos2 = (int)(l+1);
+	if(!result) result = (char*)malloc(reslen = 100);
+	while (reslen < l) result = (char*) realloc(result, reslen += 100);
+	for(i = 0; i < pos1 && text[i]; i++);
+	for(j = 0; i <= pos2 && text[i]; result[j++] = text[i++]);
+	result[j] = 0;
+	return result;
+}
+
+//copy string in src to dest, returning the stringlength of src
+//   seek better solution for long strings
+int rlp_strcpy(char*dest, int size, char*src)
+{
+	int i;
+
+	if(dest && src) {
+		for(i = 0; i < size; i++) {
+			if(!(dest[i] = src[i])) return i;
+			}
+		dest[i-1] = 0;
+		return i-1;
+		}
+	return 0;
+}
+
 // restyle formula
 void ReshapeFormula(char **text)
 {
 	int i, j, l;
 
 	if(!text || !*text || !**text) return;
-	l = strlen(*text);
+	l = (int)strlen(*text);
 	for(i = j = 0; i < l; i++) {
 		if((*text)[i] == ';') {
 			if((*text)[i+1] == ';' || (*text)[i+1] == '\n') i++;
@@ -282,8 +376,8 @@ void ReshapeFormula(char **text)
 		TmpTxt[j++] = (*text)[i];
 		}
 	TmpTxt[j] = 0;
-	if((int)strlen(TmpTxt) < l && TmpTxt[0]) {
-		free(*text);	*text = strdup(TmpTxt);	
+	if(j && j <= l && TmpTxt[0]) {
+		rlp_strcpy(*text, l+1, TmpTxt);
 		}
 }
 
@@ -294,21 +388,25 @@ void TranslateResult(anyResult *res)
 
 	switch (res->type) {
 	case ET_VALUE:
-		if(res->value == HUGE_VAL) strcpy(tr_text, "inf");
-		else if(res->value == -HUGE_VAL) strcpy(tr_text, "-inf");
+		if(res->value == HUGE_VAL) rlp_strcpy(tr_text, 80, "inf");
+		else if(res->value == -HUGE_VAL) rlp_strcpy(tr_text, 80, "-inf");
+#ifdef USE_WIN_SECURE
+		else sprintf_s(tr_text, 80, "%g", res->value);
+#else
 		else sprintf(tr_text, "%g", res->value);
+#endif
 		res->text = tr_text;				return;
 	case ET_BOOL:
-		sprintf(tr_text, "%s", ((int)res->value) ? "true" : "false"); 
+		rlp_strcpy(tr_text, 80, ((int)res->value) ? (char*)"true" : (char*)"false"); 
 		res->text = tr_text;				return;
 	case ET_DATE:
-		sprintf(tr_text, "%s", value_date(res->value, defs.fmt_date)); 
+		rlp_strcpy(tr_text, 80, value_date(res->value, defs.fmt_date)); 
 		res->text = tr_text;				return;
 	case ET_TIME:
-		sprintf(tr_text, "%s", value_date(res->value, defs.fmt_time)); 
+		rlp_strcpy(tr_text, 80, value_date(res->value, defs.fmt_time)); 
 		res->text = tr_text;				return;
 	case ET_DATETIME:
-		sprintf(tr_text, "%s", value_date(res->value, defs.fmt_datetime)); 
+		rlp_strcpy(tr_text, 80, value_date(res->value, defs.fmt_datetime)); 
 		res->text = tr_text;				return;
 	case ET_TEXT:
 		if(res->text && res->text[0])		return;
@@ -335,7 +433,7 @@ void CleanTags(char *txt, int *i1, int *i2, int *i3)
 					if(no_tags[k][l] != txt[i+l]) break;
 				if(!no_tags[k][l]){
 					na = false;
-					i += ((w=strlen(no_tags[k]))-1);
+					i += ((w=(int)strlen(no_tags[k]))-1);
 					if(i1 && *i1 > i) *i1 -= w;
 					if(i2 && *i2 > i) *i2 -= w;
 					if(i3 && *i3 > i) *i3 -= w;
@@ -374,7 +472,11 @@ char *Nat2Int(char *text)	//format locale number to intranational notation
 
 void WriteNatFloatToBuff(char *buff, double val)
 {
-	WriteFloatToBuff(buff, val);
+#ifdef USE_WIN_SECURE
+	sprintf_s(buff, 20, " %g", val);
+#else
+	sprintf(buff, " %g", val);
+#endif
 	Int2Nat(buff);
 }
 
@@ -385,10 +487,14 @@ bool Txt2Flt(char *txt, double *val)
 	if(txt && txt[0] && val) {
 		if(!txt[1] && (txt[0] == defs.DecPoint[0] || txt[0] < '0' ||
 			txt[0] > '9'))return false;
-		if(txt && txt[0] && (tmp = strdup(txt))){
+		if(txt && txt[0] && (tmp = (char*)memdup(txt, (int)strlen(txt)+1, 0))){
 			Nat2Int(tmp);
 			//the return value of sscanf only roughly identifies a number
+#ifdef USE_WIN_SECURE
+			if(!sscanf_s(tmp,"%lf", val)){
+#else
 			if(!sscanf(tmp,"%lf", val)){
+#endif
 				free(tmp);
 				return false;
 				}
@@ -403,7 +509,7 @@ void RmTrail(char *txt)
 {
 	int i;
 
-	i = strlen(txt);
+	i = (int)strlen(txt);
 	while(i >0 && (txt[i-1] == '0' || txt[i-1] < 32)) txt[--i] = 0;
 	if(i > 1 && txt[i-1] == '.') txt[i-1] = 0;
 }
@@ -420,11 +526,15 @@ double NiceValue(double fv)
 
 char *Int2ColLabel(int nr1, bool uc)
 {
-	static char RetTxt[10];
+	static char RetTxt[12];
 	int i, j, nr = nr1;
 	char base = uc ? 'A' : 'a';
 
+#ifdef USE_WIN_SECURE
+	sprintf_s(RetTxt+8, 4, "%c\0", base + (nr %26));
+#else
 	sprintf(RetTxt+8, "%c\0", base + (nr %26));
+#endif
 	nr /= 26;
 	for (i = 7; nr && i>=0; i--) {
 		j = nr %27;
@@ -435,9 +545,31 @@ char *Int2ColLabel(int nr1, bool uc)
 	return RetTxt+i+1;
 }
 
+char *mkCellRef(int row, int col)
+{
+	static char RetTxt[20];
+
+#ifdef USE_WIN_SECURE
+	sprintf_s(RetTxt, 20, "%s%d", Int2ColLabel(col, false), row+1);
+#else
+	sprintf(RetTxt, "%s%d", Int2ColLabel(col, false), row+1);
+#endif
+	return RetTxt;
+}
+
+char *mkRangeRef(int r1, int c1, int r2, int c2)
+{
+	static char RetTxt[40];
+	int cb;
+
+	cb = rlp_strcpy(RetTxt, 20, mkCellRef(r1, c1)); 
+	RetTxt[cb++] = ':';
+	rlp_strcpy(RetTxt+cb, 40-cb, mkCellRef(r2, c2));
+	return RetTxt;
+}
 //convert strings to XML specifications
 //this offers a limited support for special characters
-char *str2xml(char *str)
+char *str2xml(char *str, bool bGreek)
 {
 	int i, j;
 	wchar_t wc;
@@ -446,22 +578,33 @@ char *str2xml(char *str)
 	for(i = j = 0; str[i] && j < 4090; i++) {
 		switch(str[i]) {
 		case '"':
-			j += sprintf(TmpTxt+j, "&quot;");
+			j += rlp_strcpy(TmpTxt+j, TMP_TXT_SIZE-j, "&quot;");
 			break;
 		case '&':
-			j += sprintf(TmpTxt+j, "&amp;");
+			j += rlp_strcpy(TmpTxt+j, TMP_TXT_SIZE-j, "&amp;");
 			break;
 		case '<':
-			j += sprintf(TmpTxt+j, "&lt;");
+			j += rlp_strcpy(TmpTxt+j, TMP_TXT_SIZE-j, "&lt;");
 			break;
 		case '>':
-			j += sprintf(TmpTxt+j, "&gt;");
+			j += rlp_strcpy(TmpTxt+j, TMP_TXT_SIZE-j, "&gt;");
 			break;
 		default:
-			if((unsigned char)str[i] <= 127) TmpTxt[j++]=str[i];
+#ifdef USE_WIN_SECURE
+			if(bGreek && str[i] >= 'A' && str[i] <= 'Z') j += sprintf_s(TmpTxt+j, 20, "&#%d;", str[i] - 'A' + 0x391);
+			else if(bGreek && str[i] >= 'a' && str[i] <= 'z') j += sprintf_s(TmpTxt+j, 20, "&#%d;", str[i] - 'a' + 0x3B1);
+#else
+			if(bGreek && str[i] >= 'A' && str[i] <= 'Z') j += sprintf(TmpTxt+j, "&#%d;", str[i] - 'A' + 0x391);
+			else if(bGreek && str[i] >= 'a' && str[i] <= 'z') j += sprintf(TmpTxt+j, "&#%d;", str[i] - 'a' + 0x3B1);
+#endif
+			else if((unsigned char)str[i] <= 127) TmpTxt[j++]=str[i];
 			else {
-				if(mbtowc(&wc, str+i, 1) >0) j += sprintf(TmpTxt+j, 
-					"&#%d;", ((unsigned short)wc));
+				if(mbtowc(&wc, str+i, 1) >0) 
+#ifdef USE_WIN_SECURE
+					j += sprintf_s(TmpTxt+j, TMP_TXT_SIZE-j, "&#%d;", ((unsigned short)wc));
+#else
+					j += sprintf(TmpTxt+j, "&#%d;", ((unsigned short)wc));
+#endif
 				}
 			}
 		}
@@ -471,40 +614,44 @@ char *str2xml(char *str)
 
 // split string str into array of strings using sep as separator
 // return number of lines created in nl
+static char  *split_buf = 0L;
+static int split_buf_size, split_buf_pos;
 char **split(char *str, char sep, int *nl)
 {
-	int i, j, k, l, ns;
-	char **ptr, *txt;
+	int i, j, l, ns;
+	char **ptr;
 
-	if(!str || !sep) return 0L;
-	if(!(txt = strdup(str))) return 0L;
-	k = (int)strlen(txt);
-	for(i = ns = 0; i < k; i++) if(txt[i] == sep) ns++;
-	if(!(ptr = (char**)calloc(ns+2, sizeof(char*)))){
-		free(txt);	return 0L;
-		}
-	for(i = j = l = 0; i < k; i++) {
-		if(txt[i] == sep) {
-			txt[i] = 0;		ptr[l++] = strdup(txt+j);		j = i+1;
+	if(!str || !str[0] || !sep) return 0L;
+	split_buf_pos = 0;
+	add_to_buff(&split_buf, &split_buf_pos, &split_buf_size, str, 0);
+	if(!split_buf || !split_buf_pos) return 0L;
+	for(i = ns = 0; i < split_buf_pos; i++) if(split_buf[i] == sep) ns++;
+	if(!(ptr = (char**)calloc(ns+2, sizeof(char*)))) return 0L;
+	for(i = j = l = 0; i < split_buf_pos; i++) {
+		if(split_buf[i] == sep) {
+			split_buf[i] = 0;
+			ptr[l++] = (char*)memdup(split_buf+j, (int)strlen(split_buf+j)+1, 0);
+			j = i+1;
 			}
 		}
-	ptr[l++] = strdup(txt+j);	if(nl) *nl = l;		free(txt);
+	ptr[l++] = (char*)memdup(split_buf+j, (int)strlen(split_buf+j)+1, 0);
+	if(nl) *nl = l;
 	return ptr;
 }
 
 char *fit_num_rect(anyOutput *o, int max_width, char *num_str)
 {
 	int i, j, k, w, h, slen;
-	char mant[20], expo[20], fmt[10];
+	char mant[30], expo[30], fmt[20];
 	double num;
 
-	o->oGetTextExtent(num_str, slen = strlen(num_str), &w, &h);
+	o->oGetTextExtent(num_str, slen = (int)strlen(num_str), &w, &h);
 	if(w < (max_width-5)) return num_str;
 	//first try to remove leading zero from exponent
 	for(i = 0; i < slen; i++) if(num_str[i] == 'e') break;
 	if(num_str[i] == 'e') while (num_str[i+2] == '0') {
 		for(j = i+2; num_str[j]; j++) num_str[j] = num_str[j+1];
-		o->oGetTextExtent(num_str, slen = strlen(num_str), &w, &h);
+		o->oGetTextExtent(num_str, slen = (int)strlen(num_str), &w, &h);
 		if(w < (max_width-5)) return num_str;
 		}
 	//no success: reduce the number of digit by rounding
@@ -514,21 +661,91 @@ char *fit_num_rect(anyOutput *o, int max_width, char *num_str)
 		}
 	if(num_str[i] =='e') mant[i] = 0;						k = i - k-1;
 	for(j = 0; num_str[i]; j++, i++) expo[j] = num_str[i];		expo[j] = 0;
+#ifdef USE_WIN_SECURE
+	sscanf_s(mant, "%lf", &num);
+#else
 	sscanf(mant, "%lf", &num);	
+#endif
 	if(k >0) do {
-		sprintf(fmt, "%s.%dlf%s", "%", k, expo);
-		sprintf(num_str, fmt, num);							k--;
-		o->oGetTextExtent(num_str, slen = strlen(num_str), &w, &h);
+#ifdef USE_WIN_SECURE
+		sprintf_s(fmt, 20, "%%.%dlf%s", k, expo);
+		slen = sprintf_s(num_str, 60, fmt, num);		//num_str is much longer than 60
+#else
+		sprintf(fmt, "%%.%dlf%s", k, expo);
+		slen = sprintf(num_str, fmt, num);
+#endif
+		k--;
+		o->oGetTextExtent(num_str, slen, &w, &h);
 		if(w < (max_width-5)) return num_str;
 		}while (k >= 0);
 	//cannot fit: return hash marks instead
-	for(i = w = 0; w < (max_width-5); i++) {
-		sprintf(num_str+i, "#");
-		o->oGetTextExtent(num_str, slen = strlen(num_str), &w, &h);
+	for(i = w = 0; w < (max_width-5) && i < 11; i++) {
+		rlp_strcpy(num_str, i+2, "##########");
+		o->oGetTextExtent(num_str, i+1, &w, &h);
 		}
 	num_str[i-1] = 0;		return num_str;
 }
 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+// utilities to add a line or number to a text buffer: memory file
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+void add_to_buff(char** dest, int *pos, int *csize, char *txt, int len)
+{
+	if(!len)len = (int)strlen(txt);
+	if(!len) return;
+	if((*pos+len+1)>= *csize) {
+		*csize += 1000;
+		while(*csize < (*pos+len+1)) *csize += 1000;
+		*dest = (char*)realloc(*dest, *csize);
+		}
+	if(*dest) {
+		*pos += rlp_strcpy(*dest+*pos, len+1, txt);
+		}
+}
+
+void add_int_to_buff(char** dest, int *pos, int *csize, int value, bool lsp, int ndig)
+{
+	int len;
+	char txt[40];
+
+#ifdef USE_WIN_SECURE
+	len = sprintf_s(txt, 40, lsp ? " %d" : "%d", value);
+#else
+	len = sprintf(txt, lsp ? " %d" : "%d", value);
+#endif
+	add_to_buff(dest, pos, csize, txt, len);
+}
+
+void add_dbl_to_buff(char** dest, int *pos, int *csize, double value, bool lsp)
+{
+	int len;
+	char txt[40];
+
+#ifdef USE_WIN_SECURE
+	len = sprintf_s(txt, 40, lsp ? " %g" : "%g", value);
+#else
+	len = sprintf(txt, lsp ? " %g" : "%g", value);
+#endif
+	add_to_buff(dest, pos, csize, txt, len);
+}
+
+void add_hex_to_buff(char** dest, int *pos, int *csize, DWORD value, bool lsp)
+{
+	int len;
+	char txt[40];
+
+	if(value) {
+#ifdef USE_WIN_SECURE
+		len = sprintf_s(txt, 40, lsp ? " 0x%08x" : "0x%08x", value);
+#else
+		len = sprintf(txt, lsp ? " 0x%08x" : "0x%08x", value);
+#endif
+		add_to_buff(dest, pos, csize, txt, len);
+		}
+	else if(lsp) add_to_buff(dest, pos, csize, " 0x0", 4);
+	else add_to_buff(dest, pos, csize, "0x0", 3);
+}
+
 //----------------------------------------------------------------------------
 // bounding rectangle utilities
 //----------------------------------------------------------------------------
@@ -753,6 +970,90 @@ void DrawBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, in
 #undef MIN_SEG
 }
 
+//----------------------------------------------------------------------------
+// create a Bezier spline through data points
+static void ipol_curve(lfPOINT *p1, lfPOINT *p2, lfPOINT *p3, lfPOINT *cp1, lfPOINT *cp2)
+{
+	double dx, dy, l;
+	lfPOINT tHat;
+
+	tHat.fx = p3->fx - p1->fx;		tHat.fy = p3->fy - p1->fy;
+	l = sqrt(tHat.fx * tHat.fx + tHat.fy * tHat.fy);
+	tHat.fx /= l;					tHat.fy /= l;
+	cp1->fx = cp2->fx = p2->fx;		cp1->fy = cp2->fy = p2->fy;
+	dx = p3->fx - p2->fx;			dy = p3->fy - p2->fy;		l = sqrt(dx*dx + dy*dy)/3.0;
+	cp2->fx += (tHat.fx * l);		cp2->fy += (tHat.fy *l);
+	dx = p2->fx - p1->fx;			dy = p2->fy - p1->fy;		l = sqrt(dx*dx + dy*dy)/3.0;
+	cp1->fx -= (tHat.fx * l);		cp1->fy -= (tHat.fy *l);
+}
+
+static void mirr_vecvec(lfPOINT *a0, lfPOINT *a1, lfPOINT *v1)
+{
+	double dx, dy, as, ac, vs, vc, s, c, l;
+	lfPOINT sol1, sol2;
+
+	//mirror vector a0 -> v1 by rotation around vector a0 -> a1
+	dx = a1->fx - a0->fx;					dy = a1->fy - a0->fy;
+	l = sqrt(dx*dx + dy*dy);	as = dy/l;			ac = dx/l;
+	dx = v1->fx - a0->fx;					dy = v1->fy - a0->fy;
+	l = sqrt(dx*dx + dy*dy);	vs = dy/l;			vc = dx/l;
+	//calculate cw and ccw solution
+	s = sin((asin(as)-asin(vs))*2.0);		c = cos((acos(ac)-acos(vc))*2.0);
+	sol1.fx = dx * c + dy * s + a0->fx;		sol1.fy = dy * c - dx * s + a0->fy;
+	s = sin((asin(as)-asin(-vs))*2.0);		c = cos((acos(ac)-acos(-vc))*2.0);
+	sol2.fx = dx * c + dy * s + a0->fx;		sol2.fy = dy * c - dx * s + a0->fy;
+	//get better solution
+	dx = sol1.fx - a1->fx;					dy = sol1.fy - a1->fy;
+	l = sqrt(dx*dx + dy*dy);
+	dx = sol2.fx - a1->fx;					dy = sol2.fy - a1->fy;
+	if( l < sqrt(dx*dx + dy*dy)) {
+		v1->fx = sol1.fx;					v1->fy = sol1.fy;
+		}
+	else {
+		v1->fx = sol2.fx;					v1->fy = sol2.fy;
+		}
+}
+
+int mkCurve(lfPOINT *src, int n1, lfPOINT **dst, bool bClosed)
+{
+	int i, j, iret;
+
+	if(!src || n1 < 3) return 0;
+	if(!(*dst = (lfPOINT*)malloc((n1*3+2) * sizeof(lfPOINT))))return 0;
+	for(i = j = iret = 0; i < n1; i++, j += 3) {
+		(*dst)[j].fx = src[i].fx;		(*dst)[j].fy = src[i].fy;
+		if(i && i < (n1-1)){
+			ipol_curve(&src[i-1], &src[i], &src[i+1], *dst+j-1, *dst+j+1);
+			}
+		else if(i) {
+			iret = j+1;					break;
+			}
+		}
+	if(bClosed && iret >2) {
+		if((*dst)[j].fx != (*dst)[0].fx || (*dst)[j].fy != (*dst)[0].fy) {
+			j += 3;								iret += 3;
+			(*dst)[j].fx = (*dst)[0].fx;		(*dst)[j].fy = (*dst)[0].fy;	
+			}
+		if(j < 6) {
+			free(*dst);		*dst = 0L;				return 0;
+			}
+		ipol_curve(*dst+j-3, *dst+j, *dst+3, *dst+j-1, *dst+1);
+		ipol_curve(*dst+j-6, *dst+j-3, *dst+j, *dst+j-4, *dst+j-2);
+		}
+	else if(!bClosed && iret>3) {
+		(*dst)[1].fx = (*dst)[0].fx + (*dst)[3].fx - (*dst)[2].fx;
+		(*dst)[1].fy = (*dst)[0].fy + (*dst)[3].fy - (*dst)[2].fy;
+		mirr_vecvec(*dst, *dst+3, *dst+1);
+		(*dst)[j-1].fx = (*dst)[j].fx + (*dst)[j-3].fx - (*dst)[j-2].fx;
+		(*dst)[j-1].fy = (*dst)[j].fy + (*dst)[j-3].fy - (*dst)[j-2].fy;
+		mirr_vecvec(*dst+j, *dst+j-3, *dst+j-1);
+		}
+	else {
+		free(*dst);			*dst = 0L;				return 0;
+		}
+	return iret;
+}
+
 //----------------------------------------------------------------------------
 // create a circular polygon
 //use circular Bresenham's algorithm to draw arcs
@@ -840,19 +1141,17 @@ void InvertPolygon(POINT *pts, int nPts, LineDEF *Line, FillDEF *Fill, RECT *rc,
 void InvertLine(POINT *pts, int nPts, LineDEF *Line, RECT *rc, 
 	anyOutput *o, bool mark)
 {
+	int i;
 	LineDEF OldLine;
-	double minw = defs.GetSize(SIZE_DATA_LINE);
 
 	memcpy(&OldLine, Line, sizeof(LineDEF));
-	if(OldLine.width <= 0.001) memcpy(&OldLine, defs.GetLine(), sizeof(LineDEF));
+	if(OldLine.width <= 0.0001) OldLine.width = 0.0001;
+	for(i = 0; o->un2fiy(OldLine.width) < 1.0 && i < 50; i++) OldLine.width *= 2.0;
 	OldLine.color = mark ? Line->color : 0x00ffffffL;
-	OldLine.width = OldLine.width < minw ? minw * 3.0 : OldLine.width *3.0;
-	o->SetLine(&OldLine);
-	o->oPolyline(pts, nPts);
-	OldLine.width = Line->width;
+	OldLine.width *= 3.0;				o->SetLine(&OldLine);
+	o->oPolyline(pts, nPts);			OldLine.width = Line->width;
 	OldLine.color = mark ? Line->color ^ 0x00ffffffL : Line->color;
-	o->SetLine(&OldLine);
-	o->oPolyline(pts, nPts);
+	o->SetLine(&OldLine);				o->oPolyline(pts, nPts);
 	if(rc) o->UpdateRect(rc, false);
 }
 
@@ -1069,8 +1368,14 @@ void DeleteGO(GraphObj *go)
 	case GO_XYSTAT:			delete((xyStat*)go);		break;
 	case GO_FITFUNC3D:		delete((FitFunc3D*)go);		break;
 	case GO_BEZIER:			delete((Bezier*)go);		break;
+	case GO_TEXTFRAME:		delete((TextFrame*)go);		break;
+	case GO_NORMQUANT:		delete((NormQuant*)go);		break;
 	default:
+#ifdef USE_WIN_SECURE
+		sprintf_s(TmpTxt, TMP_TXT_SIZE, "Cannot delete Object\nwith Id %ld", go->Id);
+#else
 		sprintf(TmpTxt, "Cannot delete Object\nwith Id %ld", go->Id);
+#endif
 		ErrorBox(TmpTxt);
 		//we do not delete the object, probably we recover
 		}
@@ -1116,17 +1421,26 @@ bool BackupFile(char *FileName)
 
 	//no backup necessary if file does not exist
 	if(!(FileExist(FileName))) return true;
-	strcpy(Name, FileName);
-	i = strlen(Name)-1;
+	rlp_strcpy(Name, 512,FileName);
+	i = (int)strlen(Name)-1;
 	if(Name[--i] == '.') Name[i] = 0;
 	else if(Name[--i] == '.') Name[i] = 0;
 	else if(Name[--i] == '.') Name[i] = 0;
 	else return false;
 	i = 1;
 	do {
+#ifdef USE_WIN_SECURE
+		sprintf_s(ext, 6, ".%03d", i);
+		sprintf_s(TmpFileName, 512, "%s%s", Name, ext);
+		if(!(fopen_s(&TheFile, TmpFileName, "r"))) {
+			fclose(TheFile);
+			}
+		else break;
+#else
 		sprintf(ext, ".%03d", i);
 		sprintf(TmpFileName, "%s%s", Name, ext);
 		if((TheFile = fopen(TmpFileName, "r"))) fclose(TheFile);
+#endif
 		i++;
 	} while (i < 999 && TheFile);
 	if(i >= 999) {								//too many backups exist already
@@ -1146,7 +1460,11 @@ bool IsRlpFile(char *FileName)
 	char Line[10];
 	bool bRet = false;
 
+#ifdef USE_WIN_SECURE
+	if(fopen_s(&TheFile, FileName, "r")) return false;
+#else
 	if(0L ==(TheFile = fopen(FileName, "r"))) return false;
+#endif
 	fread(Line, 1, 8, TheFile);
 	Line[5] = 0;
 	if(0 == strcmp(Line, ";RLP "))bRet = true;
@@ -1160,7 +1478,11 @@ bool IsXmlFile(char *FileName)
 	char Line[10];
 	bool bRet = false;
 
+#ifdef USE_WIN_SECURE
+	if(fopen_s(&TheFile, FileName, "r")) return false;
+#else
 	if(0L ==(TheFile = fopen(FileName, "r"))) return false;
+#endif
 	fread(Line, 1, 8, TheFile);
 	Line[6] = 0;
 	if(0 == strcmp(Line, "<?xml "))bRet = true;
@@ -1172,11 +1494,14 @@ bool FileExist(char *FileName)
 {
 	FILE *TheFile;
 
-
+#ifdef USE_WIN_SECURE
+	if(ENOENT == fopen_s(&TheFile, FileName, "r")) return false;
+#else
 	if(0L ==(TheFile = fopen(FileName, "r"))) {
 		if(errno == ENOENT) return false;
 		return true;
 		}
+#endif
 	fclose(TheFile);
 	return true;
 }
@@ -1187,7 +1512,7 @@ bool FileExist(char *FileName)
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 bool IsPlot3D(GraphObj *g)
 {
-	if(g && (g->Id == GO_PLOT3D || g->Id == GO_FUNC3D)) return true;
+	if(g && (g->Id == GO_PLOT3D || g->Id == GO_FUNC3D || g->Id == GO_FITFUNC3D)) return true;
 	return false;
 }
 
@@ -1353,6 +1678,31 @@ DWORD CheckNewLFPoint(lfPOINT *loc, lfPOINT *old_v, lfPOINT *new_v, GraphObj *pa
 	return flags;
 }
 
+DWORD CheckNewString(char **loc, char *s_old, char *s_new, GraphObj *par, DWORD flags)
+{
+	int ocb, ncb, cb;
+
+	if(s_old && s_new) {
+		if(!strcmp(s_old, s_new)) return flags;
+		ocb = (int)strlen(s_old);		ncb = (int)strlen(s_new);
+		cb = ncb > ocb ? ncb : ocb;
+		if(cb > ocb) {
+			*loc = (char*)realloc(*loc, cb * sizeof(char)+1);
+			}
+		Undo.String(par, loc, flags);	flags |= UNDO_CONTINUE;
+		if(*loc) rlp_strcpy(*loc, cb+1, s_new); 
+		}
+	else if(!s_old && s_new && s_new[0]) {
+		Undo.String(par, loc, flags);	flags |= UNDO_CONTINUE;
+		*loc = (char *)memdup(s_new, (int)strlen(s_new) +1, 0);
+		}
+	else if(s_old && s_old[0] && !s_new) {
+		Undo.String(par, loc, flags);	flags |= UNDO_CONTINUE;
+		if(*loc) *loc[0] = 0;
+		}
+	return flags;
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //execute clipping of 3D objects
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
