@@ -1,4 +1,4 @@
-//RLPlot.h, Copyright (c) 2000-2006 R.Lackner
+//RLPlot.h, Copyright (c) 2000-2007 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -24,20 +24,24 @@
 
 #define Swap(a,b) {a^=b;b^=a;a^=b;}
 inline int iround(double a) {return a < 0.0 ?(int)(a-0.499) : (int)(a+0.499);}
-inline int WriteFloatToBuff(char *buff, double val) {return sprintf(buff, " %g", val);}
 
 #define _PI		3.1415926535897932384626433832795028841971693993751
 #define _SQRT2	1.4142135623730950488016887242096980785696718753769
 
-#ifdef _WINDOWS
+#ifdef _WINDOWS					//platform is windows
+#pragma warning( disable : 4997 )
 #include <windows.h>
-#define w_char unsigned short
+#if _MSC_VER >= 1400
+#define USE_WIN_SECURE
+#endif
+#define w_char WCHAR
 #define _SBINC	1				//scrollbox extra space/line
 #define _TXH	4.0				//graph text default size in mm
 
-#else
-#define DWORD unsigned long
-#define w_char unsigned short
+#else							//platform is *nix
+#include <sys/types.h>
+#define DWORD u_int32_t
+#define w_char wchar_t
 #define _SBINC	8				//scrollbox extra space/line
 #define _TXH	3.0				//graph text default size in mm
 #define RLP_PORT	4321		//clipboard server
@@ -77,13 +81,13 @@ enum {SIZE_MINE, SIZE_SYMBOL, SIZE_SYM_LINE, SIZE_DATA_LINE, SIZE_TEXT,
 	SIZE_A, SIZE_B, SIZE_MX, SIZE_MY, SIZE_MIN_Z, SIZE_MAX_Z, SIZE_MIN_X, SIZE_MAX_X,
 	SIZE_MIN_Y, SIZE_MAX_Y, SIZE_TICK_ANGLE, SIZE_RRECT_RAD, SIZE_XCENT, SIZE_YCENT,
 	SIZE_ZCENT, SIZE_DRADIUS, SIZE_CURSORPOS, SIZE_CURSOR_XMIN, SIZE_CURSOR_YMIN,
-	SIZE_CURSOR_XMAX, SIZE_CURSOR_YMAX, SIZE_XSTEP, SIZE_LSPC};
+	SIZE_CURSOR_XMAX, SIZE_CURSOR_YMAX, SIZE_XSTEP, SIZE_LSPC, SIZE_SCALE};
 enum {COL_SYM_LINE, COL_SYM_FILL, COL_DATA_LINE, COL_TEXT, COL_BG,
 	COL_AXIS, COL_BAR_LINE, COL_BAR_FILL, COL_ERROR_LINE, COL_BUBBLE_LINE, 
 	COL_BUBBLE_FILLLINE, COL_BUBBLE_FILL, COL_ARROW, COL_WHISKER, COL_BOX_LINE,
 	COL_DRECT, COL_GRECT, COL_GRECTLINE, COL_POLYLINE, COL_POLYGON};
 enum {MRK_NONE, MRK_INVERT, MRK_GODRAW, MRK_SSB_DRAW};
-enum {CMD_NONE, CMD_ADDCHAR, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELETE,
+enum {CMD_NONE, CMD_ADDCHAR, CMD_ADDCHARW, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELETE,
 	CMD_BACKSP, CMD_CURRLEFT, CMD_CURRIGHT, CMD_CURRUP, CMD_CURRDOWN,
 	CMD_SHIFTLEFT, CMD_SHIFTRIGHT, CMD_SHIFTUP, CMD_SHIFTDOWN,
 	CMD_NEWGRAPH, CMD_DELGRAPH, CMD_DELOBJ, CMD_DROP_PLOT, CMD_DROP_GRAPH, CMD_OPEN,
@@ -116,7 +120,8 @@ enum {CMD_NONE, CMD_ADDCHAR, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELETE,
 	CMD_HIDE_MARK, CMD_LEGEND, CMD_FILENAME, CMD_LAYERS, CMD_OBJTREE, CMD_TEXTDEF,
 	CMD_HASSTACK, CMD_WRITE_GRAPHS, CMD_SETFONT, CMD_SETSTYLE, CMD_COPY, CMD_PASTE,
 	CMD_INSROW, CMD_INSCOL, CMD_DELROW, CMD_DELCOL, CMD_ADDTXT, CMD_ETRACC, CMD_SHPGUP, 
-	CMD_SHPGDOWN, CMD_ERRDESC, CMD_SAVEDATA, CMD_GETMARK, CMD_PASTE_OBJ};
+	CMD_SHPGDOWN, CMD_ERRDESC, CMD_SAVEDATA, CMD_GETMARK, CMD_PASTE_OBJ, CMD_COL_MOUSE,
+	CMD_MARKOBJ, CMD_SCALE};
 enum {SYM_CIRCLE, SYM_CIRCLEF, SYM_RECT, SYM_RECTF, SYM_TRIAU, SYM_TRIAUF,
 	SYM_TRIAD, SYM_TRIADF, SYM_DIAMOND, SYM_DIAMONDF, SYM_PLUS, SYM_CROSS,
 	SYM_STAR, SYM_HLINE, SYM_VLINE, SYM_TEXT, SYM_POS_PARENT = 0x1000};
@@ -129,13 +134,14 @@ enum {GO_UNKNOWN, GO_AXIS, GO_TICK, GO_GRIDLINE, GO_SYMBOL, GO_BUBBLE, GO_BAR,
 	GO_DRAGHANDLE, GO_DRAGRECT, GO_DRAG3D, GO_FRAMERECT, GO_SPHERE, GO_SVGOPTIONS,
 	GO_PLANE, GO_BRICK, GO_LINESEG, GO_LINE3D, GO_GRIDLINE3D, GO_GRIDRADIAL,
 	GO_SPHSCANL, GO_DROPL3D, GO_ARROW3D, GO_PLANE3D, GO_LEGITEM, GO_LEGEND,
-	GO_OBJTREE, GO_BEZIER,
+	GO_OBJTREE, GO_BEZIER, GO_TEXTFRAME,
 	GO_PLOT = 0x100, GO_PLOTSCATT, GO_REGRESSION, GO_BARCHART, GO_BUBBLEPLOT, 
 	GO_BOXPLOT, GO_DENSDISP, GO_STACKBAR, GO_STACKPG, GO_WATERFALL, GO_POLARPLOT,
 	GO_PIECHART, GO_RINGCHART, GO_GROUP, GO_STARCHART, GO_SCATT3D, GO_PLOT3D,
 	GO_RIBBON, GO_LIMITS, GO_FUNCTION, GO_FITFUNC, GO_FREQDIST, GO_GRID3D, GO_FUNC3D,
-	GO_XYSTAT, GO_FITFUNC3D,
+	GO_XYSTAT, GO_FITFUNC3D, GO_NORMQUANT,
 	GO_GRAPH = 0x200, GO_PAGE, GO_SPREADDATA = 0x300, GO_DEFRW};
+enum {OC_UNKNOWN, OC_HIMETRIC};
 enum {FILL_NONE, FILL_HLINES, FILL_VLINES, FILL_HVCROSS, FILL_DLINEU, FILL_DLINED,
 	FILL_DCROSS, FILL_STIPPLE1, FILL_STIPPLE2, FILL_STIPPLE3, FILL_STIPPLE4, 
 	FILL_STIPPLE5, FILL_ZIGZAG, FILL_COMBS, FILL_BRICKH, FILL_BRICKV, FILL_BRICKDU, 
@@ -151,7 +157,7 @@ enum {TM_STANDARD, TM_DRAW, TM_POLYLINE, TM_POLYGON, TM_RECTANGLE, TM_ELLIPSE,
 	TM_PASTE=0x200};
 enum {MC_LAST, MC_ARROW, MC_CROSS, MC_TEXT, MC_WAIT, MC_MOVE, MC_NORTH,
 	MC_NE, MC_EAST, MC_SE, MC_SALL, MC_ZOOM, MC_PASTE, MC_DRAWPEN, MC_DRAWREC, 
-	MC_DRAWRREC, MC_DRAWELLY};
+	MC_DRAWRREC, MC_DRAWELLY, MC_TXTFRM, MC_COLWIDTH};
 enum {FILE_ERROR, FILE_READ, FILE_WRITE, INIT_VARS, SAVE_VARS};
 enum {ARROW_NOCAP, ARROW_LINE, ARROW_TRIANGLE, ARROW_UNITS = 0x100};
 enum {MENU_NONE, MENU_SPREAD, MENU_GRAPH, MENU_PAGE};
@@ -205,6 +211,10 @@ typedef struct {
 	double fy;
 	}lfPOINT;
 
+typedef struct {
+	lfPOINT sx, sy, sz;
+	}scaleINFO;
+
 typedef struct {
 	double finc, fp;
 	}RunLinePat;				//used for line patterns
@@ -319,6 +329,8 @@ typedef struct {
 	int type;
 	double value;
 	char *text;
+	double *a_data;
+	int a_count;
 	} anyResult;
 
 //the AccRange class allows to access data objects with a 'a1:b1' style
@@ -333,6 +345,8 @@ public:
 	bool NextCol(int *x);
 	bool IsInRange(int x, int y);
 	bool BoundRec(RECT *rec);
+	char *RangeDesc(void *d, int style);	//d points to a DataObj class
+	int DataTypes(void *d, int *numerical, int *strings, int *datetime);
 
 private:
 	char *txt;
@@ -342,10 +356,41 @@ private:
 	bool Parse(int start);
 };
 
+class Triangle {
+public:
+	Triangle *next;
+	fPOINT3D pt[4];
+
+	void SetRect();
+	bool TestVertex(double x, double y);
+
+private:
+	double cx, cy, r2;				//circumcircle
+	fRECT rc;						//bounding rectangle
+	lfPOINT ld[3];					//line eqations
+};
+
+class Triangulate {
+public:
+	Triangle *trl;
+
+	Triangulate(Triangle *t_list);
+	bool AddEdge(fPOINT3D *p1, fPOINT3D *p2);
+	bool AddVertex(fPOINT3D *v);
+
+private:
+	typedef struct edge {
+		edge *next;
+		fPOINT3D p1, p2;
+	};
+	edge *edges;
+};
+
 class anyOutput{
 public:
 	int units;							//use units mm, inch ...
 	int MrkMode;						//existing mark on screen
+	int OC_type;						//specify display, printer clipboard ...
 	void *MrkRect;						//pointer to e.g. the marked rectangle
 	fRECT Box1;							//User drawing rectangle
 	lfPOINT Box1z;						// add 3D to Box1
@@ -375,6 +420,7 @@ public:
 
 	anyOutput();
 	void SetRect(fRECT rec, int units, AxisDEF *x_ax, AxisDEF *y_ax);
+	void UseAxis(AxisDEF *ax, int type);
 	void SetSpace(fPOINT3D*,fPOINT3D*,int,double*,fPOINT3D*,AxisDEF*,AxisDEF*,AxisDEF*);
 	void LightSource(double x, double y);
 	DWORD VecColor(double *plane_vec, DWORD col1, DWORD col2);
@@ -429,18 +475,20 @@ public:
 	virtual bool oGetPix(int x, int y, DWORD *col){return false;};
 	virtual bool oDrawIcon(int type, int x, int y) {return false;};
 	virtual bool oGetTextExtent(char *text, int cb, int *width, int *height);
+	virtual bool oGetTextExtentW(w_char *text, int cb, int *width, int *height);
 	virtual bool oCircle(int x1, int y1, int x2, int y2, char *nam = 0L){return false;};
 	virtual bool oSphere(int cx, int cy, int r, POINT *pts, int cp, char *nam = 0L);
 	virtual bool oPolyline(POINT * pts, int cp, char *nam = 0L){return false;};
 	virtual bool oRectangle(int x1, int y1, int x2, int y2, char *nam = 0L){return false;};
 	virtual bool oSolidLine(POINT *p){return false;};
 	virtual bool oTextOut(int x, int y, char *txt, int cb){return false;};
+	virtual bool oTextOutW(int x, int y, w_char *txt, int cb){return false;};
 	virtual bool oPolygon(POINT *pts, int cp, char *nam = 0L){return false;};
-	virtual bool oArc(int x1, int y1, int x2, int y2, int quads){return false;};
 };
 
 enum {ET_UNKNOWN, ET_VALUE, ET_TEXT, ET_FORMULA, ET_ERROR, ET_BOOL, 
-	ET_DATE, ET_TIME, ET_DATETIME, ET_BUSY=0x100, ET_CIRCULAR=0x200};
+	ET_DATE, ET_TIME, ET_DATETIME, ET_BUSY=0x100, ET_CIRCULAR=0x200, ET_EMPTY=0x400, 
+	ET_NODRAW=0x800, ET_NODRAW_EMPTY=0xc00};
 
 class EditText {
 public:
@@ -465,6 +513,8 @@ public:
 	void SetRec(RECT *rc);
 	int GetX() {return loc.x;};
 	int GetY() {return loc.y;};
+	int GetRX() {return crb.x;};
+	int GetRY() {return crb.y;};
 	int Cursor() {return CursorPos;};
 	bool isValue();
 	bool isFormula();
@@ -479,26 +529,31 @@ private:
 	DWORD TextCol;
 
 	void FindType();
+	void set_etracc();
 };
 
 class fmtText {
 
 typedef struct _fmt_txt_info {
-	int tag;
+	int tag, uc, uc_len;
 	char *txt;
 }fmt_txt_info;
 
 public:
+	fmtText();
 	fmtText(anyOutput *o, int x, int y, char *txt);
 	~fmtText();
 	bool StyleAt(int idx,  TextDEF *txt_def, int *style, int *font);
 	int rightTag(char *txt, int cb);
 	int leftTag(char *txt, int cb);
+	int ucTag(char *txt, int cb, int *tl, int *ucc);
+	int ucLeft(char *txt, int cb, int *tl, int *ucc);
 	void cur_right(int *pos);
 	void cur_left(int *pos);
 	bool oGetTextExtent(anyOutput *o, int *width, int *height, int cb);
 	void SetText(anyOutput *o, char *txt, int *px, int *py);
 	void DrawText(anyOutput *o);
+	void EditMode(bool bEdit);
 
 private:
 	bool SetTextDef(TextDEF *td, int idx);
@@ -509,24 +564,76 @@ private:
 	char *src;
 	POINT pos;
 	int n_split;
+	DWORD flags;
 	fmt_txt_info *split_text;
 };
 
+class TextValue {
+
+typedef struct _TxtValItem {
+	unsigned int hv1, hv2;
+	char *text;
+	double val;
+}TextValItem;
+
+public:
+	TextValue();
+	TextValue(TextValItem **tvi, int ntvi);
+	~TextValue();
+	double GetValue(char* txt);
+	bool GetItem(int idx, char **text, double *value);
+	void Reset();
+	TextValue *Copy();
+	int Count() {return nitems;};
+
+private:
+	TextValItem **items;
+	int nitems;
+	double next, inc;
+};
+
+class RangeInfo {
+public:
+	int col_width, row_height, first_width;
+	RangeInfo(int sel, int cw, int rh, int fw, RangeInfo *next);
+	RangeInfo(int sel, char *range, RangeInfo *next);
+	~RangeInfo();
+	int SetWidth(int w);
+	int SetDefWidth(int w);
+	int SetHeight(int h);
+	int SetFirstWidth(int w);
+	int GetWidth(int col);
+	int GetHeight(int row);
+	int GetFirstWidth();
+	int Type(){return r_type;};
+	RangeInfo *Next(){return ri_next;};
+
+private:
+	int r_type;		// 0 default
+					// 1 column info
+					// 2 row info
+	RangeInfo *ri_next;
+	AccRange *ar;
+};
+
 class DataObj{
 public:
-	int cRows, cCols;
+	int cRows, cCols, c_disp, r_disp;
+	RangeInfo *ri;
 	EditText ***etRows;
 
 	DataObj();
 	~DataObj();
 	virtual bool Init(int nRows, int nCols);
+	virtual bool mpos2dpos(POINT *mp, POINT *dp, bool can_scroll){return false;};
 	virtual bool SetValue(int row, int col, double val);
 	virtual bool SetText(int row, int col, char *txt);
 	virtual bool GetValue(int row, int col, double *v);
 	virtual bool GetText(int row, int col, char *txt, int len, bool bTranslate = true);
-	char ** GetTextPtr(int row, int col);
+	char **GetTextPtr(int row, int col);
 	virtual bool GetResult(anyResult *r, int row, int col, bool use_last = false);
 	virtual bool GetSize(int *width, int *height);
+	virtual bool isEmpty(int row, int col);
 	virtual bool Select(POINT *p){return false;};
 	virtual bool WriteData(char *FileName) {return false;};
 	virtual bool AddCols(int nCols){return false;};
@@ -543,7 +650,7 @@ public:
 	StrData(DataObj *par, RECT *rc = 0L);
 	~StrData();
 	bool GetSize(int *w, int *h);
-	void StrData::RestoreData(DataObj *dest);
+	void RestoreData(DataObj *dest);
 
 private:
 	int pw, ph;
@@ -630,6 +737,7 @@ public:
 	virtual bool FileIO(int rw) {return false;};
 	virtual void * ObjThere(int x, int y);
 	virtual void Track(POINT *p, anyOutput *o);
+	virtual double DefSize(int select);
 };
 
 class ssButton:public GraphObj {
@@ -782,7 +890,7 @@ private:
 class Bar:public GraphObj {
 public:
 	Bar(GraphObj *par, DataObj *d, double x, double y, int which, 
-		int xc = -1, int xr = -1, int yc = -1, int yr = -1);
+		int xc = -1, int xr = -1, int yc = -1, int yr = -1, char *desc = 0L);
 	Bar(int src);
 	~Bar();
 	double GetSize(int select);
@@ -818,8 +926,8 @@ public:
 	anyOutput *mo;
 	RECT mrc;
 
-	DataLine(GraphObj *par, DataObj *d, char *xrange=0L, char *yrange=0L);
-	DataLine(GraphObj *par, DataObj *d, lfPOINT *val, long nval);
+	DataLine(GraphObj *par, DataObj *d, char *xrange=0L, char *yrange=0L, char *name=0L);
+	DataLine(GraphObj *par, DataObj *d, lfPOINT *val, long nval, char *name);
 	DataLine(int src);
 	~DataLine();
 	bool SetColor(int select, DWORD col);
@@ -838,7 +946,7 @@ public:
 
 class DataPolygon:public DataLine{
 public:
-	DataPolygon(GraphObj *par, DataObj *d, char *xrange=0L, char *yrange=0L);
+	DataPolygon(GraphObj *par, DataObj *d, char *xrange=0L, char *yrange=0L, char *name=0L);
 	DataPolygon(int src);
 	~DataPolygon();
 	void DoPlot(anyOutput *target);
@@ -1335,6 +1443,51 @@ private:
 	Label **Lines;
 };
 
+class TextFrame:public GraphObj{
+	enum {TF_MAXLINE=120};
+public:
+	TextFrame(GraphObj *parent, DataObj *data, lfPOINT *p1, lfPOINT *p2, char *txt);
+	TextFrame(int src);
+	~TextFrame();
+	double GetSize(int select);
+	bool SetSize(int select, double value);
+	void DoMark(anyOutput *o, bool mark);
+	void DoPlot(anyOutput *o);
+	bool Command(int cmd, void *tmpl, anyOutput *o);
+	bool PropertyDlg();
+	bool FileIO(int rw);
+	void * ObjThere(int x, int y);
+	void Track(POINT *p, anyOutput *o);
+
+private:
+	fRECT pad;
+	RECT ipad, Cursor, *tm_rec;
+	fmtText fmt_txt;
+	int nlines, linc, tm_c, csize, cpos;
+	bool bModified, bResize, has_m1, has_m2;
+	unsigned char c_char, m1_char, m2_char;
+	double lspc;
+	lfPOINT pos1, pos2;
+	POINT cur_pos, m1_pos, m2_pos, m1_cpos, m2_cpos;
+	TextDEF TextDef;
+	dragRect *drc;
+	unsigned char *text, **lines;
+	LineDEF Line, FillLine;
+	FillDEF Fill;
+
+	void text2lines(anyOutput *o);
+	void lines2text();
+	void ShowCursor(anyOutput *o);
+	void AddChar(anyOutput *o, unsigned char c);
+	void DelChar(anyOutput *o);
+	void CalcCursorPos(int x, int y, anyOutput *o);
+	void ReplMark(anyOutput *o, char *ntext);
+	void procTokens();
+	bool DoPaste(anyOutput *o);
+	void TextMark(anyOutput *o, int mode);
+	bool CopyText(anyOutput *o, bool b_cut);
+};
+
 class segment:public GraphObj{
 public:
 	segment(GraphObj *par, DataObj *d, lfPOINT *c, double r1, double r2, double a1, double a2);
@@ -1399,12 +1552,13 @@ private:
 	void AddPoints(int n, lfPOINT *p);
 
 	void FitCurve(lfPOINT *d, int npt, double error);
+	void IpolBez(lfPOINT *d, lfPOINT tHat1, lfPOINT tHat2);
 	void FitCubic(lfPOINT *d, int first, int last, lfPOINT tHat1, lfPOINT tHat2, double error);
 	void RemovePoint(lfPOINT *d, int sel);
 	void GenerateBezier(lfPOINT *d, int first, int last, double * uPrime, lfPOINT tHat1, lfPOINT tHat2, lfPOINT *bezCurve);
 	double *Reparameterize(lfPOINT *d, int first, int last, double *u, lfPOINT *bezCurve);
 	lfPOINT fBezier(int degree, lfPOINT *V, double t);
-	double * ChordLengthParameterize(lfPOINT *d, int first, int last);
+	double *ChordLengthParameterize(lfPOINT *d, int first, int last);
 	double ComputeMaxError(lfPOINT *d, int first, int last, lfPOINT *bezCurve, double *u, int *splitPoint);
 };
 
@@ -1460,7 +1614,7 @@ class LegItem:public GraphObj{
 public:
 	DWORD flags;
 
-	LegItem(GraphObj *par, DataObj *d, LineDEF *ld, LineDEF *lf, FillDEF *fill);
+	LegItem(GraphObj *par, DataObj *d, LineDEF *ld, LineDEF *lf, FillDEF *fill, char *desc);
 	LegItem(GraphObj *par, DataObj *d, LineDEF *ld, Symbol *sy);
 	LegItem(GraphObj *par, DataObj *d, LineDEF *ld, int err, char *desc);
 	LegItem(int src);
@@ -1473,7 +1627,7 @@ public:
 	bool FileIO(int rw);
 	void Track(POINT *p, anyOutput *o);
 
-	bool HasFill(LineDEF *ld, FillDEF *fd);
+	bool HasFill(LineDEF *ld, FillDEF *fd, char *desc);
 	bool HasSym(LineDEF *ld, GraphObj *sy);
 	bool HasErr(LineDEF *ld, int err);
 
@@ -1501,8 +1655,8 @@ public:
 	bool FileIO(int rw);
 	void Track(POINT *p, anyOutput *o);
 
-	bool HasFill(LineDEF *ld, FillDEF *fd);
-	bool HasSym(LineDEF *ld, GraphObj *sy);
+	bool HasFill(LineDEF *ld, FillDEF *fd, char *desc);
+	bool HasSym(LineDEF *ld, GraphObj *sy, char *desc);
 	bool HasErr(LineDEF *ld, int err, char *desc);
 
 private:
@@ -1519,9 +1673,12 @@ class Plot:public GraphObj{
 public:
 	fRECT Bounds;					//contains minima and maxima for x and y
 	bool dirty;						//rescale before redraw;
-	int use_xaxis, use_yaxis;		//this plot uses its own axes
+	int use_xaxis, use_yaxis, use_zaxis;		//this plot uses its own axes
 	lfPOINT xBounds, yBounds, zBounds;	//like Bounds but in 3D space
 	int hidden;						//plot (layer) is not visible
+	char *x_info, *y_info, *z_info;	//descriptor used e.g. for axis label or legend
+	char *data_desc;				//descriptor for data, used for legend etc
+	TextValue *x_tv, *y_tv;			//TextValue object for ordinal axes
 
 	Plot(GraphObj *par, DataObj *d);
 	virtual double GetSize(int select);
@@ -1603,6 +1760,7 @@ public:
 class FreqDist:public Plot {
 public:
 	FreqDist(GraphObj *par, DataObj *d);
+	FreqDist(GraphObj *par, DataObj *d, char* range, bool bOnce);
 	FreqDist(int src);
 	~FreqDist();
 	void DoPlot(anyOutput *o);
@@ -1612,7 +1770,7 @@ public:
 	bool FileIO(int rw);
 
 private:
-	double start, step;
+	double dmin, dmax, start, step;
 	long nPlots;
 	char *ssRef;
 	LineDEF BarLine, HatchLine;
@@ -1696,7 +1854,7 @@ class BoxPlot:public Plot {
 public:
 	BoxPlot(GraphObj *par, DataObj *d);
 	BoxPlot(int src);
-	BoxPlot(GraphObj *par, DataObj *dt, int mode, int c1, int c2, int c3);
+	BoxPlot(GraphObj *par, DataObj *dt, int mode, int c1, int c2, int c3, char *box_name);
 	~BoxPlot();
 	double GetSize(int select);
 	bool SetSize(int select, double value);
@@ -1849,7 +2007,7 @@ private:
 class Ribbon:public Plot {
 public:
 	Ribbon(GraphObj *par, DataObj *d, double z, double width, char *xr, char *yr);
-	Ribbon(GraphObj *par, DataObj *d, char *xr, char *yr, char *zr);
+	Ribbon(GraphObj *par, DataObj *d, int which, char *xr, char *yr, char *zr);
 	Ribbon(GraphObj *par, DataObj *d, GraphObj **go, int ngo);
 	Ribbon(int src);
 	~Ribbon();
@@ -1939,7 +2097,7 @@ public:
 
 class Function:public Plot{
 public:
-	Function(GraphObj *par, DataObj *d);
+	Function(GraphObj *par, DataObj *d, char *desc);
 	Function(int src);
 	~Function();
 	bool SetSize(int select, double value);
@@ -1984,6 +2142,27 @@ private:
 	Function *dl;
 };
 
+class NormQuant:public Plot{
+public:
+	NormQuant(GraphObj *par, DataObj *d, char* range);
+	NormQuant(GraphObj *par, DataObj *d, double *data, int ndata);
+	NormQuant(int src);
+	~NormQuant();
+	void DoPlot(anyOutput *o);
+	bool Command(int cmd, void *tmpl, anyOutput *o);
+	bool PropertyDlg();
+	bool FileIO(int rw);
+
+private:
+	char *ssRef;
+	int nData, nValidData;
+	double *x_vals, *y_vals, *src_data;
+	Symbol *sy;
+
+	bool ProcessData();
+};
+
+	
 class GridLine:public GraphObj{
 public:
 	DWORD flags;
@@ -2059,6 +2238,7 @@ class Axis:public GraphObj{
 public:
 	AxisDEF *axis;
 	LineDEF axline;
+	TextValue *atv;
 
 	Axis(GraphObj *par, DataObj *d, AxisDEF *ax, DWORD flags);
 	Axis(int src);
@@ -2118,9 +2298,10 @@ class Plot3D:public Plot{
 public:
 	long nPlots, nAxes;
 	Axis **Axes;
-	GraphObj **plots;
+	GraphObj **plots, **Sc_Plots;
 	double *RotDef;
 	fPOINT3D cub1, cub2, rotC;
+	int nscp;
 
 	Plot3D(GraphObj *par, DataObj *d, DWORD flags);
 	Plot3D(int src);
@@ -2142,6 +2323,7 @@ public:
 	bool AcceptObj(GraphObj *go);
 	void SortObj();
 	bool Rotate(double dx, double dy, double dz, anyOutput *o, bool accept);
+	bool AddAxis();
 
 private:
 	long nObs, nmaxObs;
@@ -2244,14 +2426,17 @@ public:
 	double GetSize(int select);
 	bool SetSize(int select, double value);
 	DWORD GetColor(int select);
+	bool SetColor(int select, DWORD col);
 	virtual void DoPlot(anyOutput *o);
 	virtual bool Command(int cmd, void *tmpl, anyOutput *o);
 	bool PropertyDlg();
 	void RegGO(void *n);
 	virtual bool FileIO(int rw);
+	virtual double DefSize(int select);
 
 private:
 	int NumAxes, AxisTempl, tickstyle, zoom_level;
+	double scale;
 	RECT rcDim, rcUpd, rc_mrk;
 	DWORD ColDR, ColGR, ColGRL;
 	AxisDEF x_axis, y_axis;
@@ -2269,6 +2454,7 @@ private:
 	bool AddAxis();
 	bool MoveObj(int cmd, GraphObj *g); 
 	bool DoZoom(char *z);
+	bool DoScale(scaleINFO *sc, anyOutput *o);
 };
 
 class Page:public Graph{
@@ -2279,6 +2465,7 @@ public:
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 	void RegGO(void *n);
 	bool FileIO(int rw);
+	double DefSize(int select);
 
 private:
 	LineDEF LineDef;
@@ -2311,14 +2498,14 @@ class notary{
 public:
 	notary();
 	~notary();
-	unsigned long RegisterGO(GraphObj *go);
+	int RegisterGO(GraphObj *go);
 	void AddRegGO(GraphObj *go);
-	bool PushGO(unsigned long id, GraphObj *go);
-	GraphObj *PopGO(unsigned long id);
+	bool PushGO(unsigned int id, GraphObj *go);
+	GraphObj *PopGO(unsigned int id);
 	void FreeStack();
 
 private:
-	unsigned long NextPopGO, NextPushGO, NextRegGO;
+	unsigned int NextPopGO, NextPushGO, NextRegGO;
 	GraphObj ***gObs;
 	GraphObj ***goStack;
 };
@@ -2405,7 +2592,7 @@ class UndoObj {
 		UNDO_LFP, UNDO_POINT, UNDO_VOIDPTR, UNDO_MOVE, UNDO_RECT,
 		UNDO_STRING, UNDO_ROTDEF, UNDO_SETGO, UNDO_LINEDEF, UNDO_FILLDEF,
 		UNDO_AXISDEF, UNDO_LFP3D, UNDO_FLOAT, UNDO_MEM, UNDO_MUTATE, 
-		UNDO_DROPGOLIST, UNDO_TEXTDEF, UNDO_SAVVAR, UNDO_DATA, UNDO_ET};
+		UNDO_DROPGOLIST, UNDO_TEXTDEF, UNDO_SAVVAR, UNDO_DATA, UNDO_ET, UNDO_TEXTBUF};
 	typedef struct _UndoInfo {
 		int cmd;
 		DWORD flags;
@@ -2434,6 +2621,11 @@ class UndoObj {
 		int *cur, *m1, *m2, vcur, vm1, vm2, row, col;
 		}EtBuff;
 
+	typedef struct _TextBuff {
+		int *psize, size, *ppos, pos;
+		unsigned char **pbuff, *buff;
+		}TextBuff;
+
 public:
 	int *pcb;
 	anyOutput *cdisp, *ldisp;
@@ -2442,6 +2634,7 @@ public:
 	UndoObj();
 	~UndoObj();
 	void Flush();
+	bool isEmpty(anyOutput *o);
 	void SetDisp(anyOutput *o);
 	void KillDisp(anyOutput *o);
 	void InvalidGO(GraphObj *go);
@@ -2463,7 +2656,7 @@ public:
 	int SaveLFP(GraphObj *go, lfPOINT *lfp, DWORD flags);
 	void MoveObj(GraphObj *go, lfPOINT *lfp, DWORD flags);
 	void ValRect(GraphObj *go, fRECT *rec, DWORD flags);
-	void String(GraphObj *go, char **s, DWORD flags);
+	int String(GraphObj *go, char **s, DWORD flags);
 	void RotDef(GraphObj *go, double **d, DWORD flags);
 	void SetGO(GraphObj *parent, GraphObj **where, GraphObj *go, DWORD flags);
 	void Line(GraphObj *go, LineDEF *ld, DWORD flags);
@@ -2475,6 +2668,7 @@ public:
 	void DataMem(GraphObj *go, void **mem, int size, long *count, DWORD flags);
 	void DataObject(GraphObj *go, anyOutput *o, DataObj *d, RECT *rc, DWORD flags);
 	void TextCell(EditText *et, anyOutput *o, char *text, int *cur, int *m1, int *m2, void* DaO, DWORD flags);
+	void TextBuffer(GraphObj *parent, int *psize, int *ppos, unsigned char **pbuff, DWORD flags, anyOutput *o); 
 
 private:
 	UndoInfo **buff, **buff0;
@@ -2506,6 +2700,8 @@ void HideTextCursorObj(anyOutput *out);
 void ShowTextCursor(anyOutput *out, RECT *disp, DWORD color);
 void HideCopyMark();
 void ShowCopyMark(anyOutput *out, RECT *mrk, int nRec);
+void InvalidateOutput(anyOutput *o);
+void SuspendAnimation(anyOutput *o, bool bSusp);
 void InitTextCursor(bool init);
 void EmptyClip();
 void CopyText(char *txt, int len);
@@ -2515,6 +2711,7 @@ void FindBrowser();
 void LoopDlgWnd();
 void CloseDlgWnd(void *hDlg);
 void ShowDlgWnd(void *hDlg);
+void ResizeDlgWnd(void *hDlg, int w, int h);
 anyOutput *NewDispClass(GraphObj *g);
 bool DelDispClass(anyOutput *w);
 anyOutput *NewBitmapClass(int w, int h, double hr, double vr);
@@ -2558,6 +2755,11 @@ double GetAxisFac(AxisDEF *axis, double delta, int direc);
 char *str_ltrim(char *str);
 char *str_rtrim(char *str);
 char *str_trim(char *str);
+void rmquot(char *str);
+int strpos(char *needle, char *haystack);
+char *strreplace(char *needle, char *replace, char *haystack);
+char *substr(char *text, int pos1, int pos2);
+int rlp_strcpy(char*dest, int size, char*src);
 void ReshapeFormula(char **text);
 void TranslateResult(anyResult *res);
 void CleanTags(char *txt, int *i1, int *i2, int *i3);
@@ -2569,9 +2771,15 @@ bool Txt2Flt(char *txt, double *val);
 void RmTrail(char *txt);
 double NiceValue(double fv);
 char *Int2ColLabel(int nr, bool uc);
-char *str2xml(char *str);
+char *mkCellRef(int row, int col);
+char *mkRangeRef(int r1, int c1, int r2, int c2);
+char *str2xml(char *str, bool bGreek);
 char **split(char *str, char sep, int *nl);
 char *fit_num_rect(anyOutput *o, int max_width, char *num_str);
+void add_to_buff(char** dest, int *pos, int *csize, char *txt, int len);
+void add_int_to_buff(char** dest, int *pos, int *csize, int value, bool lsp, int ndig);
+void add_dbl_to_buff(char** dest, int *pos, int *csize, double value, bool lsp);
+void add_hex_to_buff(char** dest, int *pos, int *csize, DWORD value, bool lsp);
 void SetMinMaxRect(RECT *rc, int x1, int y1, int x2, int y2);
 void UpdateMinMaxRect(RECT *rc, int x, int y);
 void IncrementMinMaxRect(RECT *rc, int i);
@@ -2582,6 +2790,7 @@ bool IsInPolygon(POINT *p, POINT *pts, int cp);
 bool OverlapRect(RECT *rc1, RECT *rc2);
 void AddToPolygon(long *cp, POINT *pts, POINT *np);
 void DrawBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, int depth=0);
+int mkCurve(lfPOINT *src, int n1, lfPOINT **dst, bool bClosed);
 POINT *MakeArc(int ix, int iy, int r, int qad, long *npts);
 void InvertPolygon(POINT*, int, LineDEF*, FillDEF*, RECT*, anyOutput*, bool);
 void InvertLine(POINT*, int, LineDEF*, RECT*, anyOutput*, bool);
@@ -2612,6 +2821,7 @@ DWORD CheckNewFloat(double *loc, double old_v, double new_v, GraphObj *par, DWOR
 DWORD CheckNewInt(int *loc, int old_v, int new_v, GraphObj *par, DWORD flags);
 DWORD CheckNewDword(DWORD *loc, DWORD old_v, DWORD new_v, GraphObj *par, DWORD flags);
 DWORD CheckNewLFPoint(lfPOINT *loc, lfPOINT *old_v, lfPOINT *new_v, GraphObj *par, DWORD flags);
+DWORD CheckNewString(char **loc, char *s_old, char *s_new, GraphObj *par, DWORD flags);
 void clip_line_sphere(GraphObj *par, POINT3D **li, int r, int cx, int cy, int cz);
 void clip_line_plane(GraphObj *par, POINT3D **li, POINT3D *pg, int np, double *vec);
 void clip_sphline_sphere(GraphObj *par, POINT3D *lim1, POINT3D *lim2, POINT3D *cent, 
@@ -2630,11 +2840,12 @@ void DoExportEps(GraphObj *g, char *FileName, DWORD flags);
 void DoExportTif(GraphObj *g, char *FileName, DWORD flags);
 
 //prototypes mfcalc.cpp
+char  *yywarn(char *txt, bool bNew);
 bool do_xyfunc(DataObj *, double, double, double, char *, lfPOINT **, long *, char *);
 bool do_func3D(DataObj *d, double x1, double x2, double xstep, double z1, double z2, double zstep, 
 	char *expr, char *param);
 anyResult *do_formula(DataObj *, char *);
-bool MoveFormula(DataObj *d, char *of, char *nf, int dx, int dy, int r0, int c0);
+bool MoveFormula(DataObj *d, char *of, char *nf, int nfsize, int dx, int dy, int r0, int c0);
 int do_fitfunc(DataObj *d, char *rx, char *ry, char *rz, char **par, char *expr, 
 	double conv, int maxiter, double *chi_2);
 
@@ -2645,6 +2856,7 @@ bool mrqmin(double *, double *, double *, int, double **, int, int *, int, doubl
 	void (*funcs)(double, double, double **, double *, double *, int), double *);
 bool Check_MRQerror();
 void SortArray(int n, double *vals);
+void SortArray2(int n, double *a1, double *a2);
 void SortFpArray(int n, lfPOINT *vals);
 void spline(lfPOINT *v, int n, double *y2);
 double gammln(double x);
@@ -2665,9 +2877,17 @@ double exp_freq(double x, double l, double s);
 double lognorm_dist(double x, double m, double s);
 double lognorm_freq(double x, double m, double s);
 double chi_dist(double x, double df, double);
+double chi_freq(double x, double df);
 double t_dist(double t, double df, double);
+double t_freq(double t, double df);
 double pois_dist(double x, double m, double);
 double f_dist(double f, double df1, double df2);
+double f_freq(double f, double df1, double df2);
+double ks_dist(int n, double d);
+void swilk1(int n, double *v0, double (*func)(double, double, double), double p1, double p2, 
+	bool bsorted, double *w, double *p);
+void KolSmir(int n, double *v0, double (*func)(double, double, double),
+	double p1, double p2, bool bsorted, double *d, double *p);
 double distinv(double (*sf)(double, double, double), double df1, double df2, double p, double x0);
 void d_quartile(int n, double *v, double *q1, double *q2, double *q3);
 double d_variance(int n, double *v, double *mean = 0L, double *ss = 0L);
@@ -2677,23 +2897,33 @@ double d_skew(int n, double *v);
 double d_gmean(int n, double *v);
 double d_hmean(int n, double *v);
 double d_classes(DataObj *d, double start, double step, double *v, int nv, char *range);
-double d_pearson(double *x, double *y, int n, char *dest, DataObj *data);
+double d_pearson(double *x, double *y, int n, char *dest, DataObj *data, double *ra);
 double d_rank(int n, double *v, double v1);
 void crank(int n, double *w0, double *s);
-double d_spearman(double *x, double *y, int n, char *dest, DataObj *data);
-double d_kendall(double *x, double *y, int n, char *dest, DataObj *data);
+double d_spearman(double *x, double *y, int n, char *dest, DataObj *data, double *ra);
+double d_kendall(double *x, double *y, int n, char *dest, DataObj *data, double *ra);
 double d_regression(double *x, double *y, int n, char *dest, DataObj *data);
 double d_covar(double *x, double *y, int n, char *dest, DataObj *data);
-double d_utest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
-double d_ttest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
+double d_utest(double *x, double *y, int n1, int n2, char *dest, DataObj *data, double *results);
+double d_ttest(double *x, double *y, int n1, int n2, char *dest, DataObj *data, double *results);
 double d_ttest2(double *x, double *y, int n, char *dest, DataObj *data);
 double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
+double wprob(double w, double rr, double cc);
+double ptukey(double q, double rr, double cc, double df, int lower_tail, int log_p);
+double qtukey(double p, double rr, double cc, double df, int lower_tail, int log_p);
 bool date_value(char *desc, char *fmt, double *value);
 char *value_date(double dv, char *fmt);
 double now_today();
 void split_date(double dv, int *y, int *mo, int *dom, int *dow, int *doy, int *h, int *m, double *s);
+Triangle* Triangulate1(char *xr, char *yr, char *zr, DataObj *data);
 Ribbon *SurfTria(GraphObj *parent, DataObj *data, char *text1, char *text2, char *text3);
 
 //prototypes reports.cpp
 void rep_anova(GraphObj *parent, DataObj *data);
+void rep_kruskal(GraphObj *parent, DataObj *data);
+void rep_samplestats(GraphObj *parent, DataObj *data);
 void rep_regression(GraphObj *parent, DataObj *data);
+void rep_twowaytable(GraphObj *parent, DataObj *data);
+void rep_compmeans(GraphObj *parent, DataObj *data);
+void rep_correl(GraphObj *parent, DataObj *data, int style);
+
