@@ -112,7 +112,7 @@ ssButton::Command(int cmd, void *tmpl, anyOutput *o)
 	switch (cmd) {
 	case CMD_GETTEXT:
 		if(TextDef.text && tmpl) {
-			strcpy((char*)tmpl, TextDef.text);
+			rlp_strcpy((char*)tmpl, 10, TextDef.text);
 			return true;
 			}
 		return false;
@@ -132,9 +132,10 @@ ssButton::Command(int cmd, void *tmpl, anyOutput *o)
 			}
 		return true;
 	case CMD_SETTEXT:
-		if(TextDef.text) free(TextDef.text);
-		if(tmpl) TextDef.text = strdup((char*)tmpl);
-		else TextDef.text = 0L;
+		if(tmpl) {
+			if(! TextDef.text) TextDef.text = (char*)malloc(10*sizeof(char));
+			rlp_strcpy(TextDef.text, 10, (char*)tmpl);
+			}
 		return true;
 	case CMD_GETTEXTDEF:
 		if(!tmpl) return false;
@@ -961,21 +962,41 @@ Label::AddChar(int ci, anyOutput *o)
 		if(!parent->Command(CMD_MUTATE, golist, o)) DeleteGO(golist[1]);
 		return false;
 		}
-	if(ci < 254 && ci > 31) c = (char)ci;
+	if(TextDef.text && TextDef.text[0]) i = (int)strlen(TextDef.text);
+	else i = 0;		if(CursorPos > i)CursorPos = i;
+	if(ci > 254) {
+		Undo.String(this, &TextDef.text, 0L);
+		Undo.ValInt(this, &CursorPos, UNDO_CONTINUE);
+		txt1 = (char*)malloc((i+12)*sizeof(char));
+		if(txt1) {
+			if(j=CursorPos) k = rlp_strcpy(txt1, CursorPos+1, TextDef.text);
+			else k = 0;
+#ifdef USE_WIN_SECURE
+			k += sprintf_s(txt1+k, i+12-k, "&#%d;",ci);
+#else
+			k += sprintf(txt1+k, "&#%d;",ci);
+#endif
+			CursorPos = k;
+			k += rlp_strcpy(txt1+k, i+12-k, TextDef.text+j);
+			if(TextDef.text) free(TextDef.text);
+			TextDef.text = txt1;			RedrawEdit(o);
+			}
+		return true;
+		}
+	else if( ci > 31) c = (char)ci;
 	else return false;
 	if(!TextDef.text || CursorPos < 10 || c == ' ') {
 		Undo.String(this, &TextDef.text, 0L);
 		Undo.ValInt(this, &CursorPos, UNDO_CONTINUE);
 		}
-	if(TextDef.text) txt1 = (char*)calloc((i = strlen(TextDef.text))+2, sizeof(char));
-	else txt1 = (char*)calloc((i = 0)+2, sizeof(char));
+	txt1 = (char*)malloc((i+4)* sizeof(char));
 	if(txt1) {
-		for(j = k = 0; j< i && j < CursorPos; txt1[k++] = TextDef.text[j++]);
-		txt1[k++] = c;
-		for(; j< i; txt1[k++] = TextDef.text[j++]);
+		if(j=CursorPos) k = rlp_strcpy(txt1, CursorPos+1, TextDef.text);
+		else k = 0;			txt1[k++] = c;
+		k += rlp_strcpy(txt1+k, i+12-k, TextDef.text+j);
 		if(TextDef.text) free(TextDef.text);
-		TextDef.text = txt1;
-		CursorPos++;		RedrawEdit(o);
+		TextDef.text = txt1;	txt1[k] = 0;
+		CursorPos++;			RedrawEdit(o);
 		}
 	return true;
 }
@@ -1005,7 +1026,98 @@ Label::CalcCursorPos(int x, int y, anyOutput *o)
 		x1 = x2;
 		}
 	if(pts[3].x < pts[2].x && x < pts[3].x) CursorPos = 0;
-	else CursorPos = strlen(TextDef.text);
+	else CursorPos = (int)strlen(TextDef.text);
+}
+
+void 
+TextFrame::ShowCursor(anyOutput *o)
+{
+	int cx, cy, w, h;
+
+	if(!o) return;
+	TextDef.iSize = o->un2iy(TextDef.fSize);
+#ifdef _WINDOWS
+	linc = o->un2iy(TextDef.fSize*lspc);
+#else
+	linc = o->un2iy(TextDef.fSize*lspc*1.2);
+#endif
+	o->SetTextSpec(&TextDef);
+	cy = rDims.top + linc * cur_pos.y;			cx = rDims.left;
+	cx += ipad.left;							cy += ipad.top;
+	if(cur_pos.y < nlines) {
+		if(lines[cur_pos.y] && lines[cur_pos.y][0] && cur_pos.x) {
+			fmt_txt.SetText(0L, (char*)lines[cur_pos.y], &cx, &cy);
+			if(!fmt_txt.oGetTextExtent(o, &w, &h, cur_pos.x))return;
+			}
+		else {
+			if(!o->oGetTextExtent("A", 1, &w, &h))return;	w = 0;
+			}
+		Cursor.left = cx+w;		Cursor.right = cx+w;
+		Cursor.top = cy;		Cursor.bottom = cy+linc;
+		ShowTextCursor(o, &Cursor, TextDef.ColTxt);
+		}
+}
+
+void
+TextFrame::CalcCursorPos(int x, int y, anyOutput *o)
+{
+	int i, iy, ix, x1, x2, h;
+
+	if(!o) return;
+	TextDef.iSize = o->un2iy(TextDef.fSize);
+	o->SetTextSpec(&TextDef);
+	iy = (y-rDims.top-ipad.top)/linc;		x1 = x2 = 0;
+	if(iy >= nlines) iy = nlines-1;			cur_pos.y = iy;
+	x2 = rDims.right - rDims.left -ipad.left - ipad.right;
+	x = x - rDims.left -ipad.left;
+	fmt_txt.SetText(0L, (char*)lines[iy], &x1, &x2);
+	for(i = 0,  ix = x1 = 0; lines[iy][i]; i++) {
+		if(i) fmt_txt.oGetTextExtent(o, &x1, &h, i);
+		x1 -= (TextDef.iSize >> 2);
+		if(i && x1 <= x2 && x1 >= x) {
+			if(i >1) fmt_txt.oGetTextExtent(o, &x1, &h, i-1);
+			else x1 = 0;
+			fmt_txt.oGetTextExtent(o, &x2, &h, i);
+			ix = (abs(x1-x)<abs(x2-x)) ? i = (i-1) : i;
+			break;
+			}
+		}
+	if(ix) cur_pos.x = ix;
+	else cur_pos.x = i;
+}
+
+void
+Axis::DoMark(anyOutput *o, bool mark)
+{
+	LineDEF mrkLine;
+	double minw = DefSize(SIZE_DATA_LINE);
+	POINT *arc;
+	long narc = 0;
+
+	if(axis->flags & AXIS_ANGULAR) {
+		if(mark) {
+			o->GetSize(&mrc);						mo = GetRectBitmap(&mrc, o);
+			memcpy(&mrkLine, &axline, sizeof(LineDEF));
+			mrkLine.width = axline.width < minw ? minw * 3.0 : axline.width *3.0;
+			o->SetLine(&mrkLine);
+			if(arc = MakeArc(pts[0].x, pts[0].y, pts[1].x, 0x0f, &narc)) {
+				o->oPolyline(arc, (int)narc, 0L);
+				mrkLine.color ^= 0x00ffffffL;		mrkLine.width = axline.width;
+				o->SetLine(&mrkLine);				o->oPolyline(arc, (int)narc, 0L);
+				free(arc);
+				}
+			o->UpdateRect(&mrc, false);
+			}
+		else RestoreRectBitmap(&mo, &mrc, o);
+		return;
+		}
+	if(mark){
+		memcpy(&mrc, &rDims, sizeof(RECT));
+		IncrementMinMaxRect(&mrc, 6 + o->un2ix(axline.width));
+		mo = GetRectBitmap(&mrc, o);
+		InvertLine(pts, 2, &axline, &rDims, o, mark);
+		}
+	else RestoreRectBitmap(&mo, &mrc, o);
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1015,6 +1127,7 @@ Graph::ExecTool(MouseEvent *mev)
 {
 	static POINT pl = {0, 0}, pc = {0, 0};
 	static DWORD color = 0L;
+	scaleINFO scale;
 	POINT line[5];
 	GraphObj **tmpPlots, *new_go;
 	TextDEF *td;
@@ -1023,7 +1136,7 @@ Graph::ExecTool(MouseEvent *mev)
 	double x, y;
 
 	if(!mev || !CurrDisp) return false;
-	td = 0L;
+	td = 0L;	if(mev->Action == MOUSE_LBUP) SuspendAnimation(CurrDisp, false);
 	if(ToolMode & TM_MOVE) switch (mev->Action) {
 		case MOUSE_LBDOWN:
 			pl.x = pc.x = mev->x;				pl.y = pc.y = mev->y;
@@ -1102,7 +1215,68 @@ Graph::ExecTool(MouseEvent *mev)
 		}
 	if(ToolMode == TM_PASTE) {
 		switch (mev->Action) {
+		case MOUSE_LBDOWN:
+			CurrGO = 0L;
+			CurrDisp->MrkMode = MRK_NONE;
+			pl.x = pc.x = mev->x;		pl.y = pc.y = mev->y;
+			rcDim.left = rcDim.right = rcUpd.left = rcUpd.right = mev->x;
+			rcDim.top = rcDim.bottom = rcUpd.top = rcUpd.bottom = mev->y;
+			return true;
+		case MOUSE_MOVE:
+			if(mev->StateFlags &1) {
+				if(rcUpd.left != rcUpd.right && rcUpd.top != rcUpd.bottom)
+					CurrDisp->UpdateRect(&rcUpd, false);
+				line[0].x = line[4].x = pl.x;	line[0].y = line[4].y = pl.y;
+				line[1].x = pc.x = mev->x;		line[1].y = pl.y;
+				line[2].x = mev->x;				line[2].y = pc.y = mev->y;
+				line[3].x = pl.x;				line[3].y = mev->y;
+				CurrDisp->ShowLine(line, 5, 0x00c0c0c0L);
+				memcpy(&rcUpd, &rcDim, sizeof(RECT));
+				UpdateMinMaxRect(&rcUpd, mev->x, mev->y);
+				IncrementMinMaxRect(&rcUpd, 2);
+				return true;
+				}
+			break;
 		case MOUSE_LBUP:
+			if(!PasteObj) return false;
+			pc.x = mev->x;			pc.y = mev->y;
+			if((lfp = (lfPOINT*)malloc(2 * sizeof(lfPOINT)))){
+				lfp[0].fx = CurrDisp->fix2un(pl.x - CurrDisp->VPorg.fx);
+				lfp[0].fy = CurrDisp->fiy2un(pl.y - CurrDisp->VPorg.fy);
+				lfp[1].fx = CurrDisp->fix2un(pc.x - CurrDisp->VPorg.fx);
+				lfp[1].fy = CurrDisp->fiy2un(pc.y - CurrDisp->VPorg.fy);
+				if(lfp[0].fx > lfp[1].fx) {
+					x = lfp[0].fx;		lfp[0].fx = lfp[1].fx;		lfp[1].fx = x;
+					}
+				if(lfp[0].fy > lfp[1].fy) {
+					y = lfp[0].fy;		lfp[0].fy = lfp[1].fy;		lfp[1].fy = y;
+					}
+				}
+			else {
+				DeleteGO(PasteObj);			PasteObj = 0L;
+				ToolMode = TM_STANDARD;		CurrDisp->MouseCursor(MC_ARROW, false);
+				return true;
+				}
+			scale.sx.fx = lfp[0].fx;	scale.sy.fx = lfp[0].fy;	scale.sz.fx = 0.0;
+			scale.sx.fy = scale.sy.fy = scale.sz.fy = 1.0;
+			if(PasteObj->Id == GO_GRAPH) {
+				if(abs(pc.x -pl.x) > 20 && abs(pc.y -pl.y) >20) {
+					x = ((Graph*)PasteObj)->GRect.Xmax - ((Graph*)PasteObj)->GRect.Xmin;
+					y = ((Graph*)PasteObj)->GRect.Ymax - ((Graph*)PasteObj)->GRect.Ymin;
+					scale.sx.fy = (lfp[1].fx - lfp[0].fx)/x;
+					scale.sy.fy = (lfp[1].fy - lfp[0].fy)/y;
+					//preserve aspect ratio
+					scale.sx.fy = (scale.sx.fy + scale.sy.fy) /2.0;
+					scale.sy.fy = scale.sx.fy;
+					}
+				((Graph*)PasteObj)->GRect.Xmax -= ((Graph*)PasteObj)->GRect.Xmin;
+				((Graph*)PasteObj)->GRect.Ymax -= ((Graph*)PasteObj)->GRect.Ymin;
+				((Graph*)PasteObj)->GRect.Ymin = ((Graph*)PasteObj)->GRect.Xmin = 0.0;
+				PasteObj->Command(CMD_SCALE, &scale, 0L);
+				Command(CMD_DROP_GRAPH, (void*)PasteObj, 0L);
+				}
+			else DeleteGO(PasteObj);
+			PasteObj = 0L;
 			ToolMode = TM_STANDARD;			CurrDisp->MouseCursor(MC_ARROW, false);
 			break;
 			}
@@ -1319,27 +1493,84 @@ Graph::ExecTool(MouseEvent *mev)
 		if(Plots[NumPlots]) i = NumPlots+1;
 		else i = NumPlots;
 		switch(mev->Action) {
-		case MOUSE_LBUP:
-			if(!(td = (TextDEF *)calloc(1, sizeof(TextDEF))))return false;
-			x = CurrDisp->fix2un(mev->x-CurrDisp->VPorg.fx);
-			y = CurrDisp->fiy2un(mev->y-CurrDisp->VPorg.fy);
-			td->ColTxt = defs.Color(COL_TEXT);		td->ColBg = defs.Color(COL_BG);
-			td->RotBL = td->RotCHAR = 0.0f;			td->fSize = defs.GetSize(SIZE_TEXT);
-			td->Align = TXA_VTOP | TXA_HLEFT;		td->Style = TXS_NORMAL;
-			td->Mode = TXM_TRANSPARENT;				td->Font = FONT_HELVETICA;
-			td->text = 0L;
+		case MOUSE_LBDOWN:
 			CurrGO = 0L;
 			CurrDisp->MrkMode = MRK_NONE;
 			Command(CMD_REDRAW, 0L, CurrDisp);
-			y -= td->fSize/2.0;
-			Undo.SetGO(this, &Plots[i], new Label(this, data, x, y, td, 0L), 0L);
-			if(Plots[i]){
-				NumPlots = i+1;
-				Plots[i]->DoPlot(CurrDisp);							//init
-				CurrDisp->ShowMark(CurrGO = Plots[i], MRK_GODRAW);	//edit
-				Plots[i]->moveable = 1;
+			color = 0x00cbcbcb;
+			pl.x = pc.x = mev->x;		pl.y = pc.y = mev->y;
+			rcDim.left = rcDim.right = rcUpd.left = rcUpd.right = mev->x;
+			rcDim.top = rcDim.bottom = rcUpd.top = rcUpd.bottom = mev->y;
+			return true;
+		case MOUSE_MOVE:
+			if(mev->StateFlags &1) {
+				CurrDisp->MouseCursor(MC_TXTFRM, false);
+				if(rcUpd.left != rcUpd.right && rcUpd.top != rcUpd.bottom)
+					CurrDisp->UpdateRect(&rcUpd, false);
+				line[0].x = line[4].x = pl.x;	line[0].y = line[4].y = pl.y;
+				if((ToolMode & 0x0f)==TM_ARROW) {
+					line[1].x = pc.x = mev->x;		line[1].y = pc.y = mev->y;
+					CurrDisp->ShowLine(line, 2, color);
+					}
+				else {
+					line[1].x = pc.x = mev->x;		line[1].y = pl.y;
+					line[2].x = mev->x;				line[2].y = pc.y = mev->y;
+					line[3].x = pl.x;				line[3].y = mev->y;
+					CurrDisp->ShowLine(line, 5, color);
+					}
+				memcpy(&rcUpd, &rcDim, sizeof(RECT));
+				UpdateMinMaxRect(&rcUpd, mev->x, mev->y);
+				IncrementMinMaxRect(&rcUpd, 2);
+				return true;
+				}
+			break;
+		case MOUSE_LBUP:
+			pc.x = mev->x;			pc.y = mev->y;
+			if(((abs(pc.x-pl.x) >20 && abs(pc.y-pl.y) >20)) && 
+				(lfp = (lfPOINT*)malloc(2 * sizeof(lfPOINT)))){
+				lfp[0].fx = CurrDisp->fix2un(pl.x - CurrDisp->VPorg.fx);
+				lfp[0].fy = CurrDisp->fiy2un(pl.y - CurrDisp->VPorg.fy);
+				lfp[1].fx = CurrDisp->fix2un(pc.x - CurrDisp->VPorg.fx);
+				lfp[1].fy = CurrDisp->fiy2un(pc.y - CurrDisp->VPorg.fy);
+				if(Plots[NumPlots]) i = NumPlots+1;
+				else i = NumPlots;
+				new_go = new TextFrame(this, data, &lfp[0], &lfp[1], 0L);
+				if(new_go) Undo.SetGO(this, &Plots[i], new_go, 0L);
+				if(Plots[i]){
+					NumPlots = i+1;
+					Plots[i]->DoPlot(CurrDisp);							//init
+					CurrDisp->ShowMark(CurrGO = Plots[i], MRK_GODRAW);				//edit
+					Plots[i]->moveable = 1;
+					bModified = true;
+					}
+				free(lfp);
+				CurrDisp->CheckMenu(ToolMode & 0x0f, false);
+				CurrDisp->CheckMenu(ToolMode = TM_STANDARD, true);
+				CurrDisp->MouseCursor(MC_ARROW, false);
+				}
+			else {
+				CurrDisp->MouseCursor(MC_TEXT, false);
+				if(!(td = (TextDEF *)calloc(1, sizeof(TextDEF))))return false;
+				x = CurrDisp->fix2un(mev->x-CurrDisp->VPorg.fx);
+				y = CurrDisp->fiy2un(mev->y-CurrDisp->VPorg.fy);
+				td->ColTxt = defs.Color(COL_TEXT);		td->ColBg = defs.Color(COL_BG);
+				td->RotBL = td->RotCHAR = 0.0f;			td->fSize = DefSize(SIZE_TEXT);
+				td->Align = TXA_VTOP | TXA_HLEFT;		td->Style = TXS_NORMAL;
+				td->Mode = TXM_TRANSPARENT;				td->Font = FONT_HELVETICA;
+				td->text = 0L;
+				CurrGO = 0L;
+				CurrDisp->MrkMode = MRK_NONE;
+				Command(CMD_REDRAW, 0L, CurrDisp);
+				y -= td->fSize/2.0;
+				Undo.SetGO(this, &Plots[i], new Label(this, data, x, y, td, 0L), 0L);
+				if(Plots[i]){
+					NumPlots = i+1;
+					Plots[i]->DoPlot(CurrDisp);							//init
+					CurrDisp->ShowMark(CurrGO = Plots[i], MRK_GODRAW);	//edit
+					Plots[i]->moveable = 1;
+					}
+				free(td);
 				}
-			free(td);
 			return true;
 			}
 		break;
@@ -1405,6 +1636,7 @@ Graph::DoZoom(char *z)
 	Graph *cg;
 
 	if(!z) return false;
+	HideCopyMark();
 	if(0==strcmp("fit", z)) {
 		if(CurrGraph) cg = CurrGraph;
 		else cg = this;
@@ -1414,7 +1646,7 @@ Graph::DoZoom(char *z)
 		rc_mrk.top = CurrDisp->un2ix(cg->GetSize(SIZE_GRECT_TOP))-4 
 			- iround(CurrDisp->MenuHeight);
 		rc_mrk.bottom = CurrDisp->un2ix(cg->GetSize(SIZE_GRECT_BOTTOM))+4;
-		CurrDisp->ActualSize(&cw);
+		if(!(CurrDisp->ActualSize(&cw)))return false;
 		f1 = (double)(cw.bottom - cw.top)/(double)(rc_mrk.bottom - rc_mrk.top);
 		f2 = ((double)(cw.right - cw.left)/(double)(rc_mrk.right - rc_mrk.left));
 		fac = f1 < f2 ? f1 : f2;
@@ -1447,7 +1679,7 @@ Graph::DoZoom(char *z)
 				ToolMode = TM_STANDARD;		Command(CMD_TOOLMODE, &ToolMode, CurrDisp);
 				return false;
 				}
-			CurrDisp->ActualSize(&cw);
+			if(!(CurrDisp->ActualSize(&cw)))return false;
 			fac = (double)(cw.bottom - cw.top)/(double)(rc_mrk.bottom - rc_mrk.top);
 			fac += ((double)(cw.right - cw.left)/(double)(rc_mrk.right - rc_mrk.left));
 			fac /= 2.0;
@@ -1560,10 +1792,11 @@ ObjTree::DoPlot(anyOutput *o)
 		if(list[i]) {
 			curr_obj = list[i];			n = 0;
 			if(i) while(curr_obj && curr_obj != list[0]) {
-				if(curr_obj != list[0]) n += sprintf(TmpTxt + n, " -");
+				if(curr_obj != list[0]) n += rlp_strcpy(TmpTxt+n, TMP_TXT_SIZE -n, " -");
 				if(curr_obj) curr_obj = curr_obj->parent; 
 				}
-			sprintf(TmpTxt + n, "%s%s", n ? " " : "", get_name(i));
+			if(n) TmpTxt[n++] = ' ';
+			n += rlp_strcpy(TmpTxt+n, TMP_TXT_SIZE -n, get_name(i));
 			if(list[i]->Id >= GO_PLOT && list[i]->Id < GO_GRAPH) {
 				TextDef.ColTxt = ((Plot*)list[i])->hidden ? 0x00000080L : 0x00008000L;
 				}
@@ -1615,7 +1848,7 @@ ObjTree::CreateBitmap(int *bw, int *bh, anyOutput *tmpl)
 	anyOutput *bmp = 0L;
 	int h;
 	
-	h = tmpl->un2iy(TextDef.fSize) * count;
+	h = (tmpl->un2iy(TextDef.fSize)+_SBINC) * (count+1);
 	if(h > *bh) *bh = h;
 	if(bmp = NewBitmapClass(*bw, *bh, tmpl->hres, tmpl->vres)) DoPlot(bmp);
 	return bmp;
